@@ -26,7 +26,7 @@ var _ = cmd(catUtils, func() *cli.Command {
 			args.CredName = ctx.Args().Get(0)
 			arg1 := ctx.Args().Get(1)
 			args.ProviderName = arg1
-			// In v4.0, skip the first args.ZoneNames if it it equals "-".
+			// In v4.0, skip the first args.ZoneNames if it equals "-".
 			args.ZoneNames = ctx.Args().Slice()[2:]
 
 			if arg1 != "" && arg1 != "-" {
@@ -212,7 +212,8 @@ func GetZone(args GetZoneArgs) error {
 	dspVariableName := "DSP_" + strings.ToUpper(args.CredName)
 
 	if args.OutputFormat == "js" || args.OutputFormat == "djs" {
-
+		fmt.Fprintf(w, "// generated by get-zones. This is 'a decent first draft' and requires editing.\n")
+		fmt.Fprintf(w, "\n")
 		if args.ProviderName == "-" {
 			fmt.Fprintf(w, `var %s = NewDnsProvider("%s");`+"\n",
 				dspVariableName, args.CredName)
@@ -229,10 +230,11 @@ func GetZone(args GetZoneArgs) error {
 
 		z := prettyzone.PrettySort(recs, zoneName, 0, nil)
 		switch args.OutputFormat {
-
 		case "zone":
 			fmt.Fprintf(w, "$ORIGIN %s.\n", zoneName)
-			prettyzone.WriteZoneFileRC(w, z.Records, zoneName, uint32(args.DefaultTTL), nil)
+			if err := prettyzone.WriteZoneFileRC(w, z.Records, zoneName, uint32(args.DefaultTTL), nil); err != nil {
+				return err
+			}
 			fmt.Fprintln(w)
 
 		case "js", "djs":
@@ -240,6 +242,7 @@ func GetZone(args GetZoneArgs) error {
 			if args.OutputFormat == "djs" {
 				sep = "\n\t, " // Funky comma mode
 			}
+
 			fmt.Fprintf(w, `D("%s", REG_CHANGEME%s`, zoneName, sep)
 			var o []string
 			o = append(o, fmt.Sprintf("DnsProvider(%s)", dspVariableName))
@@ -276,12 +279,11 @@ func GetZone(args GetZoneArgs) error {
 					"// NOTE: CNAME at apex may require manual editing.",
 				)
 				fmt.Fprint(w, out)
-				fmt.Fprint(w, "\nEND);\n\n")
+				fmt.Fprint(w, "\n);\n\n")
 			}
 
 		case "tsv":
 			for _, rec := range recs {
-
 				cfproxy := ""
 				if cp, ok := rec.Metadata["cloudflare_proxy"]; ok {
 					if cp == "true" {
@@ -315,7 +317,6 @@ func jsonQuoted(i string) string {
 }
 
 func formatDsl(rec *models.RecordConfig, defaultTTL uint32) string {
-
 	target := rec.GetTargetCombined()
 
 	ttl := uint32(0)
@@ -350,11 +351,13 @@ func formatDsl(rec *models.RecordConfig, defaultTTL uint32) string {
 			jsonQuoted(rec.NaptrRegexp),      // regex
 			jsonQuoted(rec.GetTargetField()), // .
 		)
+	case "SMIMEA":
+		target = fmt.Sprintf(`%d, %d, %d, "%s"`, rec.SmimeaUsage, rec.SmimeaSelector, rec.SmimeaMatchingType, rec.GetTargetField())
 	case "SSHFP":
 		target = fmt.Sprintf(`%d, %d, "%s"`, rec.SshfpAlgorithm, rec.SshfpFingerprint, rec.GetTargetField())
 	case "SOA":
 		rec.Type = "//SOA"
-		target = fmt.Sprintf(`"%s", "%s", %d, %d, %d, %d, %d`, rec.GetTargetField(), rec.SoaMbox, rec.SoaSerial, rec.SoaRefresh, rec.SoaRetry, rec.SoaExpire, rec.SoaMinttl)
+		target = fmt.Sprintf(`"%s", "%s", %d, %d, %d, %d`, rec.GetTargetField(), rec.SoaMbox, rec.SoaRefresh, rec.SoaRetry, rec.SoaExpire, rec.SoaMinttl)
 	case "SRV":
 		target = fmt.Sprintf(`%d, %d, %d, "%s"`, rec.SrvPriority, rec.SrvWeight, rec.SrvPort, rec.GetTargetField())
 	case "SVCB", "HTTPS":
@@ -364,6 +367,8 @@ func formatDsl(rec *models.RecordConfig, defaultTTL uint32) string {
 	case "TXT":
 		target = jsonQuoted(rec.GetTargetTXTJoined())
 		// TODO(tlim): If this is an SPF record, generate a SPF_BUILDER().
+	case "LUA":
+		target = fmt.Sprintf("%q, %s", rec.LuaRType, jsonQuoted(rec.GetTargetTXTJoined()))
 	case "NS":
 		// NS records at the apex should be NAMESERVER() records.
 		// DnsControl uses the API to get this info. NAMESERVER() is just
