@@ -28,6 +28,7 @@ import (
 //	  NAPTR
 //	  NS
 //	  PTR
+//	  SMIMEA
 //	  SOA
 //	  SRV
 //	  SSHFP
@@ -45,8 +46,8 @@ import (
 //	  IMPORT_TRANSFORM
 //	  NAMESERVER
 //	  NO_PURGE
-//	  NS1_URLFWD
 //	  PAGE_RULE
+//	  PORKBUN_URLFWD
 //	  PURGE
 //	  URL
 //	  URL301
@@ -84,6 +85,7 @@ import (
 // This is the subdomain path, if any, imported from the configuration. If
 // present at the time of canonicalization it is inserted between the
 // Name and origin when constructing a canonical (FQDN) target.
+// (the SubDomain is not used in processing the label itself. That was done in helpers.js)
 //
 // Idioms:
 //
@@ -91,56 +93,61 @@ import (
 type RecordConfig struct {
 	Type      string            `json:"type"` // All caps rtype name.
 	Name      string            `json:"name"` // The short name. See above.
+	NameFQDN  string            `json:"-"`    // Must end with ".$origin". See above.
 	SubDomain string            `json:"subdomain,omitempty"`
-	NameFQDN  string            `json:"-"` // Must end with ".$origin". See above.
 	target    string            // If a name, must end with "."
 	TTL       uint32            `json:"ttl,omitempty"`
 	Metadata  map[string]string `json:"meta,omitempty"`
+	FilePos   string            `json:"filepos"`
 	Original  interface{}       `json:"-"` // Store pointer to provider-specific record object. Used in diffing.
 
 	// If you add a field to this struct, also add it to the list in the UnmarshalJSON function.
-	MxPreference     uint16            `json:"mxpreference,omitempty"`
-	SrvPriority      uint16            `json:"srvpriority,omitempty"`
-	SrvWeight        uint16            `json:"srvweight,omitempty"`
-	SrvPort          uint16            `json:"srvport,omitempty"`
-	CaaTag           string            `json:"caatag,omitempty"`
-	CaaFlag          uint8             `json:"caaflag,omitempty"`
-	DsKeyTag         uint16            `json:"dskeytag,omitempty"`
-	DsAlgorithm      uint8             `json:"dsalgorithm,omitempty"`
-	DsDigestType     uint8             `json:"dsdigesttype,omitempty"`
-	DsDigest         string            `json:"dsdigest,omitempty"`
-	DnskeyFlags      uint16            `json:"dnskeyflags,omitempty"`
-	DnskeyProtocol   uint8             `json:"dnskeyprotocol,omitempty"`
-	DnskeyAlgorithm  uint8             `json:"dnskeyalgorithm,omitempty"`
-	DnskeyPublicKey  string            `json:"dnskeypublickey,omitempty"`
-	LocVersion       uint8             `json:"locversion,omitempty"`
-	LocSize          uint8             `json:"locsize,omitempty"`
-	LocHorizPre      uint8             `json:"lochorizpre,omitempty"`
-	LocVertPre       uint8             `json:"locvertpre,omitempty"`
-	LocLatitude      uint32            `json:"loclatitude,omitempty"`
-	LocLongitude     uint32            `json:"loclongitude,omitempty"`
-	LocAltitude      uint32            `json:"localtitude,omitempty"`
-	NaptrOrder       uint16            `json:"naptrorder,omitempty"`
-	NaptrPreference  uint16            `json:"naptrpreference,omitempty"`
-	NaptrFlags       string            `json:"naptrflags,omitempty"`
-	NaptrService     string            `json:"naptrservice,omitempty"`
-	NaptrRegexp      string            `json:"naptrregexp,omitempty"`
-	SshfpAlgorithm   uint8             `json:"sshfpalgorithm,omitempty"`
-	SshfpFingerprint uint8             `json:"sshfpfingerprint,omitempty"`
-	SoaMbox          string            `json:"soambox,omitempty"`
-	SoaSerial        uint32            `json:"soaserial,omitempty"`
-	SoaRefresh       uint32            `json:"soarefresh,omitempty"`
-	SoaRetry         uint32            `json:"soaretry,omitempty"`
-	SoaExpire        uint32            `json:"soaexpire,omitempty"`
-	SoaMinttl        uint32            `json:"soaminttl,omitempty"`
-	SvcPriority      uint16            `json:"svcpriority,omitempty"`
-	SvcParams        string            `json:"svcparams,omitempty"`
-	TlsaUsage        uint8             `json:"tlsausage,omitempty"`
-	TlsaSelector     uint8             `json:"tlsaselector,omitempty"`
-	TlsaMatchingType uint8             `json:"tlsamatchingtype,omitempty"`
-	R53Alias         map[string]string `json:"r53_alias,omitempty"`
-	AzureAlias       map[string]string `json:"azure_alias,omitempty"`
-	UnknownTypeName  string            `json:"unknown_type_name,omitempty"`
+	MxPreference       uint16            `json:"mxpreference,omitempty"`
+	SrvPriority        uint16            `json:"srvpriority,omitempty"`
+	SrvWeight          uint16            `json:"srvweight,omitempty"`
+	SrvPort            uint16            `json:"srvport,omitempty"`
+	CaaTag             string            `json:"caatag,omitempty"`
+	CaaFlag            uint8             `json:"caaflag,omitempty"`
+	DsKeyTag           uint16            `json:"dskeytag,omitempty"`
+	DsAlgorithm        uint8             `json:"dsalgorithm,omitempty"`
+	DsDigestType       uint8             `json:"dsdigesttype,omitempty"`
+	DsDigest           string            `json:"dsdigest,omitempty"`
+	DnskeyFlags        uint16            `json:"dnskeyflags,omitempty"`
+	DnskeyProtocol     uint8             `json:"dnskeyprotocol,omitempty"`
+	DnskeyAlgorithm    uint8             `json:"dnskeyalgorithm,omitempty"`
+	DnskeyPublicKey    string            `json:"dnskeypublickey,omitempty"`
+	LocVersion         uint8             `json:"locversion,omitempty"`
+	LocSize            uint8             `json:"locsize,omitempty"`
+	LocHorizPre        uint8             `json:"lochorizpre,omitempty"`
+	LocVertPre         uint8             `json:"locvertpre,omitempty"`
+	LocLatitude        uint32            `json:"loclatitude,omitempty"`
+	LocLongitude       uint32            `json:"loclongitude,omitempty"`
+	LocAltitude        uint32            `json:"localtitude,omitempty"`
+	LuaRType           string            `json:"luartype,omitempty"`
+	NaptrOrder         uint16            `json:"naptrorder,omitempty"`
+	NaptrPreference    uint16            `json:"naptrpreference,omitempty"`
+	NaptrFlags         string            `json:"naptrflags,omitempty"`
+	NaptrService       string            `json:"naptrservice,omitempty"`
+	NaptrRegexp        string            `json:"naptrregexp,omitempty"`
+	SmimeaUsage        uint8             `json:"smimeausage,omitempty"`
+	SmimeaSelector     uint8             `json:"smimeaselector,omitempty"`
+	SmimeaMatchingType uint8             `json:"smimeamatchingtype,omitempty"`
+	SshfpAlgorithm     uint8             `json:"sshfpalgorithm,omitempty"`
+	SshfpFingerprint   uint8             `json:"sshfpfingerprint,omitempty"`
+	SoaMbox            string            `json:"soambox,omitempty"`
+	SoaSerial          uint32            `json:"soaserial,omitempty"`
+	SoaRefresh         uint32            `json:"soarefresh,omitempty"`
+	SoaRetry           uint32            `json:"soaretry,omitempty"`
+	SoaExpire          uint32            `json:"soaexpire,omitempty"`
+	SoaMinttl          uint32            `json:"soaminttl,omitempty"`
+	SvcPriority        uint16            `json:"svcpriority,omitempty"`
+	SvcParams          string            `json:"svcparams,omitempty"`
+	TlsaUsage          uint8             `json:"tlsausage,omitempty"`
+	TlsaSelector       uint8             `json:"tlsaselector,omitempty"`
+	TlsaMatchingType   uint8             `json:"tlsamatchingtype,omitempty"`
+	R53Alias           map[string]string `json:"r53_alias,omitempty"`
+	AzureAlias         map[string]string `json:"azure_alias,omitempty"`
+	UnknownTypeName    string            `json:"unknown_type_name,omitempty"`
 
 	// Cloudflare-specific fields:
 	// When these are used, .target is set to a human-readable version (only to be used for display purposes).
@@ -152,19 +159,20 @@ type RecordConfig struct {
 //	When these are used, .target is set to a human-readable version (only to be used for display purposes).
 type CloudflareSingleRedirectConfig struct {
 	//
-	Code int `json:"code,omitempty"` // 301 or 302
+	Code uint16 `json:"code,omitempty"` // 301 or 302
 	// PR == PageRule
-	PRDisplay     string `json:"pr_display,omitempty"` // How is this displayed to the user
-	PRMatcher     string `json:"pr_matcher,omitempty"`
-	PRReplacement string `json:"pr_replacement,omitempty"`
-	PRPriority    int    `json:"pr_priority,omitempty"` // Really an identifier for the rule.
+	PRWhen     string `json:"pr_when,omitempty"`
+	PRThen     string `json:"pr_then,omitempty"`
+	PRPriority int    `json:"pr_priority,omitempty"` // Really an identifier for the rule.
+	PRDisplay  string `json:"pr_display,omitempty"`  // How is this displayed to the user (SetTarget) for CF_REDIRECT/CF_TEMP_REDIRECT
 	//
 	// SR == SingleRedirect
-	SRDisplay        string `json:"sr_display,omitempty"` // How is this displayed to the user
-	SRMatcher        string `json:"sr_matcher,omitempty"`
-	SRReplacement    string `json:"sr_replacement,omitempty"`
+	SRName           string `json:"sr_name,omitempty"` // How is this displayed to the user
+	SRWhen           string `json:"sr_when,omitempty"`
+	SRThen           string `json:"sr_then,omitempty"`
 	SRRRulesetID     string `json:"sr_rulesetid,omitempty"`
 	SRRRulesetRuleID string `json:"sr_rulesetruleid,omitempty"`
+	SRDisplay        string `json:"sr_display,omitempty"` // How is this displayed to the user (SetTarget) for CF_SINGLE_REDIRECT
 }
 
 // MarshalJSON marshals RecordConfig.
@@ -195,50 +203,56 @@ func (rc *RecordConfig) UnmarshalJSON(b []byte) error {
 		target    string            // If a name, must end with "."
 		TTL       uint32            `json:"ttl,omitempty"`
 		Metadata  map[string]string `json:"meta,omitempty"`
-		Original  interface{}       `json:"-"` // Store pointer to provider-specific record object. Used in diffing.
+		FilePos   string            `json:"filepos"` // Where in the file this record was defined.
+		Original  interface{}       `json:"-"`       // Store pointer to provider-specific record object. Used in diffing.
+		Args      []any             `json:"args,omitempty"`
 
-		MxPreference     uint16            `json:"mxpreference,omitempty"`
-		SrvPriority      uint16            `json:"srvpriority,omitempty"`
-		SrvWeight        uint16            `json:"srvweight,omitempty"`
-		SrvPort          uint16            `json:"srvport,omitempty"`
-		CaaTag           string            `json:"caatag,omitempty"`
-		CaaFlag          uint8             `json:"caaflag,omitempty"`
-		DsKeyTag         uint16            `json:"dskeytag,omitempty"`
-		DsAlgorithm      uint8             `json:"dsalgorithm,omitempty"`
-		DsDigestType     uint8             `json:"dsdigesttype,omitempty"`
-		DsDigest         string            `json:"dsdigest,omitempty"`
-		DnskeyFlags      uint16            `json:"dnskeyflags,omitempty"`
-		DnskeyProtocol   uint8             `json:"dnskeyprotocol,omitempty"`
-		DnskeyAlgorithm  uint8             `json:"dnskeyalgorithm,omitempty"`
-		DnskeyPublicKey  string            `json:"dnskeypublickey,omitempty"`
-		LocVersion       uint8             `json:"locversion,omitempty"`
-		LocSize          uint8             `json:"locsize,omitempty"`
-		LocHorizPre      uint8             `json:"lochorizpre,omitempty"`
-		LocVertPre       uint8             `json:"locvertpre,omitempty"`
-		LocLatitude      int               `json:"loclatitude,omitempty"`
-		LocLongitude     int               `json:"loclongitude,omitempty"`
-		LocAltitude      uint32            `json:"localtitude,omitempty"`
-		NaptrOrder       uint16            `json:"naptrorder,omitempty"`
-		NaptrPreference  uint16            `json:"naptrpreference,omitempty"`
-		NaptrFlags       string            `json:"naptrflags,omitempty"`
-		NaptrService     string            `json:"naptrservice,omitempty"`
-		NaptrRegexp      string            `json:"naptrregexp,omitempty"`
-		SshfpAlgorithm   uint8             `json:"sshfpalgorithm,omitempty"`
-		SshfpFingerprint uint8             `json:"sshfpfingerprint,omitempty"`
-		SoaMbox          string            `json:"soambox,omitempty"`
-		SoaSerial        uint32            `json:"soaserial,omitempty"`
-		SoaRefresh       uint32            `json:"soarefresh,omitempty"`
-		SoaRetry         uint32            `json:"soaretry,omitempty"`
-		SoaExpire        uint32            `json:"soaexpire,omitempty"`
-		SoaMinttl        uint32            `json:"soaminttl,omitempty"`
-		SvcPriority      uint16            `json:"svcpriority,omitempty"`
-		SvcParams        string            `json:"svcparams,omitempty"`
-		TlsaUsage        uint8             `json:"tlsausage,omitempty"`
-		TlsaSelector     uint8             `json:"tlsaselector,omitempty"`
-		TlsaMatchingType uint8             `json:"tlsamatchingtype,omitempty"`
-		R53Alias         map[string]string `json:"r53_alias,omitempty"`
-		AzureAlias       map[string]string `json:"azure_alias,omitempty"`
-		UnknownTypeName  string            `json:"unknown_type_name,omitempty"`
+		MxPreference       uint16            `json:"mxpreference,omitempty"`
+		SrvPriority        uint16            `json:"srvpriority,omitempty"`
+		SrvWeight          uint16            `json:"srvweight,omitempty"`
+		SrvPort            uint16            `json:"srvport,omitempty"`
+		CaaTag             string            `json:"caatag,omitempty"`
+		CaaFlag            uint8             `json:"caaflag,omitempty"`
+		DsKeyTag           uint16            `json:"dskeytag,omitempty"`
+		DsAlgorithm        uint8             `json:"dsalgorithm,omitempty"`
+		DsDigestType       uint8             `json:"dsdigesttype,omitempty"`
+		DsDigest           string            `json:"dsdigest,omitempty"`
+		DnskeyFlags        uint16            `json:"dnskeyflags,omitempty"`
+		DnskeyProtocol     uint8             `json:"dnskeyprotocol,omitempty"`
+		DnskeyAlgorithm    uint8             `json:"dnskeyalgorithm,omitempty"`
+		DnskeyPublicKey    string            `json:"dnskeypublickey,omitempty"`
+		LocVersion         uint8             `json:"locversion,omitempty"`
+		LocSize            uint8             `json:"locsize,omitempty"`
+		LocHorizPre        uint8             `json:"lochorizpre,omitempty"`
+		LocVertPre         uint8             `json:"locvertpre,omitempty"`
+		LocLatitude        int               `json:"loclatitude,omitempty"`
+		LocLongitude       int               `json:"loclongitude,omitempty"`
+		LocAltitude        uint32            `json:"localtitude,omitempty"`
+		LuaRType           string            `json:"luartype,omitempty"`
+		NaptrOrder         uint16            `json:"naptrorder,omitempty"`
+		NaptrPreference    uint16            `json:"naptrpreference,omitempty"`
+		NaptrFlags         string            `json:"naptrflags,omitempty"`
+		NaptrService       string            `json:"naptrservice,omitempty"`
+		NaptrRegexp        string            `json:"naptrregexp,omitempty"`
+		SmimeaUsage        uint8             `json:"smimeausage,omitempty"`
+		SmimeaSelector     uint8             `json:"smimeaselector,omitempty"`
+		SmimeaMatchingType uint8             `json:"smimeamatchingtype,omitempty"`
+		SshfpAlgorithm     uint8             `json:"sshfpalgorithm,omitempty"`
+		SshfpFingerprint   uint8             `json:"sshfpfingerprint,omitempty"`
+		SoaMbox            string            `json:"soambox,omitempty"`
+		SoaSerial          uint32            `json:"soaserial,omitempty"`
+		SoaRefresh         uint32            `json:"soarefresh,omitempty"`
+		SoaRetry           uint32            `json:"soaretry,omitempty"`
+		SoaExpire          uint32            `json:"soaexpire,omitempty"`
+		SoaMinttl          uint32            `json:"soaminttl,omitempty"`
+		SvcPriority        uint16            `json:"svcpriority,omitempty"`
+		SvcParams          string            `json:"svcparams,omitempty"`
+		TlsaUsage          uint8             `json:"tlsausage,omitempty"`
+		TlsaSelector       uint8             `json:"tlsaselector,omitempty"`
+		TlsaMatchingType   uint8             `json:"tlsamatchingtype,omitempty"`
+		R53Alias           map[string]string `json:"r53_alias,omitempty"`
+		AzureAlias         map[string]string `json:"azure_alias,omitempty"`
+		UnknownTypeName    string            `json:"unknown_type_name,omitempty"`
 
 		EnsureAbsent bool `json:"ensure_absent,omitempty"` // Override NO_PURGE and delete this record
 
@@ -249,10 +263,16 @@ func (rc *RecordConfig) UnmarshalJSON(b []byte) error {
 		return err
 	}
 
+	recj.FilePos = FixPosition(recj.FilePos)
+
 	// Copy the exported fields.
-	copier.CopyWithOption(&rc, &recj, copier.Option{IgnoreEmpty: true, DeepCopy: true})
+	if err := copier.CopyWithOption(&rc, &recj, copier.Option{IgnoreEmpty: true, DeepCopy: true}); err != nil {
+		return err
+	}
 	// Set each unexported field.
-	rc.SetTarget(recj.Target)
+	if err := rc.SetTarget(recj.Target); err != nil {
+		return err
+	}
 
 	// Some sanity checks:
 	if recj.Type != rc.Type {
@@ -268,6 +288,16 @@ func (rc *RecordConfig) UnmarshalJSON(b []byte) error {
 	return nil
 }
 
+func FixPosition(str string) string {
+	if str == "" {
+		return ""
+	}
+	str = strings.TrimSpace(str)
+	str = strings.ReplaceAll(str, "\n", " ")
+	str = strings.TrimPrefix(str, "at <anonymous>:")
+	return fmt.Sprintf("[line:%s]", str)
+}
+
 // Copy returns a deep copy of a RecordConfig.
 func (rc *RecordConfig) Copy() (*RecordConfig, error) {
 	newR := &RecordConfig{}
@@ -285,14 +315,13 @@ func (rc *RecordConfig) Copy() (*RecordConfig, error) {
 // short must not have a training dot: That would mean you have a FQDN, and
 // shouldn't be using SetLabel().  Maybe SetLabelFromFQDN()?
 func (rc *RecordConfig) SetLabel(short, origin string) {
-
 	// Assertions that make sure the function is being used correctly:
 	if strings.HasSuffix(origin, ".") {
 		panic(fmt.Errorf("origin (%s) is not supposed to end with a dot", origin))
 	}
 	if strings.HasSuffix(short, ".") {
 		if short != "**current-domain**" {
-			panic(fmt.Errorf("short (%s) is not supposed to end with a dot", origin))
+			panic(fmt.Errorf("short (%s) is not supposed to end with a dot", short))
 		}
 	}
 
@@ -314,7 +343,6 @@ func (rc *RecordConfig) SetLabel(short, origin string) {
 // fqdn may have a trailing "." but it is not required.
 // origin may not have a trailing dot.
 func (rc *RecordConfig) SetLabelFromFQDN(fqdn, origin string) {
-
 	// Assertions that make sure the function is being used correctly:
 	if strings.HasSuffix(origin, ".") {
 		panic(fmt.Errorf("origin (%s) is not supposed to end with a dot", origin))
@@ -357,10 +385,12 @@ func (rc *RecordConfig) ToComparableNoTTL() string {
 		return fmt.Sprintf("%s %v %d %d %d %d", rc.target, rc.SoaMbox, rc.SoaRefresh, rc.SoaRetry, rc.SoaExpire, rc.SoaMinttl)
 		// SoaSerial is not included because it isn't used in comparisons.
 	case "TXT":
-		//fmt.Fprintf(os.Stdout, "DEBUG: ToComNoTTL raw txts=%s q=%q\n", rc.target, rc.target)
+		// fmt.Fprintf(os.Stdout, "DEBUG: ToComNoTTL raw txts=%s q=%q\n", rc.target, rc.target)
 		r := txtutil.EncodeQuoted(rc.target)
-		//fmt.Fprintf(os.Stdout, "DEBUG: ToComNoTTL cmp txts=%s q=%q\n", r, r)
+		// fmt.Fprintf(os.Stdout, "DEBUG: ToComNoTTL cmp txts=%s q=%q\n", r, r)
 		return r
+	case "LUA":
+		return rc.luaCombined()
 	case "UNKNOWN":
 		return fmt.Sprintf("rtype=%s rdata=%s", rc.UnknownTypeName, rc.target)
 	}
@@ -369,7 +399,6 @@ func (rc *RecordConfig) ToComparableNoTTL() string {
 
 // ToRR converts a RecordConfig to a dns.RR.
 func (rc *RecordConfig) ToRR() dns.RR {
-
 	// Don't call this on fake types.
 	rdtype, ok := dns.StringToType[rc.Type]
 	if !ok {
@@ -440,8 +469,15 @@ func (rc *RecordConfig) ToRR() dns.RR {
 		rr.(*dns.NAPTR).Replacement = rc.GetTargetField()
 	case dns.TypeNS:
 		rr.(*dns.NS).Ns = rc.GetTargetField()
+	case dns.TypeOPENPGPKEY:
+		rr.(*dns.OPENPGPKEY).PublicKey = rc.GetTargetField()
 	case dns.TypePTR:
 		rr.(*dns.PTR).Ptr = rc.GetTargetField()
+	case dns.TypeSMIMEA:
+		rr.(*dns.SMIMEA).Usage = rc.SmimeaUsage
+		rr.(*dns.SMIMEA).MatchingType = rc.SmimeaMatchingType
+		rr.(*dns.SMIMEA).Selector = rc.SmimeaSelector
+		rr.(*dns.SMIMEA).Certificate = rc.GetTargetField()
 	case dns.TypeSOA:
 		rr.(*dns.SOA).Ns = rc.GetTargetField()
 		rr.(*dns.SOA).Mbox = rc.SoaMbox
@@ -483,7 +519,6 @@ func (rc *RecordConfig) ToRR() dns.RR {
 
 // GetDependencies returns the FQDNs on which this record dependents
 func (rc *RecordConfig) GetDependencies() []string {
-
 	switch rc.Type {
 	// #rtype_variations
 	case "NS", "SRV", "CNAME", "DNAME", "MX", "ALIAS", "AZURE_ALIAS", "R53_ALIAS":
@@ -526,6 +561,10 @@ func (rc *RecordConfig) Key() RecordKey {
 
 // GetSVCBValue returns the SVCB Key/Values as a list of Key/Values.
 func (rc *RecordConfig) GetSVCBValue() []dns.SVCBKeyValue {
+	if !strings.Contains(rc.SvcParams, "IGNORE+DNSCONTROL") {
+		rc.SvcParams = strings.ReplaceAll(rc.SvcParams, "ech=IGNORE", "ech=IGNORE+DNSCONTROL+++")
+	}
+
 	record, err := dns.NewRR(fmt.Sprintf("%s %s %d %s %s", rc.NameFQDN, rc.Type, rc.SvcPriority, rc.target, rc.SvcParams))
 	if err != nil {
 		log.Fatalf("could not parse SVCB record: %s", err)
@@ -608,11 +647,11 @@ func Downcase(recs []*RecordConfig) {
 		r.Name = strings.ToLower(r.Name)
 		r.NameFQDN = strings.ToLower(r.NameFQDN)
 		switch r.Type { // #rtype_variations
-		case "AKAMAICDN", "ALIAS", "AAAA", "ANAME", "CNAME", "DNAME", "DS", "DNSKEY", "MX", "NS", "NAPTR", "PTR", "SRV", "TLSA":
+		case "AKAMAICDN", "ALIAS", "AAAA", "ANAME", "CNAME", "DNAME", "DS", "DNSKEY", "MX", "NS", "NAPTR", "OPENPGPKEY", "SMIMEA", "PTR", "SRV", "TLSA", "AZURE_ALIAS":
 			// Target is case insensitive. Downcase it.
 			r.target = strings.ToLower(r.target)
 			// BUGFIX(tlim): isn't ALIAS in the wrong case statement?
-		case "A", "CAA", "CF_REDIRECT", "CF_TEMP_REDIRECT", "CF_WORKER_ROUTE", "DHCID", "IMPORT_TRANSFORM", "LOC", "SSHFP", "TXT":
+		case "A", "CAA", "CLOUDFLAREAPI_SINGLE_REDIRECT", "CF_REDIRECT", "CF_TEMP_REDIRECT", "CF_WORKER_ROUTE", "DHCID", "IMPORT_TRANSFORM", "LOC", "SSHFP", "TXT", "ADGUARDHOME_A_PASSTHROUGH", "ADGUARDHOME_AAAA_PASSTHROUGH":
 			// Do nothing. (IP address or case sensitive target)
 		case "SOA":
 			if r.target != "DEFAULT_NOT_SET." {
@@ -636,7 +675,7 @@ func CanonicalizeTargets(recs []*RecordConfig, origin string) {
 		case "ALIAS", "ANAME", "CNAME", "DNAME", "DS", "DNSKEY", "MX", "NS", "NAPTR", "PTR", "SRV":
 			// Target is a hostname that might be a shortname. Turn it into a FQDN.
 			r.target = dnsutil.AddOrigin(r.target, originFQDN)
-		case "A", "AKAMAICDN", "CAA", "DHCID", "CF_REDIRECT", "CF_TEMP_REDIRECT", "CF_WORKER_ROUTE", "HTTPS", "IMPORT_TRANSFORM", "LOC", "SSHFP", "SVCB", "TLSA", "TXT":
+		case "A", "AKAMAICDN", "CAA", "DHCID", "CLOUDFLAREAPI_SINGLE_REDIRECT", "CF_REDIRECT", "CF_TEMP_REDIRECT", "CF_WORKER_ROUTE", "HTTPS", "IMPORT_TRANSFORM", "LOC", "OPENPGPKEY", "SMIMEA", "SSHFP", "SVCB", "TLSA", "TXT", "ADGUARDHOME_A_PASSTHROUGH", "ADGUARDHOME_AAAA_PASSTHROUGH":
 			// Do nothing.
 		case "SOA":
 			if r.target != "DEFAULT_NOT_SET." {
