@@ -90,6 +90,10 @@ func (rc *RecordConfig) PopulateFromStringFunc(rtype, contents, origin string, t
 		return rc.SetTargetMXString(contents)
 	case "NAPTR":
 		return rc.SetTargetNAPTRString(contents)
+	case "OPENPGPKEY":
+		return rc.SetTarget(contents)
+	case "SMIMEA":
+		return rc.SetTargetSMIMEAString(contents)
 	case "SOA":
 		return rc.SetTargetSOAString(contents)
 	case "SPF", "TXT":
@@ -101,6 +105,21 @@ func (rc *RecordConfig) PopulateFromStringFunc(rtype, contents, origin string, t
 			return fmt.Errorf("invalid TXT record: %s", contents)
 		}
 		return rc.SetTargetTXT(t)
+	case "LUA":
+		luaType, payload := ParseLuaContent(contents)
+		rc.LuaRType = luaType
+		if txtFn != nil {
+			value, err := txtFn(payload)
+			if err != nil {
+				return fmt.Errorf("invalid LUA record: %s", contents)
+			}
+			return rc.SetTargetTXT(value)
+		}
+		value, err := DecodeLuaPayload(payload)
+		if err != nil {
+			return fmt.Errorf("invalid LUA record: %s", contents)
+		}
+		return rc.SetTargetTXT(value)
 	case "SRV":
 		return rc.SetTargetSRVString(contents)
 	case "SSHFP":
@@ -110,7 +129,7 @@ func (rc *RecordConfig) PopulateFromStringFunc(rtype, contents, origin string, t
 	case "TLSA":
 		return rc.SetTargetTLSAString(contents)
 	default:
-		//return fmt.Errorf("unknown rtype (%s) when parsing (%s) domain=(%s)", rtype, contents, origin)
+		// return fmt.Errorf("unknown rtype (%s) when parsing (%s) domain=(%s)", rtype, contents, origin)
 		return MakeUnknown(rc, rtype, contents, origin)
 	}
 }
@@ -180,10 +199,22 @@ func (rc *RecordConfig) PopulateFromString(rtype, contents, origin string) error
 		return rc.SetTargetMXString(contents)
 	case "NAPTR":
 		return rc.SetTargetNAPTRString(contents)
+	case "OPENPGPKEY":
+		return rc.SetTarget(contents)
+	case "SMIMEA":
+		return rc.SetTargetSMIMEAString(contents)
 	case "SOA":
 		return rc.SetTargetSOAString(contents)
 	case "SPF", "TXT":
 		return rc.SetTargetTXTs(ParseQuotedTxt(contents))
+	case "LUA":
+		luaType, payload := ParseLuaContent(contents)
+		rc.LuaRType = luaType
+		value, err := DecodeLuaPayload(payload)
+		if err != nil {
+			return fmt.Errorf("invalid LUA record: %s", contents)
+		}
+		return rc.SetTargetTXT(value)
 	case "SRV":
 		return rc.SetTargetSRVString(contents)
 	case "SSHFP":
