@@ -5,6 +5,7 @@ import (
 	"net"
 	"strings"
 
+	"github.com/StackExchange/dnscontrol/v4/pkg/txtutil"
 	"github.com/miekg/dns"
 )
 
@@ -31,7 +32,7 @@ func (rc *RecordConfig) GetTargetIP() net.IP {
 // string. How TXT records are encoded is defined by encodeFn.  If encodeFn is
 // nil the TXT data is returned unaltered.
 func (rc *RecordConfig) GetTargetCombinedFunc(encodeFn func(s string) string) string {
-	if rc.Type == "TXT" {
+	if rc.Type == "TXT" || rc.Type == "LUA" {
 		if encodeFn == nil {
 			return rc.target
 		}
@@ -45,10 +46,11 @@ func (rc *RecordConfig) GetTargetCombinedFunc(encodeFn func(s string) string) st
 // WARNING: How TXT records are handled is buggy but we can't change it because
 // code depends on the bugs. Use Get GetTargetCombinedFunc() instead.
 func (rc *RecordConfig) GetTargetCombined() string {
-
 	// Pseudo records:
 	if _, ok := dns.StringToType[rc.Type]; !ok {
 		switch rc.Type { // #rtype_variations
+		case "LUA":
+			return rc.luaCombined()
 		case "R53_ALIAS":
 			// Differentiate between multiple R53_ALIASs on the same label.
 			return fmt.Sprintf("%s atype=%s zone_id=%s evaluate_target_health=%s", rc.target, rc.R53Alias["type"], rc.R53Alias["zone_id"], rc.R53Alias["evaluate_target_health"])
@@ -82,7 +84,7 @@ func (rc *RecordConfig) zoneFileQuoted() string {
 	// TODO(tlim): Request the dns project add a function that returns
 	// the string without the header.
 	if rc.Type == "NAPTR" && rc.GetTargetField() == "" {
-		rc.SetTarget(".")
+		rc.MustSetTarget(".")
 	}
 	rr := rc.ToRR()
 	header := rr.Header().String()
@@ -93,6 +95,26 @@ func (rc *RecordConfig) zoneFileQuoted() string {
 	return full[len(header):]
 }
 
+func (rc *RecordConfig) luaCombined() string {
+	rtype := rc.luaTypeUpper()
+	payload := rc.target
+	if rtype == "" {
+		return payload
+	}
+	payload = txtutil.EncodeQuoted(payload)
+	if payload == "" {
+		return rtype
+	}
+	return fmt.Sprintf("%s %s", rtype, payload)
+}
+
+func (rc *RecordConfig) luaTypeUpper() string {
+	if rc.LuaRType == "" {
+		return ""
+	}
+	return strings.ToUpper(rc.LuaRType)
+}
+
 // GetTargetRFC1035Quoted returns the target as it would be in an
 // RFC1035-style zonefile.
 // Do not use this function if RecordConfig might be a pseudo-rtype
@@ -104,15 +126,18 @@ func (rc *RecordConfig) GetTargetRFC1035Quoted() string {
 // GetTargetDebug returns a string with the various fields spelled out.
 func (rc *RecordConfig) GetTargetDebug() string {
 	target := rc.target
-	if rc.Type == "TXT" {
+	//if rc.Type == "TXT" {
+	if rc.HasFormatIdenticalToTXT() {
 		target = fmt.Sprintf("%q", target)
 	}
 	content := fmt.Sprintf("%s %s %s %d", rc.Type, rc.NameFQDN, target, rc.TTL)
 	switch rc.Type { // #rtype_variations
-	case "A", "AAAA", "AKAMAICDN", "CNAME", "DHCID", "NS", "PTR", "TXT":
+	case "A", "AAAA", "AKAMAICDN", "CNAME", "DHCID", "NS", "OPENPGPKEY", "PTR", "TXT":
 		// Nothing special.
+	case "LUA":
+		content += " luartype=" + rc.luaTypeUpper()
 	case "AZURE_ALIAS":
-		content += fmt.Sprintf(" type=%s", rc.AzureAlias["type"])
+		content += " type=" + rc.AzureAlias["type"]
 	case "CAA":
 		content += fmt.Sprintf(" caatag=%s caaflag=%d", rc.CaaTag, rc.CaaFlag)
 	case "DS":
@@ -125,6 +150,8 @@ func (rc *RecordConfig) GetTargetDebug() string {
 		content += fmt.Sprintf(" naptrorder=%d naptrpreference=%d naptrflags=%s naptrservice=%s naptrregexp=%s", rc.NaptrOrder, rc.NaptrPreference, rc.NaptrFlags, rc.NaptrService, rc.NaptrRegexp)
 	case "R53_ALIAS":
 		content += fmt.Sprintf(" type=%s zone_id=%s evaluate_target_health=%s", rc.R53Alias["type"], rc.R53Alias["zone_id"], rc.R53Alias["evaluate_target_health"])
+	case "SMIMEA":
+		content += fmt.Sprintf(" smimeausage=%d smimeaselector=%d smimeamatchingtype=%d", rc.SmimeaUsage, rc.SmimeaSelector, rc.SmimeaMatchingType)
 	case "SOA":
 		content = fmt.Sprintf("%s ns=%v mbox=%v serial=%v refresh=%v retry=%v expire=%v minttl=%v", rc.Type, rc.target, rc.SoaMbox, rc.SoaSerial, rc.SoaRefresh, rc.SoaRetry, rc.SoaExpire, rc.SoaMinttl)
 	case "SRV":
@@ -153,9 +180,16 @@ func (rc *RecordConfig) SetTarget(target string) error {
 	return nil
 }
 
+// MustSetTarget is like SetTarget, but panics if an error occurs.
+// It should only be used in _test.go files and in the init() function.
+func (rc *RecordConfig) MustSetTarget(target string) {
+	if err := rc.SetTarget(target); err != nil {
+		panic(err)
+	}
+}
+
 // SetTargetIP sets the target to an IP, verifying this is an appropriate rtype.
 func (rc *RecordConfig) SetTargetIP(ip net.IP) error {
 	// TODO(tlim): Verify the rtype is appropriate for an IP.
-	rc.SetTarget(ip.String())
-	return nil
+	return rc.SetTarget(ip.String())
 }
