@@ -12,6 +12,13 @@
 // debugging/developing this code, it may be faster to specify the
 // -dev file to have helpers.js read from the file instead.
 
+// If this javascript interpreter doesn't have a .endsWith() function on strings, add one.
+if (typeof String.prototype.endsWith !== 'function') {
+    String.prototype.endsWith = function (suffix) {
+        return this.indexOf(suffix, this.length - suffix.length) !== -1;
+    };
+}
+
 var conf = {
     registrars: [],
     dns_providers: [],
@@ -104,6 +111,7 @@ function newDomain(name, registrar) {
         registrar: registrar,
         meta: {},
         records: [],
+        rawrecords: [],
         recordsabsent: [],
         dnsProviders: {},
         defaultTTL: 0,
@@ -147,8 +155,20 @@ function D(name, registrar) {
         var m = arguments[i];
         processDargs(m, domain);
     }
+
+    // handle the empty tag ("example.com!" -> "example.com")
+    // replace name with result of removing the empty tag if it exists
+    // keep track so we can explain the situation in the error message
+    var withoutEmptyTag = _removeEmptyTag(name);
+    name = withoutEmptyTag[0];
+    var tagWasRemoved = withoutEmptyTag[1];
+
     if (conf.domain_names.indexOf(name) !== -1) {
-        throw name + ' is declared more than once';
+        var message = name + ' is declared more than once';
+        if (tagWasRemoved) {
+            message += ' (check empty tags)';
+        }
+        throw message;
     }
     conf.domains.push(domain);
     conf.domain_names.push(name);
@@ -176,10 +196,17 @@ function D_EXTEND(name) {
             ' was not declared yet and therefore cannot be updated. Use D() before.'
         );
     }
+
+    // Handle weird REV() case.
+    if (name.indexOf('/') !== -1) {
+        name = name.substring(name.indexOf('.') + 1);
+    }
+
     domain.obj.subdomain = name.substr(
         0,
         name.length - domain.obj.name.length - 1
     );
+
     for (var i = 1; i < arguments.length; i++) {
         var m = arguments[i];
         processDargs(m, domain.obj);
@@ -187,15 +214,31 @@ function D_EXTEND(name) {
     conf.domains[domain.id] = domain.obj; // let's overwrite the object.
 }
 
+// _removeEmptyTag(domain): Remove empty tag.
+function _removeEmptyTag(name) {
+    var tagWasRemoved = false;
+    if (name.slice(-1) === '!') {
+        name = name.slice(0, name.length - 1);
+        tagWasRemoved = true;
+    }
+    return [name, tagWasRemoved];
+}
+
 // _getDomainObject(name): This implements the domain matching
 // algorithm used by D_EXTEND(). Candidate matches are an exact match
 // of the domain's name, or if name is a proper subdomain of the
 // domain's name. The longest match is returned.
 function _getDomainObject(name) {
+    var nameTrimmedTag = _removeEmptyTag(name);
+    name = nameTrimmedTag[0];
     var domain = null;
     var domainLen = 0;
     for (var i = 0; i < conf.domains.length; i++) {
         var thisName = conf.domains[i]['name'];
+        // check for empty tag
+        var thisNameTrimmedTag = _removeEmptyTag(thisName);
+        thisName = thisNameTrimmedTag[0];
+
         var desiredSuffix = '.' + thisName;
         var foundSuffix = name.substr(-desiredSuffix.length);
         // If this is an exact match or the suffix matches...
@@ -490,6 +533,27 @@ var NAPTR = recordBuilder('NAPTR', {
     },
 });
 
+// OPENPGPKEY(name,target, recordModifiers...)
+var OPENPGPKEY = recordBuilder('OPENPGPKEY');
+
+// name, usage, selector, matchingtype, certificate
+var SMIMEA = recordBuilder('SMIMEA', {
+    args: [
+        ['name', _.isString],
+        ['usage', _.isNumber],
+        ['selector', _.isNumber],
+        ['matchingtype', _.isNumber],
+        ['target', _.isString], // recordBuilder needs a "target" argument
+    ],
+    transform: function (record, args, modifiers) {
+        record.name = args.name + '._smimecert';
+        record.smimeausage = args.usage;
+        record.smimeaselector = args.selector;
+        record.smimeamatchingtype = args.matchingtype;
+        record.target = args.target;
+    },
+});
+
 // SOA(name,ns,mbox,refresh,retry,expire,minimum, recordModifiers...)
 var SOA = recordBuilder('SOA', {
     args: [
@@ -604,6 +668,23 @@ var TXT = recordBuilder('TXT', {
     },
 });
 
+var LUA = recordBuilder('LUA', {
+  args: [
+    ['name', _.isString],
+    ['rtype', _.isString],
+    ['target', isStringOrArray],
+  ],
+  transform: function (record, args, modifiers) {
+    record.name = args.name;
+    record.luartype = args.rtype.toUpperCase();
+    if (_.isString(args.target)) {
+      record.target = args.target;
+    } else {
+      record.target = args.target.join('');
+    }
+  },
+});
+
 // Parses coordinates of the form 41°24'12.2"N 2°10'26.5"E
 function parseDMSCoordinatesString(inputString) {
     var lat = inputString.match(/(-?\d+).(\d+).([\d\.]+).?\ ?([NS])/);
@@ -682,10 +763,33 @@ function getENotationInt(x) {
        0cm = 0e0 == 0
     */
     size = x * 100; // get cm value
-    // get the m^e version of size
-    array = size.toExponential(0).split('e+').map(Number);
+
+    // Convert the number to scientific notation
+    var exp = Math.floor(Math.log10(size)); // Get the exponent (base 10)
+    var mantissa = size / Math.pow(10, exp); // Normalize mantissa
+
+    // Normalize the mantissa to fit into 4 bits (between 0 and 15)
+    while (mantissa < 1 && exp > 0) {
+        mantissa *= 10;
+        exp -= 1;
+    }
+
+    /* Four-bit values greater than 9 are undefined, as are values with a base
+    of zero and a non-zero exponent.
+    */
+
+    // Ensure mantissa and exponent are within 4-bit range (0-15) but no greater than 9
+    mantissa = Math.floor(mantissa); // We truncate decimals
+    if (mantissa > 9) {
+        mantissa = 9; // Cap mantissa at 9
+    }
+    if (exp < 0) {
+        exp = 0; // We cap negative exponents at 0
+    } else if (exp > 9) {
+        exp = 9; // Cap exponent at 9
+    }
     // convert it to 4bit:4bit uint8
-    m_e = (array[0] << 4) | array[1];
+    m_e = (mantissa << 4) | (exp & 0xf);
     return m_e;
 }
 
@@ -734,10 +838,30 @@ function locStringBuilder(record, args) {
     }
 
     // handle altitude, size, horizontal precision, vertical precision
-    precisionbuffer = args.alt.toString() + 'm';
-    precisionbuffer += ' ' + args.siz.toString() + 'm';
-    precisionbuffer += ' ' + args.hp.toString() + 'm';
-    precisionbuffer += ' ' + args.vp.toString() + 'm';
+    // alt -100000.00 .. 42849672.95m
+    // size, horizontal precision, vertical precision 0 .. 90000000.00m
+    precisionbuffer =
+        (args.alt < -100000
+            ? -100000
+            : args.alt > 42849672.95
+              ? 42849672.95
+              : args.alt.toString()) + 'm';
+    precisionbuffer +=
+        ' ' +
+        (args.siz > 90000000
+            ? 90000000
+            : args.siz < 0
+              ? 0
+              : args.siz.toString()) +
+        'm';
+    precisionbuffer +=
+        ' ' +
+        (args.hp > 90000000 ? 90000000 : args.hp < 0 ? 0 : args.hp.toString()) +
+        'm';
+    precisionbuffer +=
+        ' ' +
+        (args.vp > 90000000 ? 90000000 : args.vp < 0 ? 0 : args.vp.toString()) +
+        'm';
 
     record.target = nsstring + ewstring + precisionbuffer;
 
@@ -762,23 +886,23 @@ function locDMSBuilder(record, args) {
     // W
     else record.loclongitude = LOCPrimeMeridian - lon;
     // Altitude
-    record.localtitude = (args.alt + LOCAltitudeBase) * 100;
+    record.localtitude = parseInt((args.alt + LOCAltitudeBase) * 100);
+    // 'cast' altitude to fit 'uint32'
+    record.localtitude =
+        record.localtitude > 4294967295
+            ? 4294967295
+            : record.localtitude < 0
+              ? 0
+              : record.localtitude;
     // Size
     record.locsize = getENotationInt(args.siz);
     // Horizontal Precision
     m_e = args.hp;
     record.lochorizpre = getENotationInt(args.hp);
-    // if (m_e != 0) {
-    // } else {
-    //     record.lochorizpre = 22; // 10,000m default
-    // }
+
     // Vertical Precision
     m_e = args.vp;
     record.locvertpre = getENotationInt(args.vp);
-    // if (m_e != 0) {
-    // } else {
-    //     record.lochorizpre = 19; // 10m default
-    // }
 }
 
 // LOC(name,d1,m1,s1,ns,d2,m2,s2,ew,alt,siz,hp,vp, recordModifiers...)
@@ -799,11 +923,69 @@ var LOC = recordBuilder('LOC', {
         ['vp', _.isNumber], // vertical precision
     ],
     transform: function (record, args, modifiers) {
+        validateIntegers(args);
+
         record = locStringBuilder(record, args);
         record = locDMSBuilder(record, args);
     },
 });
 
+// Post-validation function for LOC that checks if degrees and minutes are integers
+function validateIntegers(args) {
+    if (args.d1 % 1 !== 0) {
+        throw (
+            "Degrees N/S shall be an integer: record '" +
+            args.name +
+            "': *" +
+            args.d1 +
+            '*, ' +
+            args.m1 +
+            ', ' +
+            args.s1 +
+            ', ...'
+        );
+    }
+    if (args.m1 % 1 !== 0) {
+        throw (
+            "Minutes N/S shall be an integer: record '" +
+            args.name +
+            "': " +
+            args.d1 +
+            ', *' +
+            args.m1 +
+            '*, ' +
+            args.s1 +
+            ', ...'
+        );
+    }
+    if (args.d2 % 1 !== 0) {
+        throw (
+            "Degrees E/W shall be an integer: record '" +
+            args.name +
+            "': *" +
+            args.d2 +
+            '*, ' +
+            args.m2 +
+            ', ' +
+            args.s2 +
+            ', ...'
+        );
+    }
+    if (args.m2 % 1 !== 0) {
+        throw (
+            "Minutes E/W shall be an integer: record '" +
+            args.name +
+            "': " +
+            args.d2 +
+            ', *' +
+            args.m2 +
+            '*, ' +
+            args.s2 +
+            ', ...'
+        );
+    }
+}
+
 function ConvertDDToDMS(D, longitude) {
     //stackoverflow, baby. do not re-order the rows.
     return {
@@ -929,7 +1111,7 @@ function IGNORE_TARGET(target, rType) {
     return IGNORE('*', rType, target);
 }
 
-// IMPORT_TRANSFORM(translation_table, domain)
+// IMPORT_TRANSFORM(translation_table, domain, ttl)
 var IMPORT_TRANSFORM = recordBuilder('IMPORT_TRANSFORM', {
     args: [['translation_table'], ['domain'], ['ttl', _.isNumber]],
     transform: function (record, args, modifiers) {
@@ -940,6 +1122,23 @@ var IMPORT_TRANSFORM = recordBuilder('IMPORT_TRANSFORM', {
     },
 });
 
+// IMPORT_TRANSFORM_STRIP(translation_table, domain, ttl, suffixstrip)
+var IMPORT_TRANSFORM_STRIP = recordBuilder('IMPORT_TRANSFORM', {
+    args: [
+        ['translation_table'],
+        ['domain'],
+        ['ttl', _.isNumber],
+        ['suffixstrip'],
+    ],
+    transform: function (record, args, modifiers) {
+        record.name = '@';
+        record.target = args.domain;
+        record.meta['transform_table'] = format_tt(args.translation_table);
+        record.ttl = args.ttl;
+        record.meta['transform_suffixstrip'] = args.suffixstrip;
+    },
+});
+
 // PURGE()
 function PURGE(d) {
     d.KeepUnknown = false;
@@ -1084,32 +1283,60 @@ function recordBuilder(type, opts) {
             modifiers.push(arguments[i]);
         }
 
+        // Record which line called this record type.
+        // NB(tlim): Hopefully we can find a better way to do this in the
+        // future. Right now we're faking that there was an error just to parse
+        // out the line number. That's inefficient but I can't find anything better.
+        // This will certainly break if we change to a different Javascript interpreter.
+        // Hopefully any other interpreter will have a better way to do this.
+        var positionLines = new Error().stack.split('\n');
+        var position = positionLines[positionLines.length - 2];
+
         return function (d) {
             var record = {
                 type: type,
                 meta: {},
                 ttl: d.defaultTTL,
+                filepos: position,
             };
 
             opts.applyModifier(record, modifiers);
             opts.transform(record, parsedArgs, modifiers);
 
             // Handle D_EXTEND() with subdomains.
+            // Fix the labels.  (Fixing targets is done in pkg/normalize/validate.go)
             if (
                 d.subdomain &&
+                record.type != 'CF_SINGLE_REDIRECT' &&
                 record.type != 'CF_REDIRECT' &&
                 record.type != 'CF_TEMP_REDIRECT' &&
-                record.type != 'CF_WORKER_ROUTE'
+                record.type != 'CF_WORKER_ROUTE' &&
+                record.type != 'ADGUARDHOME_A_PASSTHROUGH' &&
+                record.type != 'ADGUARDHOME_AAAA_PASSTHROUGH'
             ) {
-                fqdn = [d.subdomain, d.name].join('.');
-
                 record.subdomain = d.subdomain;
+
+                // @ sub dom                  ->   sub sub
+                // one two dom                ->   one.two
+                // 4.3.2.1.in-addr.arpa 4.3   ->   4.3 2.1.in-addr.arpa
+                // 1.2.3.4  sub               ->   1.2.3.4 sub
+
                 if (record.name == '@') {
-                    record.subdomain = d.subdomain;
                     record.name = d.subdomain;
-                } else if (fqdn != record.name && record.type != 'PTR') {
-                    record.subdomain = d.subdomain;
-                    record.name += '.' + d.subdomain;
+                } else if (record.name.match(/^\d+\.\d+\.\d+\.\d+$/)) {
+                    // leave it alone
+                } else if (d.name.endsWith('.ip6.arpa')) {
+                    record.name = d.subdomain;
+                    d.subdomain = undefined;
+                } else if (record.name.endsWith('.in-addr.arpa')) {
+                    if (record.name.endsWith(d.subdomain)) {
+                        record.name = record.name.slice(
+                            0,
+                            -d.subdomain.length - 1
+                        );
+                    }
+                } else {
+                    record.name = record.name + '.' + d.subdomain;
                 }
             }
 
@@ -1253,12 +1480,18 @@ var CF_WORKER_ROUTE = recordBuilder('CF_WORKER_ROUTE', {
     },
 });
 
+var ADGUARDHOME_A_PASSTHROUGH = recordBuilder('ADGUARDHOME_A_PASSTHROUGH');
+
+var ADGUARDHOME_AAAA_PASSTHROUGH = recordBuilder(
+    'ADGUARDHOME_AAAA_PASSTHROUGH'
+);
+
 var URL = recordBuilder('URL');
 var URL301 = recordBuilder('URL301');
 var FRAME = recordBuilder('FRAME');
-var NS1_URLFWD = recordBuilder('NS1_URLFWD');
 var CLOUDNS_WR = recordBuilder('CLOUDNS_WR');
-
+var PORKBUN_URLFWD = recordBuilder('PORKBUN_URLFWD');
+var BUNNY_DNS_RDR = recordBuilder('BUNNY_DNS_RDR');
 // LOC_BUILDER_DD takes an object:
 // label: The DNS label for the LOC record. (default: '@')
 // x: Decimal X coordinate.
@@ -1501,8 +1734,14 @@ function SPF_BUILDER(value) {
 // label: The DNS label for the CAA record. (default: '@')
 // iodef: The contact mail address. (optional)
 // iodef_critical: Boolean if sending report is required/critical. If not supported, certificate should be refused. (optional)
-// issue: List of CAs which are allowed to issue certificates for the domain (creates one record for each).
-// issuewild: Allowed CAs which can issue wildcard certificates for this domain. (creates one record for each)
+// issue: List of CAs which are allowed to issue certificates for the domain (creates one record for each), or the string 'none'.
+// issuewild: List of allowed CAs which can issue wildcard certificates for this domain, or the string 'none'. (creates one record for each)
+// issuevmc: List of allowed CAs which can issue VMC certificates for this domain, or the string 'none'. (creates one record for each)
+// issuemail: List of allowed CAs which can issue email certificates for this domain, or the string 'none'. (creates one record for each)
+// issue_critical: Boolean if issue entries are critical. If not supported, certificate should be refused. (optional)
+// issuewild_critical: Boolean if issuewild entries are critical. If not supported, certificate should be refused. (optional)
+// issuevmc_critical: Boolean if issuevmc entries are critical. If not supported, certificate should be refused. (optional)
+// issuemail_critical: Boolean if issuemail entries are critical. If not supported, certificate should be refused. (optional)
 // ttl: The time for TTL, integer or string. (default: not defined, using DefaultTTL)
 
 function CAA_BUILDER(value) {
@@ -1512,15 +1751,24 @@ function CAA_BUILDER(value) {
 
     if (value.issue && value.issue == 'none') value.issue = [';'];
     if (value.issuewild && value.issuewild == 'none') value.issuewild = [';'];
+    if (value.issuevmc && value.issuevmc == 'none') value.issuevmc = [';'];
+    if (value.issuemail && value.issuemail == 'none') value.issuemail = [';'];
 
     if (
-        (!value.issue && !value.issuewild) ||
+        (!value.issue &&
+            !value.issuewild &&
+            !value.issuevmc &&
+            !value.issuemail) ||
         (value.issue &&
             value.issue.length == 0 &&
             value.issuewild &&
-            value.issuewild.length == 0)
+            value.issuewild.length == 0 &&
+            value.issuevmc &&
+            value.issuevmc.length == 0 &&
+            value.issuemail &&
+            value.issuemail.length == 0)
     ) {
-        throw 'CAA_BUILDER requires at least one entry at issue or issuewild';
+        throw 'CAA_BUILDER requires at least one entry at issue, issuewild, issuevmc or issuemail';
     }
 
     var CAA_TTL = function () {};
@@ -1559,9 +1807,242 @@ function CAA_BUILDER(value) {
             );
     }
 
+    if (value.issuevmc) {
+        var flag = function () {};
+        if (value.issuevmc_critical) {
+            flag = CAA_CRITICAL;
+        }
+        for (var i = 0, len = value.issuevmc.length; i < len; i++)
+            r.push(
+                CAA(value.label, 'issuevmc', value.issuevmc[i], flag, CAA_TTL)
+            );
+    }
+
+    if (value.issuemail) {
+        var flag = function () {};
+        if (value.issuemail_critical) {
+            flag = CAA_CRITICAL;
+        }
+        for (var i = 0, len = value.issuemail.length; i < len; i++)
+            r.push(
+                CAA(value.label, 'issuemail', value.issuemail[i], flag, CAA_TTL)
+            );
+    }
+
     return r;
 }
 
+/**
+ * Encodes a string into DKIM-specific quoted-printable format.
+ *
+ * This function converts characters that are outside the range of printable ASCII
+ * characters, semicolons, DEL, or above ASCII 127 into their quoted-printable
+ * hex representation, prefixed by '='. This encoding is used in DKIM signatures
+ * to handle characters safely.
+ *
+ * @param {string} str - The input string to encode.
+ * @returns {string} The DKIM quoted-printable encoded string.
+ */
+function _encodeDKIMQuotedPrintable(str) {
+    var hexChars = '0123456789ABCDEF'.split('');
+    var result = '';
+
+    for (var i = 0; i < str.length; i++) {
+        var charCode = str.charCodeAt(i);
+        if (
+            charCode < 0x21 ||
+            charCode === 0x3b ||
+            charCode === 0x3d ||
+            charCode === 0x7f ||
+            charCode > 0x7f
+        ) {
+            result +=
+                '=' + hexChars[(charCode >>> 4) & 15] + hexChars[charCode & 15];
+        } else {
+            result += str.charAt(i);
+        }
+    }
+    return result;
+}
+
+/**
+ * Builds a DKIM DNS TXT record according to RFC 6376 its updates
+ * @param {Object} value - Configuration object for the DKIM record.
+ * @param {string} value.selector - The selector subdividing the namespace for the domain. **(Required)**
+ * @param {string} [value.pubkey] - The base64-encoded public key (RSA or Ed25519).
+ *   May be empty for key revocation or non-sending domains.
+ * @param {string} [value.label='@'] - The DNS label for the DKIM record (`[selector]._domainkey` prefix is added).
+ * @param {string} [value.version='DKIM1'] - The DKIM version (`v=` tag). Currently, only `"DKIM1"` is supported.
+ * @param {string|string[]} [value.hashtypes] - Acceptable hash algorithms for signing (`h=` tag).
+ *   - Supported values for RSA: `'sha1'`, `'sha256'`
+ *   - Supported values for Ed25519: `'sha256'`
+ * @param {string} [value.keytype='rsa'] - Key algorithm type (`k=` tag).
+ *   - Supported values: `'rsa'`, `'ed25519'`
+ * @param {string|string[]} [value.servicetypes] - Service types using this key (`s=` tag).
+ *   - Supported values: `'*'`, `'email'`
+ *   - `'*'` allows all services; `'email'` restricts usage to email only.
+ * @param {string|string[]} [value.flags] - Flags modifying selector interpretation (`t=` tag).
+ *   - Supported values: `'y'` (testing mode), `'s'` (subdomain restriction)
+ * @param {string} [value.note] - Human-readable note for the record (`n=` tag).
+ * @param {number} [value.ttl] - DNS TTL value in seconds.
+ *
+ * @throws {Error} If a required field is missing or a value is invalid.
+ * @returns {Object} DNS TXT record entries for DKIM
+ */
+
+function DKIM_BUILDER(value) {
+    value = value || {};
+
+    // ========================================
+    // PHASE 1: NORMALIZATION
+    // ========================================
+
+    // Apply defaults using _.defaults()
+    value = _.defaults(value, {
+        version: 'DKIM1',
+        pubkey: '',
+        label: '@',
+    });
+
+    // Normalize string|array fields to always be arrays
+    if (!_.isEmpty(value.hashtypes)) {
+        value.hashtypes = _.isString(value.hashtypes)
+            ? [value.hashtypes]
+            : value.hashtypes;
+    }
+
+    if (!_.isEmpty(value.servicetypes)) {
+        value.servicetypes = _.isString(value.servicetypes)
+            ? [value.servicetypes]
+            : value.servicetypes;
+    }
+
+    if (!_.isEmpty(value.flags)) {
+        value.flags = _.isString(value.flags) ? [value.flags] : value.flags;
+    }
+
+    // ========================================
+    // PHASE 2: VALIDATION (Fail Fast)
+    // ========================================
+
+    // Static allowed values
+    var ALLOWED_VERSIONS = ['DKIM1'];
+    var ALLOWED_KEYTYPES = ['rsa', 'ed25519'];
+    var ALLOWED_HASHTYPES = {
+        rsa: ['sha1', 'sha256'],
+        ed25519: ['sha256'],
+    };
+    var ALLOWED_SERVICETYPES = ['*', 'email'];
+    var ALLOWED_FLAGS = ['y', 's'];
+
+    // Required fields
+    if (_.isEmpty(value.selector)) {
+        throw 'DKIM_BUILDER selector cannot be empty';
+    }
+
+    // Version validation
+    if (!_.contains(ALLOWED_VERSIONS, value.version)) {
+        throw (
+            'DKIM_BUILDER version must be one of: ' +
+            ALLOWED_VERSIONS.join(', ')
+        );
+    }
+
+    // Keytype validation
+    if (
+        !_.isEmpty(value.keytype) &&
+        !_.contains(ALLOWED_KEYTYPES, value.keytype)
+    ) {
+        throw (
+            'DKIM_BUILDER keytype must be one of: ' +
+            ALLOWED_KEYTYPES.join(', ') +
+            ', ' +
+            value.keytype +
+            ' given'
+        );
+    }
+
+    // Hashtypes validation (now always an array after normalization)
+    if (!_.isEmpty(value.hashtypes)) {
+        var allowedHashtypes = ALLOWED_HASHTYPES[value.keytype || 'rsa'];
+        var invalidHashtypes = _.difference(value.hashtypes, allowedHashtypes);
+        if (invalidHashtypes.length > 0) {
+            throw (
+                'DKIM_BUILDER hashtypes for ' +
+                value.keytype +
+                ' must be one of: ' +
+                allowedHashtypes.join(', ')
+            );
+        }
+    }
+
+    // Servicetypes validation (now always an array after normalization)
+    if (!_.isEmpty(value.servicetypes)) {
+        var invalidServicetypes = _.difference(
+            value.servicetypes,
+            ALLOWED_SERVICETYPES
+        );
+        if (invalidServicetypes.length > 0) {
+            throw (
+                'DKIM_BUILDER servicetypes must be one of: ' +
+                ALLOWED_SERVICETYPES.join(', ')
+            );
+        }
+    }
+
+    // Flags validation (now always an array after normalization)
+    if (!_.isEmpty(value.flags)) {
+        var invalidFlags = _.difference(value.flags, ALLOWED_FLAGS);
+        if (invalidFlags.length > 0) {
+            throw (
+                'DKIM_BUILDER flags must be one of: ' + ALLOWED_FLAGS.join(', ')
+            );
+        }
+    }
+
+    // ========================================
+    // PHASE 3: BUILD OUTPUT
+    // ========================================
+
+    // Build record RFC 6376 order: v=, h=, k=, n=, p=, s=, t=
+    var record = [];
+
+    record.push('v=' + value.version);
+
+    if (value.hashtypes) {
+        record.push('h=' + value.hashtypes.join(':'));
+    }
+
+    if (value.keytype) {
+        record.push('k=' + value.keytype);
+    }
+
+    if (!_.isEmpty(value.note)) {
+        record.push('n=' + _encodeDKIMQuotedPrintable(value.note));
+    }
+
+    record.push('p=' + value.pubkey);
+
+    if (value.servicetypes) {
+        record.push('s=' + value.servicetypes.join(':'));
+    }
+
+    if (value.flags) {
+        record.push('t=' + value.flags.join(':'));
+    }
+
+    // Build label
+    var fullLabel = value.selector + '._domainkey';
+    if (value.label !== '@') {
+        fullLabel += '.' + value.label;
+    }
+
+    // Handle TTL
+    var DKIM_TTL = value.ttl ? TTL(value.ttl) : function () {};
+
+    return TXT(fullLabel, record.join('; '), DKIM_TTL);
+}
+
 // DMARC_BUILDER takes an object:
 // label: The DNS label for the DMARC record (_dmarc prefix is added; default: '@')
 // version: The DMARC version, by default DMARC1 (optional)
@@ -1746,7 +2227,7 @@ function M365_BUILDER(name, value) {
             );
         }
 
-        value.domainGUID = name.replace('.', '-');
+        value.domainGUID = name.replace(/\./g, '-');
     }
 
     if (value.dkim && !value.initialDomain) {
@@ -1915,3 +2396,70 @@ var DISABLE_REPEATED_DOMAIN_CHECK = { skip_fqdn_check: 'true' };
 // D("bar.com", ...
 //     A("foo.bar.com", "10.1.1.1", DISABLE_REPEATED_DOMAIN_CHECK),
 // )
+
+// ============================================================
+
+// RTYPES
+
+// Background:
+// Old-style commands: Commands built using recordBuild() are the original
+// style.  They all validation and pre-processing here in helpers.js. This
+// seemed like a good idea at the time, but in hindsight it puts a burden on the
+// developer to know both Javascript and go.
+
+// New-style commands: Command built using rawrecordBuilder() are the new style.
+// They simply pack up the arguments listed in dnsconfig.js and store them in
+// .rawrecords. This is passed to the Go code, which is responsible for all
+// validation, pre-processing, etc.  The benefit is this minimizes the need for
+// Javascript knowledge, and allows us to use the testing platform build into
+// Go.
+
+function rawrecordBuilder(type) {
+    return function () {
+        // Copy the raw args:
+        var rawArgs = [];
+        for (var i = 0; i < arguments.length; i++) {
+            rawArgs.push(arguments[i]);
+        }
+
+        return function (d) {
+            var record = {
+                type: type,
+            };
+
+            // Process the args: Functions are executed, objects are assumed to
+            // be meta and stored, strings are assumed to be args and are
+            // stored.
+            // NB(tlim): Allowing for the intermixing of args and meta seems
+            // bad.  It might be better to simply preserve the first n items as
+            // args, then assume the rest are metas. That would be more similar
+            // to the old style functions. However at this time I can't think of
+            // a reason this isn't sufficient.
+            var processedArgs = [];
+            var processedMetas = [];
+            for (var i = 0; i < rawArgs.length; i++) {
+                var r = rawArgs[i];
+                if (_.isFunction(r)) {
+                    r(record);
+                } else if (_.isObject(r)) {
+                    processedMetas.push(r);
+                } else {
+                    processedArgs.push(r);
+                }
+            }
+            // Store the processed args.
+            record.args = processedArgs;
+            record.metas = processedMetas;
+
+            // Add this raw record to the list of records.
+            d.rawrecords.push(record);
+
+            return record;
+        };
+    };
+}
+
+// PLEASE KEEP THIS LIST ALPHABETICAL!
+
+// CLOUDFLAREAPI:
+var CF_SINGLE_REDIRECT = rawrecordBuilder('CLOUDFLAREAPI_SINGLE_REDIRECT');
