@@ -13,7 +13,7 @@ func splitChunks(buf string, lim int) []string {
 		chunks = append(chunks, chunk)
 	}
 	if len(buf) > 0 {
-		chunks = append(chunks, buf[:])
+		chunks = append(chunks, buf)
 	}
 	return chunks
 }
