@@ -15,18 +15,17 @@ import (
 	"crypto/tls"
 	"encoding/base64"
 	"encoding/json"
+	"errors"
 	"fmt"
-	"math"
-	"math/rand"
 	"net"
 	"strings"
+	"sync"
 	"time"
 
 	"github.com/StackExchange/dnscontrol/v4/models"
 	"github.com/StackExchange/dnscontrol/v4/pkg/diff2"
 	"github.com/StackExchange/dnscontrol/v4/pkg/printer"
 	"github.com/StackExchange/dnscontrol/v4/providers"
-	"github.com/fatih/color"
 	"github.com/miekg/dns"
 )
 
@@ -40,35 +39,47 @@ var features = providers.DocumentationNotes{
 	// The default for unlisted capabilities is 'Cannot'.
 	// See providers/capabilities.go for the entire list of capabilities.
 	providers.CanAutoDNSSEC:          providers.Can("Just warn when DNSSEC is requested but no RRSIG is found in the AXFR or warn when DNSSEC is not requested but RRSIG are found in the AXFR."),
-	providers.CanGetZones:            providers.Cannot(),
-	providers.CanConcur:              providers.Cannot(),
+	providers.CanConcur:              providers.Can(),
 	providers.CanUseCAA:              providers.Can(),
 	providers.CanUseDHCID:            providers.Can(),
+	providers.CanUseDNAME:            providers.Can(),
+	providers.CanUseDS:               providers.Can(),
 	providers.CanUseHTTPS:            providers.Can(),
-	providers.CanUseLOC:              providers.Unimplemented(),
+	providers.CanUseLOC:              providers.Can(),
 	providers.CanUseNAPTR:            providers.Can(),
 	providers.CanUsePTR:              providers.Can(),
 	providers.CanUseSRV:              providers.Can(),
 	providers.CanUseSSHFP:            providers.Can(),
 	providers.CanUseSVCB:             providers.Can(),
 	providers.CanUseTLSA:             providers.Can(),
-	providers.DocCreateDomains:       providers.Cannot(),
 	providers.DocDualHost:            providers.Cannot(),
 	providers.DocOfficiallySupported: providers.Cannot(),
+	// Possible to support via catalog zones (RFC 9432), but those are not
+	// directly supported by DNSControl right now (although nothing is stopping
+	// you from manually updating a catalog zone using DNSControl if you wish).
+	providers.CanGetZones:      providers.Cannot(),
+	providers.DocCreateDomains: providers.Cannot(),
+	// Not a valid RR type, so impossible to encode in an RFC-compliant DNS
+	// packet.
+	providers.CanUseAlias: providers.Cannot(),
+	// These are both supported by RFC 2136 (DDNS), but neither work with
+	// DNSControl right now.
+	providers.CanUseSOA:    providers.Cannot(),
+	providers.CanUseDNSKEY: providers.Cannot(),
 }
 
 // axfrddnsProvider stores the client info for the provider.
 type axfrddnsProvider struct {
-	rand                *rand.Rand
-	master              string
-	updateMode          string
-	transferServer      string
-	transferMode        string
-	nameservers         []*models.Nameserver
-	transferKey         *Key
-	updateKey           *Key
-	hasDnssecRecords    bool
-	serverHasBuggyCNAME bool
+	master         string
+	updateMode     string
+	transferServer string
+	transferMode   string
+	nameservers    []*models.Nameserver
+	transferKey    *Key
+	updateKey      *Key
+
+	mu               sync.Mutex // protects hasDnssecRecords during concurrent collection.
+	hasDnssecRecords map[string]bool
 }
 
 func initAxfrDdns(config map[string]string, providermeta json.RawMessage) (providers.DNSServiceProvider, error) {
@@ -76,7 +87,7 @@ func initAxfrDdns(config map[string]string, providermeta json.RawMessage) (provi
 	// providermeta -- the json blob from NewReq('name', 'TYPE', providermeta)
 	var err error
 	api := &axfrddnsProvider{
-		rand: rand.New(rand.NewSource(int64(time.Now().Nanosecond()))),
+		hasDnssecRecords: map[string]bool{},
 	}
 	param := &Param{}
 	if len(providermeta) != 0 {
@@ -98,8 +109,7 @@ func initAxfrDdns(config map[string]string, providermeta json.RawMessage) (provi
 	}
 	if config["update-mode"] != "" {
 		switch config["update-mode"] {
-		case "tcp",
-			"tcp-tls":
+		case "tcp", "tcp-tls":
 			api.updateMode = config["update-mode"]
 		case "udp":
 			api.updateMode = ""
@@ -107,12 +117,11 @@ func initAxfrDdns(config map[string]string, providermeta json.RawMessage) (provi
 			printer.Printf("[Warning] AXFRDDNS: Unknown update-mode in `creds.json` (%s)\n", config["update-mode"])
 		}
 	} else {
-		api.updateMode = ""
+		api.updateMode = "tcp"
 	}
 	if config["transfer-mode"] != "" {
 		switch config["transfer-mode"] {
-		case "tcp",
-			"tcp-tls":
+		case "tcp", "tcp-tls":
 			api.transferMode = config["transfer-mode"]
 		default:
 			printer.Printf("[Warning] AXFRDDNS: Unknown transfer-mode in `creds.json` (%s)\n", config["transfer-mode"])
@@ -128,7 +137,7 @@ func initAxfrDdns(config map[string]string, providermeta json.RawMessage) (provi
 	} else if len(api.nameservers) != 0 {
 		api.master = api.nameservers[0].Name + ":53"
 	} else {
-		return nil, fmt.Errorf("nameservers list is empty: creds.json needs a default `nameservers` or an explicit `master`")
+		return nil, errors.New("nameservers list is empty: creds.json needs a default `nameservers` or an explicit `master`")
 	}
 	if config["transfer-server"] != "" {
 		api.transferServer = config["transfer-server"]
@@ -148,9 +157,7 @@ func initAxfrDdns(config map[string]string, providermeta json.RawMessage) (provi
 	}
 	switch strings.ToLower(strings.TrimSpace(config["buggy-cname"])) {
 	case "yes", "true":
-		api.serverHasBuggyCNAME = true
-	default:
-		api.serverHasBuggyCNAME = false
+		printer.Warnf("'buggy-cname' is deprecated as it is no longer necessary.\n")
 	}
 	for key := range config {
 		switch key {
@@ -161,6 +168,7 @@ func initAxfrDdns(config map[string]string, providermeta json.RawMessage) (provi
 			"transfer-server",
 			"update-mode",
 			"transfer-mode",
+			"buggy-cname",
 			"domain",
 			"TYPE":
 			continue
@@ -172,11 +180,14 @@ func initAxfrDdns(config map[string]string, providermeta json.RawMessage) (provi
 }
 
 func init() {
+	const providerName = "AXFRDDNS"
+	const providerMaintainer = "@hnrgrgr"
 	fns := providers.DspFuncs{
 		Initializer:   initAxfrDdns,
 		RecordAuditor: AuditRecords,
 	}
-	providers.RegisterDomainServiceProviderType("AXFRDDNS", fns, features)
+	providers.RegisterDomainServiceProviderType(providerName, fns, features)
+	providers.RegisterMaintainer(providerName, providerMaintainer)
 }
 
 // Param is used to decode extra parameters sent to provider.
@@ -205,8 +216,12 @@ func readKey(raw string, kind string) (*Key, error) {
 		algo = dns.HmacMD5
 	case "hmac-sha1", "sha1":
 		algo = dns.HmacSHA1
+	case "hmac-sha224", "sha224":
+		algo = dns.HmacSHA224
 	case "hmac-sha256", "sha256":
 		algo = dns.HmacSHA256
+	case "hmac-sha384", "sha384":
+		algo = dns.HmacSHA384
 	case "hmac-sha512", "sha512":
 		algo = dns.HmacSHA512
 	default:
@@ -226,8 +241,8 @@ func (c *axfrddnsProvider) GetNameservers(domain string) ([]*models.Nameserver,
 }
 
 func (c *axfrddnsProvider) getAxfrConnection() (*dns.Transfer, error) {
-	var con net.Conn = nil
-	var err error = nil
+	var con net.Conn
+	var err error
 	if c.transferMode == "tcp-tls" {
 		con, err = tls.Dial("tcp", c.transferServer, &tls.Config{})
 	} else {
@@ -254,8 +269,7 @@ func (c *axfrddnsProvider) FetchZoneRecords(domain string) ([]dns.RR, error) {
 	request.SetAxfr(domain + ".")
 
 	if c.transferKey != nil {
-		transfer.TsigSecret =
-			map[string]string{c.transferKey.id: c.transferKey.secret}
+		transfer.TsigSecret = map[string]string{c.transferKey.id: c.transferKey.secret}
 		request.SetTsig(c.transferKey.id, c.transferKey.algo, 300, time.Now().Unix())
 		if c.transferKey.algo == dns.HmacMD5 {
 			transfer.TsigProvider = md5Provider(c.transferKey.secret)
@@ -280,12 +294,10 @@ func (c *axfrddnsProvider) FetchZoneRecords(domain string) ([]dns.RR, error) {
 		rawRecords = append(rawRecords, msg.RR...)
 	}
 	return rawRecords, nil
-
 }
 
 // GetZoneRecords gets the records of a zone and returns them in RecordConfig format.
 func (c *axfrddnsProvider) GetZoneRecords(domain string, meta map[string]string) (models.Records, error) {
-
 	rawRecords, err := c.FetchZoneRecords(domain)
 	if err != nil {
 		return nil, err
@@ -302,6 +314,7 @@ func (c *axfrddnsProvider) GetZoneRecords(domain string, meta map[string]string)
 			dns.TypeNSEC,
 			dns.TypeNSEC3,
 			dns.TypeNSEC3PARAM,
+			dns.TypeZONEMD,
 			65534:
 			// Ignoring DNSSec RRs, but replacing it with a single
 			// "TXT" placeholder
@@ -336,25 +349,25 @@ func (c *axfrddnsProvider) GetZoneRecords(domain string, meta map[string]string)
 		foundRecords = append(foundRecords, foundDNSSecRecords)
 	}
 
-	c.hasDnssecRecords = false
 	if len(foundRecords) >= 1 {
 		last := foundRecords[len(foundRecords)-1]
 		if last.Type == "TXT" &&
 			last.Name == dnssecDummyLabel &&
 			last.GetTargetTXTSegmentCount() == 1 &&
 			last.GetTargetTXTSegmented()[0] == dnssecDummyTxt {
-			c.hasDnssecRecords = true
+			c.mu.Lock()
+			c.hasDnssecRecords[domain] = true
+			c.mu.Unlock()
 			foundRecords = foundRecords[0:(len(foundRecords) - 1)]
 		}
 	}
 
 	return foundRecords, nil
-
 }
 
 // BuildCorrection return a Correction for a given set of DDNS update and the corresponding message.
-func (c *axfrddnsProvider) BuildCorrection(dc *models.DomainConfig, msgs []string, update *dns.Msg) *models.Correction {
-	if update == nil {
+func (c *axfrddnsProvider) BuildCorrection(dc *models.DomainConfig, msgs []string, updates []*dns.Msg) *models.Correction {
+	if updates == nil {
 		return &models.Correction{
 			Msg: fmt.Sprintf("DDNS UPDATES to '%s' (primary master: '%s'). Changes:\n%s", dc.Name, c.master, strings.Join(msgs, "\n")),
 		}
@@ -362,27 +375,28 @@ func (c *axfrddnsProvider) BuildCorrection(dc *models.DomainConfig, msgs []strin
 	return &models.Correction{
 		Msg: fmt.Sprintf("DDNS UPDATES to '%s' (primary master: '%s'). Changes:\n%s", dc.Name, c.master, strings.Join(msgs, "\n")),
 		F: func() error {
-
-			client := new(dns.Client)
-			client.Net = c.updateMode
-			client.Timeout = dnsTimeout
-			if c.updateKey != nil {
-				client.TsigSecret =
-					map[string]string{c.updateKey.id: c.updateKey.secret}
-				update.SetTsig(c.updateKey.id, c.updateKey.algo, 300, time.Now().Unix())
-				if c.updateKey.algo == dns.HmacMD5 {
-					client.TsigProvider = md5Provider(c.updateKey.secret)
+			for _, update := range updates {
+				update.Compress = true
+				client := new(dns.Client)
+				client.Net = c.updateMode
+				client.Timeout = dnsTimeout
+				if c.updateKey != nil {
+					client.TsigSecret = map[string]string{c.updateKey.id: c.updateKey.secret}
+					update.SetTsig(c.updateKey.id, c.updateKey.algo, 300, time.Now().Unix())
+					if c.updateKey.algo == dns.HmacMD5 {
+						client.TsigProvider = md5Provider(c.updateKey.secret)
+					}
 				}
-			}
 
-			msg, _, err := client.Exchange(update, c.master)
-			if err != nil {
-				return err
-			}
-			if msg.MsgHdr.Rcode != 0 {
-				return fmt.Errorf("[Error] AXFRDDNS: nameserver refused to update the zone: %s (%d)",
-					dns.RcodeToString[msg.MsgHdr.Rcode],
-					msg.MsgHdr.Rcode)
+				msg, _, err := client.Exchange(update, c.master)
+				if err != nil {
+					return err
+				}
+				if msg.MsgHdr.Rcode != 0 {
+					return fmt.Errorf("[Error] AXFRDDNS: nameserver refused to update the zone: %s (%d)",
+						dns.RcodeToString[msg.MsgHdr.Rcode],
+						msg.MsgHdr.Rcode)
+				}
 			}
 
 			return nil
@@ -390,19 +404,6 @@ func (c *axfrddnsProvider) BuildCorrection(dc *models.DomainConfig, msgs []strin
 	}
 }
 
-// hasDeletionForName returns true if there exist a corrections for [name] which is a deletion
-func hasDeletionForName(changes diff2.ChangeList, name string) bool {
-	for _, change := range changes {
-		switch change.Type {
-		case diff2.DELETE:
-			if change.Old[0].Name == name {
-				return true
-			}
-		}
-	}
-	return false
-}
-
 // hasNSDeletion returns true if there exist a correction that deletes or changes an NS record
 func hasNSDeletion(changes diff2.ChangeList) bool {
 	for _, change := range changes {
@@ -423,19 +424,21 @@ func hasNSDeletion(changes diff2.ChangeList) bool {
 }
 
 // GetZoneRecordsCorrections returns a list of corrections that will turn existing records into dc.Records.
-func (c *axfrddnsProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, foundRecords models.Records) ([]*models.Correction, error) {
+func (c *axfrddnsProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, foundRecords models.Records) ([]*models.Correction, int, error) {
 	// Ignoring the SOA, others providers don't manage it either.
 	if len(foundRecords) >= 1 && foundRecords[0].Type == "SOA" {
 		foundRecords = foundRecords[1:]
 	}
 
 	// TODO(tlim): This check should be done on all providers. Move to the global validation code.
-	if dc.AutoDNSSEC == "on" && !c.hasDnssecRecords {
-		printer.Printf("Warning: AUTODNSSEC is enabled, but no DNSKEY or RRSIG record was found in the AXFR answer!\n")
+	c.mu.Lock()
+	if dc.AutoDNSSEC == "on" && !c.hasDnssecRecords[dc.Name] {
+		printer.Printf("Warning: AUTODNSSEC is enabled for %s, but no DNSKEY or RRSIG record was found in the AXFR answer!\n", dc.Name)
 	}
-	if dc.AutoDNSSEC == "off" && c.hasDnssecRecords {
-		printer.Printf("Warning: AUTODNSSEC is disabled, but DNSKEY or RRSIG records were found in the AXFR answer!\n")
+	if dc.AutoDNSSEC == "off" && c.hasDnssecRecords[dc.Name] {
+		printer.Printf("Warning: AUTODNSSEC is disabled for %s, but DNSKEY or RRSIG records were found in the AXFR answer!\n", dc.Name)
 	}
+	c.mu.Unlock()
 
 	// An RFC2136-compliant server must silently ignore an
 	// update that inserts a non-CNAME RRset when a CNAME RR
@@ -452,37 +455,33 @@ func (c *axfrddnsProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, fo
 
 	var msgs []string
 	var reports []string
-	var msgs2 []string
-	update := new(dns.Msg)
-	update.SetUpdate(dc.Name + ".")
-	update.Id = uint16(c.rand.Intn(math.MaxUint16))
-	update2 := new(dns.Msg)
-	update2.SetUpdate(dc.Name + ".")
-	update2.Id = uint16(c.rand.Intn(math.MaxUint16))
-	hasTwoCorrections := false
+	updates := []*dns.Msg{}
 
-	dummyNs1, err := dns.NewRR(dc.Name + ". IN NS 255.255.255.255")
+	dummyNs1, err := dns.NewRR(dc.Name + ". IN NS dnscontrol.invalid.")
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
-	dummyNs2, err := dns.NewRR(dc.Name + ". IN NS 255.255.255.255")
+	dummyNs2, err := dns.NewRR(dc.Name + ". IN NS dnscontrol.invalid.")
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 
-	changes, err := diff2.ByRecord(foundRecords, dc, nil)
+	changes, actualChangeCount, err := diff2.ByRecord(foundRecords, dc, nil)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 	if changes == nil {
-		return nil, nil
+		return nil, 0, nil
 	}
 
+	update := new(dns.Msg)
+	update.SetUpdate(dc.Name + ".")
+
 	// A DNS server should silently ignore a DDNS update that removes
 	// the last NS record of a zone. Since modifying a record is
 	// implemented by successively a deletion of the old record and an
 	// insertion of the new one, then modifying all the NS record of a
-	// zone might will fail (even if the the deletion and insertion
+	// zone might will fail (even if the deletion and insertion
 	// are grouped in a single batched update).
 	//
 	// To avoid this case, we will first insert a dummy NS record,
@@ -496,53 +495,73 @@ func (c *axfrddnsProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, fo
 		update.Insert([]dns.RR{dummyNs1})
 	}
 
+	i := 1
+	appendFinalUpdate := true
+
 	for _, change := range changes {
 		switch change.Type {
 		case diff2.DELETE:
 			msgs = append(msgs, change.Msgs[0])
-			update.Remove([]dns.RR{change.Old[0].ToRR()})
+			// It's semantically invalid for any RRs to exist alongside a
+			// CNAME RR
+			if change.Old[0].Type == "CNAME" {
+				update.RemoveName([]dns.RR{change.Old[0].ToRR()})
+			} else {
+				update.Remove([]dns.RR{change.Old[0].ToRR()})
+			}
 		case diff2.CREATE:
-			if c.serverHasBuggyCNAME &&
-				change.New[0].Type == "CNAME" &&
-				hasDeletionForName(changes, change.New[0].Name) {
-				hasTwoCorrections = true
-				msgs2 = append(msgs2, change.Msgs[0])
-				update2.Insert([]dns.RR{change.New[0].ToRR()})
-			} else {
-				msgs = append(msgs, change.Msgs[0])
-				update.Insert([]dns.RR{change.New[0].ToRR()})
+			msgs = append(msgs, change.Msgs[0])
+			// It's semantically invalid for any RRs to exist alongside a
+			// CNAME RR
+			if change.New[0].Type == "CNAME" {
+				update.RemoveName([]dns.RR{change.New[0].ToRR()})
 			}
+			update.Insert([]dns.RR{change.New[0].ToRR()})
 		case diff2.CHANGE:
-			if c.serverHasBuggyCNAME && change.New[0].Type == "CNAME" {
-				msgs = append(msgs, change.Msgs[0]+color.RedString(" (delete)"))
-				update.Remove([]dns.RR{change.Old[0].ToRR()})
-				hasTwoCorrections = true
-				msgs2 = append(msgs2, change.Msgs[0]+color.GreenString(" (create)"))
-				update2.Insert([]dns.RR{change.New[0].ToRR()})
+			msgs = append(msgs, change.Msgs[0])
+			// It's semantically invalid for any RRs to exist alongside a
+			// CNAME RR
+			if (change.New[0].Type == "CNAME") || (change.Old[0].Type == "CNAME") {
+				update.RemoveName([]dns.RR{change.Old[0].ToRR()})
 			} else {
-				msgs = append(msgs, change.Msgs[0])
 				update.Remove([]dns.RR{change.Old[0].ToRR()})
-				update.Insert([]dns.RR{change.New[0].ToRR()})
 			}
+			update.Insert([]dns.RR{change.New[0].ToRR()})
 		case diff2.REPORT:
 			reports = append(reports, change.Msgs...)
 		}
+
+		// Chunk packets that exceed 2^14 = 16 KiB.
+		// A single DNS RR can theoretically reach 64 KiB, the total packet limit.
+		// This is a compromise, succeeding whenever RRs are not bigger than about 64 KiB - 16 KiB = 48 KiB.
+		if update.Len() >= 2<<13 {
+			updates = append(updates, update)
+			update = new(dns.Msg)
+			update.SetUpdate(dc.Name + ".")
+			appendFinalUpdate = false
+			i = 1
+		} else {
+			appendFinalUpdate = true
+			i++
+		}
 	}
 
 	if hasNSDeletion {
 		update.Remove([]dns.RR{dummyNs2})
+		appendFinalUpdate = true
+	}
+
+	if appendFinalUpdate {
+		updates = append(updates, update)
 	}
 
 	returnValue := []*models.Correction{}
 
 	if len(msgs) > 0 {
-		returnValue = append(returnValue, c.BuildCorrection(dc, msgs, update))
-	}
-	if hasTwoCorrections && len(msgs2) > 0 {
-		returnValue = append(returnValue, c.BuildCorrection(dc, msgs2, update2))
+		returnValue = append(returnValue, c.BuildCorrection(dc, msgs, updates))
 	}
 	if len(reports) > 0 {
 		returnValue = append(returnValue, c.BuildCorrection(dc, reports, nil))
 	}
-	return returnValue, nil
+	return returnValue, actualChangeCount, nil
 }
