@@ -3,7 +3,10 @@ package azuredns
 import (
 	"context"
 	"encoding/json"
+	"errors"
 	"fmt"
+	"net/http"
+	"slices"
 	"strings"
 	"time"
 
@@ -11,6 +14,7 @@ import (
 	aauth "github.com/Azure/azure-sdk-for-go/sdk/azidentity"
 	adns "github.com/Azure/azure-sdk-for-go/sdk/resourcemanager/dns/armdns"
 	"github.com/Azure/go-autorest/autorest/to"
+
 	"github.com/StackExchange/dnscontrol/v4/models"
 	"github.com/StackExchange/dnscontrol/v4/pkg/diff2"
 	"github.com/StackExchange/dnscontrol/v4/pkg/printer"
@@ -25,24 +29,53 @@ type azurednsProvider struct {
 	subscriptionID *string
 }
 
+// Modified `newAzureDNSDsp` to maintain backward compatibility with the new OIDC support.
 func newAzureDNSDsp(conf map[string]string, metadata json.RawMessage) (providers.DNSServiceProvider, error) {
 	return newAzureDNS(conf, metadata)
 }
 
+// Updated function to prioritize DefaultAzureCredential and fallback to OIDC or client-secret-based methods.
 func newAzureDNS(m map[string]string, _ json.RawMessage) (*azurednsProvider, error) {
 	subID, rg := m["SubscriptionID"], m["ResourceGroup"]
 	clientID, clientSecret, tenantID := m["ClientID"], m["ClientSecret"], m["TenantID"]
-	credential, authErr := aauth.NewClientSecretCredential(tenantID, clientID, clientSecret, nil)
-	if authErr != nil {
-		return nil, authErr
+	useOIDC := m["UseOIDC"] == "true"
+
+	var credential azcore.TokenCredential
+	var authErr error
+
+	// Authentication Logic
+	if useOIDC {
+		// OIDC Authentication with `InteractiveBrowserCredential`
+		oidcCredentialOpts := aauth.InteractiveBrowserCredentialOptions{
+			TenantID: tenantID,
+		}
+		credential, authErr = aauth.NewInteractiveBrowserCredential(&oidcCredentialOpts)
+		if authErr != nil {
+			return nil, fmt.Errorf("failed to create OIDC credential: %w", authErr)
+		}
+	} else if clientID != "" && clientSecret != "" {
+		// Client ID and Secret-based Authentication
+		credential, authErr = aauth.NewClientSecretCredential(tenantID, clientID, clientSecret, nil)
+		if authErr != nil {
+			return nil, fmt.Errorf("failed to create Client Secret credential: %w", authErr)
+		}
+	} else {
+		// Default Azure Credential as the default mechanism
+		credential, authErr = aauth.NewDefaultAzureCredential(nil)
+		if authErr != nil {
+			return nil, fmt.Errorf("failed to create Default Azure credential: %w", authErr)
+		}
 	}
-	zonesClient, zoneErr := adns.NewZonesClient(subID, credential, nil)
-	if zoneErr != nil {
-		return nil, zoneErr
+
+	// Create DNS clients using the selected credential
+	zonesClient, err := adns.NewZonesClient(subID, credential, nil)
+	if err != nil {
+		return nil, fmt.Errorf("failed to create zones client: %w", err)
 	}
-	recordsClient, recordErr := adns.NewRecordSetsClient(subID, credential, nil)
-	if recordErr != nil {
-		return nil, recordErr
+
+	recordsClient, err := adns.NewRecordSetsClient(subID, credential, nil)
+	if err != nil {
+		return nil, fmt.Errorf("failed to create records client: %w", err)
 	}
 
 	api := &azurednsProvider{
@@ -51,10 +84,12 @@ func newAzureDNS(m map[string]string, _ json.RawMessage) (*azurednsProvider, err
 		resourceGroup:  to.StringPtr(rg),
 		subscriptionID: to.StringPtr(subID),
 	}
-	err := api.getZones()
-	if err != nil {
+
+	// Initialize zones
+	if err := api.getZones(); err != nil {
 		return nil, err
 	}
+
 	return api, nil
 }
 
@@ -78,12 +113,15 @@ var features = providers.DocumentationNotes{
 }
 
 func init() {
+	const providerName = "AZURE_DNS"
+	const providerMaintainer = "@vatsalyagoel"
 	fns := providers.DspFuncs{
 		Initializer:   newAzureDNSDsp,
 		RecordAuditor: AuditRecords,
 	}
-	providers.RegisterDomainServiceProviderType("AZURE_DNS", fns, features)
-	providers.RegisterCustomRecordType("AZURE_ALIAS", "AZURE_DNS", "")
+	providers.RegisterDomainServiceProviderType(providerName, fns, features)
+	providers.RegisterCustomRecordType("AZURE_ALIAS", providerName, "")
+	providers.RegisterMaintainer(providerName, providerMaintainer)
 }
 
 func (a *azurednsProvider) getExistingZones() ([]*adns.Zone, error) {
@@ -137,6 +175,13 @@ func (a *azurednsProvider) GetNameservers(domain string) ([]*models.Nameserver,
 		for _, ns := range zone.Properties.NameServers {
 			nss = append(nss, *ns)
 		}
+
+		nonDefaultNss, err := a.getNameNonDefaultNameServers(domain, nss)
+		if err != nil {
+			return nil, err
+		}
+
+		nss = append(nss, nonDefaultNss...)
 	}
 
 	return models.ToNameserversStripTD(nss)
@@ -157,6 +202,52 @@ func (a *azurednsProvider) ListZones() ([]string, error) {
 	return zones, nil
 }
 
+func (a *azurednsProvider) getNameNonDefaultNameServers(domain string, nss []string) ([]string, error) {
+	zone, ok := a.zones[domain]
+	if !ok {
+		return nil, errNoExist{domain}
+	}
+	zoneName := *zone.Name
+	var nameServers []string
+	ctx, cancel := context.WithTimeout(context.Background(), 6000*time.Second)
+	defer cancel()
+	recordsPager := a.recordsClient.NewListByTypePager(*a.resourceGroup, zoneName, "NS", nil)
+
+	for recordsPager.More() {
+		waitTime := 1
+	retry:
+		nextResult, recordsErr := recordsPager.NextPage(ctx)
+
+		if recordsErr != nil {
+			err := recordsErr
+			var e *azcore.ResponseError
+			if errors.As(err, &e) {
+				if e.StatusCode == http.StatusTooManyRequests {
+					waitTime = waitTime * 2
+					if waitTime > 300 {
+						return nil, err
+					}
+					printer.Printf("AZURE_DNS: rate-limit paused for %v.\n", waitTime)
+					time.Sleep(time.Duration(waitTime+1) * time.Second)
+					goto retry
+				}
+			}
+		}
+
+		for _, record := range nextResult.Value {
+			if record.Properties != nil && domain == removeTrailingDot(*record.Properties.Fqdn) {
+				for _, ns := range record.Properties.NsRecords {
+					if !slices.Contains(nss, *ns.Nsdname) {
+						nameServers = append(nameServers, *ns.Nsdname)
+					}
+				}
+			}
+		}
+	}
+
+	return nameServers, nil
+}
+
 // GetZoneRecords gets the records of a zone and returns them in RecordConfig format.
 func (a *azurednsProvider) GetZoneRecords(domain string, meta map[string]string) (models.Records, error) {
 	existingRecords, _, _, err := a.getExistingRecords(domain)
@@ -187,23 +278,28 @@ func (a *azurednsProvider) getExistingRecords(domain string) (models.Records, []
 }
 
 // GetZoneRecordsCorrections returns a list of corrections that will turn existing records into dc.Records.
-func (a *azurednsProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, existingRecords models.Records) ([]*models.Correction, error) {
+func (a *azurednsProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, existingRecords models.Records) ([]*models.Correction, int, error) {
 	var corrections []*models.Correction
 
 	// Azure is a "ByRecordSet" API.
 
-	changes, err := diff2.ByRecordSet(existingRecords, dc, nil)
+	changes, actualChangeCount, err := diff2.ByRecordSet(existingRecords, dc, nil)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 
 	for _, change := range changes {
-
 		// Copy all param values to local variables to avoid overwrites
 		msgs := change.MsgsJoined
 		dcn := dc.Name
 		chaKey := change.Key
 
+		if change.Type == diff2.CHANGE || change.Type == diff2.CREATE {
+			if chaKey.Type == "NS" && dcn == removeTrailingDot(change.Key.NameFQDN) {
+				change.New = deduplicateNameServerTargets(change.New)
+			}
+		}
+
 		switch change.Type {
 		case diff2.REPORT:
 			corrections = append(corrections, &models.Correction{Msg: change.MsgsJoined})
@@ -227,11 +323,10 @@ func (a *azurednsProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, ex
 		}
 	}
 
-	return corrections, nil
+	return corrections, actualChangeCount, nil
 }
 
 func (a *azurednsProvider) recordCreate(zoneName string, reckey models.RecordKey, recs models.Records) error {
-
 	rrset, azRecType, err := a.recordToNativeDiff2(reckey, recs)
 	if err != nil {
 		return err
@@ -246,14 +341,15 @@ func (a *azurednsProvider) recordCreate(zoneName string, reckey models.RecordKey
 	rrset.Properties.TTL = &i
 
 	waitTime := 1
-retry:
 
+retry:
 	ctx, cancel := context.WithTimeout(context.Background(), 6000*time.Second)
 	defer cancel()
 	_, err = a.recordsClient.CreateOrUpdate(ctx, *a.resourceGroup, zoneName, recordName, azRecType, *rrset, nil)
 
-	if e, ok := err.(*azcore.ResponseError); ok {
-		if e.StatusCode == 429 {
+	var e *azcore.ResponseError
+	if errors.As(err, &e) {
+		if e.StatusCode == http.StatusTooManyRequests {
 			waitTime = waitTime * 2
 			if waitTime > 300 {
 				return err
@@ -268,7 +364,6 @@ retry:
 }
 
 func (a *azurednsProvider) recordDelete(zoneName string, reckey models.RecordKey) error {
-
 	shortName := strings.TrimSuffix(reckey.NameFQDN, "."+zoneName)
 	if shortName == zoneName {
 		shortName = "@"
@@ -286,8 +381,9 @@ retry:
 	defer cancel()
 	_, err = a.recordsClient.Delete(ctx, *a.resourceGroup, zoneName, shortName, azRecType, nil)
 
-	if e, ok := err.(*azcore.ResponseError); ok {
-		if e.StatusCode == 429 {
+	var e *azcore.ResponseError
+	if errors.As(err, &e) {
+		if e.StatusCode == http.StatusTooManyRequests {
 			waitTime = waitTime * 2
 			if waitTime > 300 {
 				return err
@@ -482,7 +578,6 @@ func nativeToRecords(set *adns.RecordSet, origin string) []*models.RecordConfig
 // NOTE recordToNativeDiff2 is really "convert []RecordConfig to rrset".
 
 func (a *azurednsProvider) recordToNativeDiff2(recordKey models.RecordKey, recordConfig []*models.RecordConfig) (*adns.RecordSet, adns.RecordType, error) {
-
 	recordKeyType := recordKey.Type
 	//	if recordKeyType == "AZURE_ALIAS" {
 	//		fmt.Fprintf(os.Stderr, "DEBUG: XXXXXXXXXXXXXXXXXXXXXXX %v\n", recordKeyType)
@@ -569,7 +664,6 @@ func (a *azurednsProvider) fetchRecordSets(zoneName string) ([]*adns.RecordSet,
 	recordsPager := a.recordsClient.NewListAllByDNSZonePager(*a.resourceGroup, zoneName, nil)
 
 	for recordsPager.More() {
-
 		waitTime := 1
 	retry:
 
@@ -577,9 +671,9 @@ func (a *azurednsProvider) fetchRecordSets(zoneName string) ([]*adns.RecordSet,
 
 		if recordsErr != nil {
 			err := recordsErr
-			if e, ok := err.(*azcore.ResponseError); ok {
-
-				if e.StatusCode == 429 {
+			var e *azcore.ResponseError
+			if errors.As(err, &e) {
+				if e.StatusCode == http.StatusTooManyRequests {
 					waitTime = waitTime * 2
 					if waitTime > 300 {
 						return nil, err
@@ -597,7 +691,7 @@ func (a *azurednsProvider) fetchRecordSets(zoneName string) ([]*adns.RecordSet,
 	return records, nil
 }
 
-func (a *azurednsProvider) EnsureZoneExists(domain string) error {
+func (a *azurednsProvider) EnsureZoneExists(domain string, metadata map[string]string) error {
 	if _, ok := a.zones[domain]; ok {
 		return nil
 	}
@@ -606,9 +700,26 @@ func (a *azurednsProvider) EnsureZoneExists(domain string) error {
 	ctx, cancel := context.WithTimeout(context.Background(), 6000*time.Second)
 	defer cancel()
 
-	_, err := a.zonesClient.CreateOrUpdate(ctx, *a.resourceGroup, domain, adns.Zone{Location: to.StringPtr("global")}, nil)
+	z, err := a.zonesClient.CreateOrUpdate(ctx, *a.resourceGroup, domain, adns.Zone{Location: to.StringPtr("global")}, nil)
 	if err != nil {
 		return err
 	}
+	a.zones[domain] = &z.Zone
 	return nil
 }
+
+func removeTrailingDot(record string) string {
+	return strings.TrimSuffix(record, ".")
+}
+
+func deduplicateNameServerTargets(newRecs models.Records) models.Records {
+	dedupedMap := make(map[string]bool)
+	var deduped models.Records
+	for _, rec := range newRecs {
+		if !dedupedMap[rec.GetTargetField()] {
+			dedupedMap[rec.GetTargetField()] = true
+			deduped = append(deduped, rec)
+		}
+	}
+	return deduped
+}
