@@ -4,6 +4,7 @@ import (
 	"context"
 	"encoding/json"
 	"fmt"
+	"net/http"
 	"strings"
 	"time"
 
@@ -66,7 +67,7 @@ var features = providers.DocumentationNotes{
 	// The default for unlisted capabilities is 'Cannot'.
 	// See providers/capabilities.go for the entire list of capabilities.
 	providers.CanGetZones:            providers.Can(),
-	providers.CanConcur:              providers.Cannot(),
+	providers.CanConcur:              providers.Unimplemented(),
 	providers.CanUseAlias:            providers.Cannot("Azure DNS does not provide a generic ALIAS functionality. Use AZURE_ALIAS instead."),
 	providers.CanUseAzureAlias:       providers.Can(),
 	providers.CanUseCAA:              providers.Cannot("Azure Private DNS does not support CAA records"),
@@ -82,12 +83,15 @@ var features = providers.DocumentationNotes{
 }
 
 func init() {
+	const providerName = "AZURE_PRIVATE_DNS"
+	const providerMaintainer = "@matthewmgamble"
 	fns := providers.DspFuncs{
 		Initializer:   newAzureDNSDsp,
 		RecordAuditor: AuditRecords,
 	}
-	providers.RegisterDomainServiceProviderType("AZURE_PRIVATE_DNS", fns, features)
-	providers.RegisterCustomRecordType("AZURE_ALIAS", "AZURE_PRIVATE_DNS", "")
+	providers.RegisterDomainServiceProviderType(providerName, fns, features)
+	providers.RegisterCustomRecordType("AZURE_ALIAS", providerName, "")
+	providers.RegisterMaintainer(providerName, providerMaintainer)
 }
 
 func (a *azurednsProvider) getExistingZones() ([]*adns.PrivateZone, error) {
@@ -184,16 +188,15 @@ func (a *azurednsProvider) getExistingRecords(domain string) (models.Records, []
 }
 
 // GetZoneRecordsCorrections returns a list of corrections that will turn existing records into dc.Records.
-func (a *azurednsProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, existingRecords models.Records) ([]*models.Correction, error) {
+func (a *azurednsProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, existingRecords models.Records) ([]*models.Correction, int, error) {
 	var corrections []*models.Correction
 
-	changes, err := diff2.ByRecordSet(existingRecords, dc, nil)
+	changes, actualChangeCount, err := diff2.ByRecordSet(existingRecords, dc, nil)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 
 	for _, change := range changes {
-
 		// Copy all param values to local variables to avoid overwrites
 		msgs := change.MsgsJoined
 		dcn := dc.Name
@@ -222,11 +225,10 @@ func (a *azurednsProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, ex
 		}
 	}
 
-	return corrections, nil
+	return corrections, actualChangeCount, nil
 }
 
 func (a *azurednsProvider) recordCreate(zoneName string, reckey models.RecordKey, recs models.Records) error {
-
 	rrset, azRecType, err := a.recordToNativeDiff2(reckey, recs)
 	if err != nil {
 		return err
@@ -248,7 +250,7 @@ retry:
 	_, err = a.recordsClient.CreateOrUpdate(ctx, *a.resourceGroup, zoneName, azRecType, recordName, *rrset, nil)
 
 	if e, ok := err.(*azcore.ResponseError); ok {
-		if e.StatusCode == 429 {
+		if e.StatusCode == http.StatusTooManyRequests {
 			waitTime = waitTime * 2
 			if waitTime > 300 {
 				return err
@@ -263,7 +265,6 @@ retry:
 }
 
 func (a *azurednsProvider) recordDelete(zoneName string, reckey models.RecordKey) error {
-
 	shortName := strings.TrimSuffix(reckey.NameFQDN, "."+zoneName)
 	if shortName == zoneName {
 		shortName = "@"
@@ -282,7 +283,7 @@ retry:
 	_, err = a.recordsClient.Delete(ctx, *a.resourceGroup, zoneName, azRecType, shortName, nil)
 
 	if e, ok := err.(*azcore.ResponseError); ok {
-		if e.StatusCode == 429 {
+		if e.StatusCode == http.StatusTooManyRequests {
 			waitTime = waitTime * 2
 			if waitTime > 300 {
 				return err
@@ -342,7 +343,6 @@ func nativeToRecords(set *adns.RecordSet, origin string) []*models.RecordConfig
 			}
 		} else {
 			panic(fmt.Errorf("nativeToRecords rtype %v unimplemented", *set.Type))
-
 		}
 	case "Microsoft.Network/privateDnsZones/AAAA":
 		if set.Properties.AaaaRecords != nil {
@@ -424,7 +424,6 @@ func nativeToRecords(set *adns.RecordSet, origin string) []*models.RecordConfig
 // NOTE recordToNativeDiff2 is really "convert []RecordConfig to rrset".
 
 func (a *azurednsProvider) recordToNativeDiff2(recordKey models.RecordKey, recordConfig []*models.RecordConfig) (*adns.RecordSet, adns.RecordType, error) {
-
 	recordKeyType := recordKey.Type
 	//	if recordKeyType == "AZURE_ALIAS" {
 	//		fmt.Fprintf(os.Stderr, "DEBUG: XXXXXXXXXXXXXXXXXXXXXXX %v\n", recordKeyType)
@@ -498,7 +497,6 @@ func (a *azurednsProvider) fetchRecordSets(zoneName string) ([]*adns.RecordSet,
 	recordsPager := a.recordsClient.NewListPager(*a.resourceGroup, zoneName, nil)
 
 	for recordsPager.More() {
-
 		waitTime := 1
 	retry:
 
@@ -507,8 +505,7 @@ func (a *azurednsProvider) fetchRecordSets(zoneName string) ([]*adns.RecordSet,
 		if recordsErr != nil {
 			err := recordsErr
 			if e, ok := err.(*azcore.ResponseError); ok {
-
-				if e.StatusCode == 429 {
+				if e.StatusCode == http.StatusTooManyRequests {
 					waitTime = waitTime * 2
 					if waitTime > 300 {
 						return nil, err
@@ -526,7 +523,7 @@ func (a *azurednsProvider) fetchRecordSets(zoneName string) ([]*adns.RecordSet,
 	return records, nil
 }
 
-func (a *azurednsProvider) EnsureZoneExists(domain string) error {
+func (a *azurednsProvider) EnsureZoneExists(domain string, metadata map[string]string) error {
 	if _, ok := a.zones[domain]; ok {
 		return nil
 	}
