@@ -3,22 +3,24 @@ package cloudflare
 import (
 	"context"
 	"encoding/json"
+	"errors"
 	"fmt"
 	"net"
 	"os"
 	"strconv"
 	"strings"
-	"sync"
 
+	"github.com/cloudflare/cloudflare-go"
+	"github.com/fatih/color"
 	"golang.org/x/net/idna"
 
 	"github.com/StackExchange/dnscontrol/v4/models"
 	"github.com/StackExchange/dnscontrol/v4/pkg/diff2"
 	"github.com/StackExchange/dnscontrol/v4/pkg/printer"
 	"github.com/StackExchange/dnscontrol/v4/pkg/transform"
+	"github.com/StackExchange/dnscontrol/v4/pkg/zonecache"
 	"github.com/StackExchange/dnscontrol/v4/providers"
-	"github.com/cloudflare/cloudflare-go"
-	"github.com/fatih/color"
+	"github.com/StackExchange/dnscontrol/v4/providers/cloudflare/rtypes/cfsingleredirect"
 )
 
 /*
@@ -48,6 +50,7 @@ var features = providers.DocumentationNotes{
 	providers.CanUseAlias:            providers.Can("CF automatically flattens CNAME records into A records dynamically"),
 	providers.CanUseCAA:              providers.Can(),
 	providers.CanUseDNSKEY:           providers.Cannot(),
+	providers.CanUseDS:               providers.Can(),
 	providers.CanUseDSForChildren:    providers.Can(),
 	providers.CanUseHTTPS:            providers.Can(),
 	providers.CanUseLOC:              providers.Cannot(),
@@ -63,14 +66,17 @@ var features = providers.DocumentationNotes{
 }
 
 func init() {
+	const providerName = "CLOUDFLAREAPI"
+	const providerMaintainer = "@tresni"
 	fns := providers.DspFuncs{
 		Initializer:   newCloudflare,
 		RecordAuditor: AuditRecords,
 	}
-	providers.RegisterDomainServiceProviderType("CLOUDFLAREAPI", fns, features)
-	providers.RegisterCustomRecordType("CF_REDIRECT", "CLOUDFLAREAPI", "")
-	providers.RegisterCustomRecordType("CF_TEMP_REDIRECT", "CLOUDFLAREAPI", "")
-	providers.RegisterCustomRecordType("CF_WORKER_ROUTE", "CLOUDFLAREAPI", "")
+	providers.RegisterDomainServiceProviderType(providerName, fns, features)
+	providers.RegisterCustomRecordType("CF_REDIRECT", providerName, "")
+	providers.RegisterCustomRecordType("CF_TEMP_REDIRECT", providerName, "")
+	providers.RegisterCustomRecordType("CF_WORKER_ROUTE", providerName, "")
+	providers.RegisterMaintainer(providerName, providerMaintainer)
 }
 
 // cloudflareProvider is the handle for API calls.
@@ -83,58 +89,31 @@ type cloudflareProvider struct {
 	cfClient        *cloudflare.API
 	//
 	manageSingleRedirects bool // New "Single Redirects"-style redirects.
+	//
+	// Used by
+	tcLogFilename string   // Transcode Log file name
+	tcLogFh       *os.File // Transcode Log file handle
+	tcZone        string   // Transcode Current zone
 
-	sync.Mutex                      // Protects all access to the following fields:
-	domainIndex map[string]string   // Cache of zone name to zone ID.
-	nameservers map[string][]string // Cache of zone name to list of nameservers.
+	zoneCache zonecache.ZoneCache[cloudflare.Zone]
 }
 
-// TODO(dlemenkov): remove this function after deleting all commented code referecing it
-//func labelMatches(label string, matches []string) bool {
-//	printer.Debugf("DEBUG: labelMatches(%#v, %#v)\n", label, matches)
-//	for _, tst := range matches {
-//		if label == tst {
-//			return true
-//		}
-//	}
-//	return false
-//}
-
 // GetNameservers returns the nameservers for a domain.
 func (c *cloudflareProvider) GetNameservers(domain string) ([]*models.Nameserver, error) {
-
-	c.Lock()
-	defer c.Unlock()
-	if err := c.cacheDomainList(); err != nil {
+	z, err := c.zoneCache.GetZone(domain)
+	if err != nil {
 		return nil, err
 	}
-
-	ns, ok := c.nameservers[domain]
-	if !ok {
-		return nil, fmt.Errorf("nameservers for %s not found in cloudflare cache(%q)", domain, c.accountID)
-	}
-	return models.ToNameservers(ns)
+	return models.ToNameservers(z.NameServers)
 }
 
 // ListZones returns a list of the DNS zones.
 func (c *cloudflareProvider) ListZones() ([]string, error) {
-
-	c.Lock()
-	defer c.Unlock()
-	if err := c.cacheDomainList(); err != nil {
-		return nil, err
-	}
-
-	zones := make([]string, 0, len(c.domainIndex))
-	for d := range c.domainIndex {
-		zones = append(zones, d)
-	}
-	return zones, nil
+	return c.zoneCache.GetZoneNames()
 }
 
 // GetZoneRecords gets the records of a zone and returns them in RecordConfig format.
 func (c *cloudflareProvider) GetZoneRecords(domain string, meta map[string]string) (models.Records, error) {
-
 	domainID, err := c.getDomainID(domain)
 	if err != nil {
 		return nil, err
@@ -158,17 +137,6 @@ func (c *cloudflareProvider) GetZoneRecords(domain string, meta map[string]strin
 		}
 	}
 
-	// // FIXME(tlim) Why is this needed???
-	// // I don't know. Let's comment it out and see if anything breaks.
-	// for i := len(records) - 1; i >= 0; i-- {
-	// 	rec := records[i]
-	// 	// Delete ignore labels
-	// 	if labelMatches(dnsutil.TrimDomainName(rec.Original.(cloudflare.DNSRecord).Name, dc.Name), c.ignoredLabels) {
-	// 		printer.Debugf("ignored_label: %s\n", rec.Original.(cloudflare.DNSRecord).Name)
-	// 		records = append(records[:i], records[i+1:]...)
-	// 	}
-	// }
-
 	if c.manageRedirects { // if old
 		prs, err := c.getPageRules(domainID, domain)
 		if err != nil {
@@ -179,14 +147,11 @@ func (c *cloudflareProvider) GetZoneRecords(domain string, meta map[string]strin
 
 	if c.manageSingleRedirects { // if new xor old
 		// Download the list of Single Redirects.
-		// For each one, generate a CLOUDFLAREAPI_SINGLE_REDIRECT record
-		// Append these records to `records`
+		// For each one, generate a SINGLEREDIRECT record
 		prs, err := c.getSingleRedirects(domainID, domain)
 		if err != nil {
 			return nil, err
 		}
-		//printer.Printf("DEBUG: Single Redirects")
-		//fmt.Fprintf(os.Stdout, "DEBUG: Single Redirects")
 		records = append(records, prs...)
 	}
 
@@ -205,38 +170,30 @@ func (c *cloudflareProvider) GetZoneRecords(domain string, meta map[string]strin
 }
 
 func (c *cloudflareProvider) getDomainID(name string) (string, error) {
-
-	c.Lock()
-	defer c.Unlock()
-	if err := c.cacheDomainList(); err != nil {
+	z, err := c.zoneCache.GetZone(name)
+	if err != nil {
 		return "", err
 	}
-
-	id, ok := c.domainIndex[name]
-	if !ok {
-		return "", fmt.Errorf("'%s' not a zone in cloudflare account", name)
-	}
-	return id, nil
+	return z.ID, nil
 }
 
 // GetZoneRecordsCorrections returns a list of corrections that will turn existing records into dc.Records.
-func (c *cloudflareProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, records models.Records) ([]*models.Correction, error) {
-
+func (c *cloudflareProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, records models.Records) ([]*models.Correction, int, error) {
 	for _, rec := range dc.Records {
 		if rec.Type == "ALIAS" {
-			rec.Type = "CNAME"
+			rec.ChangeType("CNAME", dc.Name)
 		}
 	}
 
 	if err := c.preprocessConfig(dc); err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 
 	checkNSModifications(dc)
 
 	domainID, err := c.getDomainID(dc.Name)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 
 	for _, rec := range dc.Records {
@@ -253,13 +210,12 @@ func (c *cloudflareProvider) GetZoneRecordsCorrections(dc *models.DomainConfig,
 	var corrections []*models.Correction
 
 	// Cloudflare is a "ByRecord" API.
-	instructions, err := diff2.ByRecord(records, dc, genComparable)
+	instructions, actualChangeCount, err := diff2.ByRecord(records, dc, genComparable)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 
 	for _, inst := range instructions {
-
 		addToFront := false
 		var corrs []*models.Correction
 
@@ -280,8 +236,6 @@ func (c *cloudflareProvider) GetZoneRecordsCorrections(dc *models.DomainConfig,
 		case diff2.DELETE:
 			deleteRec := inst.Old[0]
 			deleteRecType := deleteRec.Type
-			//deleteRecOrig := deleteRec.Original
-			//corrs = c.mkDeleteCorrection(deleteRecType, deleteRecOrig, domainID, msg)
 			corrs = c.mkDeleteCorrection(deleteRecType, deleteRec, domainID, msg)
 			// DS records must always have a corresponding NS record.
 			// Therefore, we remove DS records before any NS records.
@@ -309,7 +263,7 @@ func (c *cloudflareProvider) GetZoneRecordsCorrections(dc *models.DomainConfig,
 		})
 	}
 
-	return corrections, nil
+	return corrections, actualChangeCount, nil
 }
 
 func genComparable(rec *models.RecordConfig) string {
@@ -340,7 +294,7 @@ func (c *cloudflareProvider) mkCreateCorrection(newrec *models.RecordConfig, dom
 			Msg: msg,
 			F:   func() error { return c.createWorkerRoute(domainID, newrec.GetTargetField()) },
 		}}
-	case "CLOUDFLAREAPI_SINGLE_REDIRECT":
+	case cfsingleredirect.SINGLEREDIRECT:
 		return []*models.Correction{{
 			Msg: msg,
 			F: func() error {
@@ -353,15 +307,13 @@ func (c *cloudflareProvider) mkCreateCorrection(newrec *models.RecordConfig, dom
 }
 
 func (c *cloudflareProvider) mkChangeCorrection(oldrec, newrec *models.RecordConfig, domainID string, msg string) []*models.Correction {
-
 	var idTxt string
 	switch oldrec.Type {
 	case "PAGE_RULE":
 		idTxt = oldrec.Original.(cloudflare.PageRule).ID
 	case "WORKER_ROUTE":
 		idTxt = oldrec.Original.(cloudflare.WorkerRoute).ID
-	case "CLOUDFLAREAPI_SINGLE_REDIRECT":
-		//idTxt = oldrec.Original.(cloudflare.RulesetRule).ID
+	case cfsingleredirect.SINGLEREDIRECT:
 		idTxt = oldrec.CloudflareRedirect.SRRRulesetID
 	default:
 		idTxt = oldrec.Original.(cloudflare.DNSRecord).ID
@@ -376,7 +328,7 @@ func (c *cloudflareProvider) mkChangeCorrection(oldrec, newrec *models.RecordCon
 				return c.updatePageRule(idTxt, domainID, *newrec.CloudflareRedirect)
 			},
 		}}
-	case "CLOUDFLAREAPI_SINGLE_REDIRECT":
+	case cfsingleredirect.SINGLEREDIRECT:
 		return []*models.Correction{{
 			Msg: msg,
 			F: func() error {
@@ -393,7 +345,7 @@ func (c *cloudflareProvider) mkChangeCorrection(oldrec, newrec *models.RecordCon
 	default:
 		e := oldrec.Original.(cloudflare.DNSRecord)
 		proxy := e.Proxiable && newrec.Metadata[metaProxy] != "off"
-		//fmt.Fprintf(os.Stderr, "DEBUG: proxy := %v && %v != off is... %v\n", e.Proxiable, newrec.Metadata[metaProxy], proxy)
+		// fmt.Fprintf(os.Stderr, "DEBUG: proxy := %v && %v != off is... %v\n", e.Proxiable, newrec.Metadata[metaProxy], proxy)
 		return []*models.Correction{{
 			Msg: msg,
 			F:   func() error { return c.modifyRecord(domainID, e.ID, proxy, newrec) },
@@ -402,14 +354,13 @@ func (c *cloudflareProvider) mkChangeCorrection(oldrec, newrec *models.RecordCon
 }
 
 func (c *cloudflareProvider) mkDeleteCorrection(recType string, origRec *models.RecordConfig, domainID string, msg string) []*models.Correction {
-
 	var idTxt string
 	switch recType {
 	case "PAGE_RULE":
 		idTxt = origRec.Original.(cloudflare.PageRule).ID
 	case "WORKER_ROUTE":
 		idTxt = origRec.Original.(cloudflare.WorkerRoute).ID
-	case "CLOUDFLAREAPI_SINGLE_REDIRECT":
+	case cfsingleredirect.SINGLEREDIRECT:
 		idTxt = origRec.Original.(cloudflare.RulesetRule).ID
 	default:
 		idTxt = origRec.Original.(cloudflare.DNSRecord).ID
@@ -424,12 +375,7 @@ func (c *cloudflareProvider) mkDeleteCorrection(recType string, origRec *models.
 				return c.deletePageRule(origRec.Original.(cloudflare.PageRule).ID, domainID)
 			case "WORKER_ROUTE":
 				return c.deleteWorkerRoute(origRec.Original.(cloudflare.WorkerRoute).ID, domainID)
-			case "CLOUDFLAREAPI_SINGLE_REDIRECT":
-				//o := origRec.Original.(cloudflare.Ruleset)
-				//printer.Printf("DEBUG: DELETE %+v\n", o)
-				// printer.Printf("DEBUG: DELETE ID = %+v\n", o.ID)
-				// printer.Printf("DEBUG: DELETE ACTION %+v\n", o.ActionParameters)
-				// printer.Printf("DEBUG: DELETE FROMVALUE %+v\n", o.ActionParameters.FromValue)
+			case cfsingleredirect.SINGLEREDIRECT:
 				return c.deleteSingleRedirects(domainID, *origRec.CloudflareRedirect)
 			default:
 				return c.deleteDNSRecord(origRec.Original.(cloudflare.DNSRecord), domainID)
@@ -461,7 +407,7 @@ func checkNSModifications(dc *models.DomainConfig) {
 func (c *cloudflareProvider) checkUniversalSSL(dc *models.DomainConfig, id string) (changed bool, newState bool, err error) {
 	expectedStr := dc.Metadata[metaUniversalSSL]
 	if expectedStr == "" {
-		return false, false, fmt.Errorf("metadata not set")
+		return false, false, errors.New("metadata not set")
 	}
 
 	if actual, err := c.getUniversalSSL(id); err == nil {
@@ -478,7 +424,7 @@ func (c *cloudflareProvider) checkUniversalSSL(dc *models.DomainConfig, id strin
 		}
 		return false, expected, nil
 	}
-	return false, false, fmt.Errorf("error receiving universal ssl state")
+	return false, false, errors.New("error receiving universal ssl state")
 }
 
 const (
@@ -499,6 +445,12 @@ func checkProxyVal(v string) (string, error) {
 
 func (c *cloudflareProvider) preprocessConfig(dc *models.DomainConfig) error {
 
+	for _, rec := range dc.Records {
+		if rec.Type == "ALIAS" {
+			rec.ChangeType("CNAME", dc.Name)
+		}
+	}
+
 	// Determine the default proxy setting.
 	var defProxy string
 	var err error
@@ -523,16 +475,16 @@ func (c *cloudflareProvider) preprocessConfig(dc *models.DomainConfig) error {
 	// A and CNAMEs: Validate. If null, set to default.
 	// else: Make sure it wasn't set.  Set to default.
 	// iterate backwards so first defined page rules have highest priority
-	currentPrPrio := 1
+	prPriority := 0
 	for i := len(dc.Records) - 1; i >= 0; i-- {
 		rec := dc.Records[i]
 		if rec.Metadata == nil {
 			rec.Metadata = map[string]string{}
 		}
 		// cloudflare uses "1" to mean "auto-ttl"
-		// if we get here and ttl is not specified (or is the dnscontrol default of 300),
+		// if we get here and ttl is not specified
 		// use automatic mode instead.
-		if rec.TTL == 0 || rec.TTL == 300 {
+		if rec.TTL == 0 {
 			rec.TTL = 1
 		}
 		if rec.TTL != 1 && rec.TTL < 60 {
@@ -557,80 +509,67 @@ func (c *cloudflareProvider) preprocessConfig(dc *models.DomainConfig) error {
 			}
 		}
 
-		// CF_REDIRECT record types. Encode target as $FROM,$TO,$PRIO,$CODE
+		// CF_REDIRECT record types:
 		if rec.Type == "CF_REDIRECT" || rec.Type == "CF_TEMP_REDIRECT" {
 			if !c.manageRedirects && !c.manageSingleRedirects {
-				return fmt.Errorf("you must add 'manage_single_redirects: true' metadata to cloudflare provider to use CF_REDIRECT/CF_TEMP_REDIRECT records")
+				return errors.New("you must add 'manage_single_redirects: true' metadata to cloudflare provider to use CF_REDIRECT/CF_TEMP_REDIRECT records")
 			}
-			// parts := strings.Split(rec.GetTargetField(), ",")
-			// if len(parts) != 2 {
-			// 	return fmt.Errorf("invalid data specified for cloudflare redirect record")
-			// }
-			code := 301
+			code := uint16(301)
 			if rec.Type == "CF_TEMP_REDIRECT" {
 				code = 302
 			}
 
+			part := strings.SplitN(rec.GetTargetField(), ",", 2)
+			prWhen, prThen := part[0], part[1]
+			prPriority++
+
+			// Convert this record to a PAGE_RULE.
+			if err := cfsingleredirect.MakePageRule(rec, prPriority, code, prWhen, prThen); err != nil {
+				return err
+			}
+			rec.SetLabel("@", dc.Name)
+
 			if c.manageRedirects && !c.manageSingleRedirects {
-				// Old-Style only.  Convert this record to PAGE_RULE.
-				//printer.Printf("DEBUG: prepro() target=%q\n", rec.GetTargetField())
-				sr, err := newCfsrFromUserInput(rec.GetTargetField(), code, currentPrPrio)
-				if err != nil {
-					return err
-				}
-				fixPageRule(rec, sr)
-				currentPrPrio++
+				// Old-Style only.  No additional work needed.
 			} else if !c.manageRedirects && c.manageSingleRedirects {
-				// New-Style only.  Convert this record to a CLOUDFLAREAPI_SINGLE_REDIRECT.
-				sr, err := newCfsrFromUserInput(rec.GetTargetField(), code, currentPrPrio)
-				if err != nil {
+				// New-Style only.  Convert PAGE_RULE to SINGLEREDIRECT.
+				if err := cfsingleredirect.TranscodePRtoSR(rec); err != nil {
 					return err
 				}
-				err = fixSingleRedirect(rec, sr)
-				if err != nil {
+				if err := c.LogTranscode(dc.Name, rec.CloudflareRedirect); err != nil {
 					return err
 				}
 			} else {
-				// Both!  Convert this record to PAGE_RULE and append an additional CLOUDFLAREAPI_SINGLE_REDIRECT.
-
-				target := rec.GetTargetField()
+				// Both old-style and new-style enabled!
+				// Retain the PAGE_RULE and append an additional SINGLEREDIRECT.
 
 				// make a copy:
 				newRec, err := rec.Copy()
 				if err != nil {
 					return err
 				}
-
 				// The copy becomes the CF SingleRedirect
-				sr, err := newCfsrFromUserInput(target, code, currentPrPrio)
-				if err != nil {
+				if err := cfsingleredirect.TranscodePRtoSR(rec); err != nil {
 					return err
 				}
-				err = fixSingleRedirect(newRec, sr)
-				if err != nil {
+				if err := c.LogTranscode(dc.Name, rec.CloudflareRedirect); err != nil {
 					return err
 				}
-
 				// Append the copy to the end of the list.
 				dc.Records = append(dc.Records, newRec)
 
-				// The original becomes the PAGE_RULE:
-				sr, err = newCfsrFromUserInput(target, code, currentPrPrio)
-				if err != nil {
-					return err
-				}
-				fixPageRule(rec, sr)
-				currentPrPrio++
-
+				// The original PAGE_RULE remains untouched.
 			}
-
-		}
-
-		// CF_WORKER_ROUTE record types. Encode target as $PATTERN,$SCRIPT
-		if rec.Type == "CF_WORKER_ROUTE" {
+		} else if rec.Type == cfsingleredirect.SINGLEREDIRECT {
+			// SINGLEREDIRECT record types. Verify they are enabled.
+			if !c.manageSingleRedirects {
+				return errors.New("you must add 'manage_single_redirects: true' metadata to cloudflare provider to use CF_SINGLE__REDIRECT records")
+			}
+		} else if rec.Type == "CF_WORKER_ROUTE" {
+			// CF_WORKER_ROUTE record types. Encode target as $PATTERN,$SCRIPT
 			parts := strings.Split(rec.GetTargetField(), ",")
 			if len(parts) != 2 {
-				return fmt.Errorf("invalid data specified for cloudflare worker record")
+				return errors.New("invalid data specified for cloudflare worker record")
 			}
 			rec.TTL = 1
 			rec.Type = "WORKER_ROUTE"
@@ -656,37 +595,55 @@ func (c *cloudflareProvider) preprocessConfig(dc *models.DomainConfig) error {
 			return err
 		}
 		rec.Metadata[metaOriginalIP] = rec.GetTargetField()
-		rec.SetTarget(newIP.String())
+		if err := rec.SetTarget(newIP.String()); err != nil {
+			return err
+		}
 	}
 
 	return nil
 }
 
-func fixPageRule(rc *models.RecordConfig, sr *models.CloudflareSingleRedirectConfig) {
-	rc.Type = "PAGE_RULE"
-	rc.TTL = 1
-	rc.SetTarget(sr.PRDisplay)
-	rc.CloudflareRedirect = sr
-}
+func (c *cloudflareProvider) LogTranscode(zone string, redirect *models.CloudflareSingleRedirectConfig) error {
+	// No filename? Don't log anything.
+	filename := c.tcLogFilename
+	if filename == "" {
+		return nil
+	}
 
-func fixSingleRedirect(rc *models.RecordConfig, sr *models.CloudflareSingleRedirectConfig) error {
-	rc.Type = "CLOUDFLAREAPI_SINGLE_REDIRECT"
-	rc.TTL = 1
-	rc.SetTarget(sr.SRDisplay)
-	rc.CloudflareRedirect = sr
+	// File not opened already? Open it.
+	if c.tcLogFh == nil {
+		f, err := os.OpenFile(filename, os.O_APPEND|os.O_WRONLY|os.O_CREATE, 0o600)
+		if err != nil {
+			return err
+		}
+		c.tcLogFh = f
+	}
+	fh := c.tcLogFh
 
-	err := addNewStyleFields(sr)
+	// Output "D(zone)"  if needed.
+	var text string
+	if c.tcZone != zone {
+		text = fmt.Sprintf("D(%q, ...\n", zone)
+	}
+	c.tcZone = zone
+
+	// Generate the new command and output.
+	text = text + fmt.Sprintf("    CF_SINGLE_REDIRECT(%q,\n                       %03d,\n                       '%s',\n                       '%s'\n    ),\n",
+		redirect.SRName, redirect.Code,
+		redirect.SRWhen, redirect.SRThen)
+	_, err := fh.WriteString(text)
 	return err
 }
 
 func newCloudflare(m map[string]string, metadata json.RawMessage) (providers.DNSServiceProvider, error) {
 	api := &cloudflareProvider{}
+	api.zoneCache = zonecache.New(api.fetchAllZones)
 	// check api keys from creds json file
 	if m["apitoken"] == "" && (m["apikey"] == "" || m["apiuser"] == "") {
-		return nil, fmt.Errorf("if cloudflare apitoken is not set, apikey and apiuser must be provided")
+		return nil, errors.New("if cloudflare apitoken is not set, apikey and apiuser must be provided")
 	}
 	if m["apitoken"] != "" && (m["apikey"] != "" || m["apiuser"] != "") {
-		return nil, fmt.Errorf("if cloudflare apitoken is set, apikey and apiuser should not be provided")
+		return nil, errors.New("if cloudflare apitoken is set, apikey and apiuser should not be provided")
 	}
 
 	optRP := cloudflare.UsingRetryPolicy(20, 1, 120)
@@ -722,7 +679,8 @@ func newCloudflare(m map[string]string, metadata json.RawMessage) (providers.DNS
 			ManageRedirects bool     `json:"manage_redirects"` // Old-style PAGE_RULE-based redirects
 			ManageWorkers   bool     `json:"manage_workers"`
 			//
-			ManageSingleRedirects bool `json:"manage_single_redirects"` // New-style Dynamic "Single Redirects"
+			ManageSingleRedirects bool   `json:"manage_single_redirects"` // New-style Dynamic "Single Redirects"
+			TranscodeLogFilename  string `json:"transcode_log"`           // Log the PAGE_RULE conversions.
 		}{}
 		err := json.Unmarshal([]byte(metadata), parsedMeta)
 		if err != nil {
@@ -730,6 +688,7 @@ func newCloudflare(m map[string]string, metadata json.RawMessage) (providers.DNS
 		}
 		api.manageSingleRedirects = parsedMeta.ManageSingleRedirects
 		api.manageRedirects = parsedMeta.ManageRedirects
+		api.tcLogFilename = parsedMeta.TranscodeLogFilename
 		api.manageWorkers = parsedMeta.ManageWorkers
 		// ignored_labels:
 		api.ignoredLabels = append(api.ignoredLabels, parsedMeta.IgnoredLabels...)
@@ -837,13 +796,13 @@ func uint16Zero(value interface{}) uint16 {
 	return 0
 }
 
-// intZero converts value to int or returns 0.
-func intZero(value interface{}) int {
+// intZero converts value to uint16 or returns 0.
+func intZero(value interface{}) uint16 {
 	switch v := value.(type) {
 	case float64:
-		return int(v)
+		return uint16(v)
 	case int:
-		return v
+		return uint16(v)
 	case nil:
 	}
 	return 0
@@ -861,6 +820,16 @@ func stringDefault(value interface{}, def string) string {
 
 func (c *cloudflareProvider) nativeToRecord(domain string, cr cloudflare.DNSRecord) (*models.RecordConfig, error) {
 
+	// ALIAS in Cloudflare works like CNAME.
+	if cr.Type == "ALIAS" {
+		cr.Type = "CNAME"
+	}
+
+	// workaround for https://github.com/StackExchange/dnscontrol/issues/446
+	if cr.Type == "SPF" {
+		cr.Type = "TXT"
+	}
+
 	// normalize cname,mx,ns records with dots to be consistent with our config format.
 	if cr.Type == "ALIAS" || cr.Type == "CNAME" || cr.Type == "MX" || cr.Type == "NS" || cr.Type == "PTR" {
 		if cr.Content != "." {
@@ -875,11 +844,6 @@ func (c *cloudflareProvider) nativeToRecord(domain string, cr cloudflare.DNSReco
 	}
 	rc.SetLabelFromFQDN(cr.Name, domain)
 
-	// workaround for https://github.com/StackExchange/dnscontrol/issues/446
-	if cr.Type == "SPF" {
-		cr.Type = "TXT"
-	}
-
 	if cr.Type == "A" || cr.Type == "AAAA" || cr.Type == "CNAME" {
 		if cr.Proxied != nil {
 			if *(cr.Proxied) {
@@ -929,27 +893,21 @@ func getProxyMetadata(r *models.RecordConfig) map[string]string {
 		proxied = r.Metadata[metaProxy] != "off"
 	}
 	return map[string]string{
-		"proxy": fmt.Sprint(proxied),
+		"proxy": strconv.FormatBool(proxied),
 	}
 }
 
 // EnsureZoneExists creates a zone if it does not exist
-func (c *cloudflareProvider) EnsureZoneExists(domain string) error {
-
-	c.Lock()
-	defer c.Unlock()
-	if err := c.cacheDomainList(); err != nil {
+func (c *cloudflareProvider) EnsureZoneExists(domain string, metadata map[string]string) error {
+	if ok, err := c.zoneCache.HasZone(domain); err != nil || ok {
 		return err
 	}
-
-	if _, ok := c.domainIndex[domain]; ok {
-		return nil
-	}
-	var id string
 	id, err := c.createZone(domain)
+	if err != nil {
+		return err
+	}
 	printer.Printf("Added zone for %s to Cloudflare account: %s\n", domain, id)
-	clear(c.domainIndex) // clear the cache so that the next caller has to refresh it, thus loading the new ID.
-	return err
+	return nil
 }
 
 // PrepareCloudflareTestWorkers creates Cloudflare Workers required for CF_WORKER_ROUTE integration tests.
