@@ -2,6 +2,7 @@ package cloudns
 
 import (
 	"encoding/json"
+	"errors"
 	"fmt"
 	"strconv"
 	"strings"
@@ -10,6 +11,7 @@ import (
 	"github.com/StackExchange/dnscontrol/v4/pkg/diff"
 	"github.com/StackExchange/dnscontrol/v4/providers"
 	"github.com/miekg/dns/dnsutil"
+	"golang.org/x/time/rate"
 )
 
 /*
@@ -22,16 +24,12 @@ Info required in `creds.json`:
 // NewCloudns creates the provider.
 func NewCloudns(m map[string]string, metadata json.RawMessage) (providers.DNSServiceProvider, error) {
 	c := &cloudnsProvider{}
+	c.requestLimit = rate.NewLimiter(10, 10)
 
 	c.creds.id, c.creds.password, c.creds.subid = m["auth-id"], m["auth-password"], m["sub-auth-id"]
 
 	if (c.creds.id == "" && c.creds.subid == "") || c.creds.password == "" {
-		return nil, fmt.Errorf("missing ClouDNS auth-id or sub-auth-id and auth-password")
-	}
-
-	// Get a domain to validate authentication
-	if err := c.fetchDomainList(); err != nil {
-		return nil, err
+		return nil, errors.New("missing ClouDNS auth-id or sub-auth-id and auth-password")
 	}
 
 	return c, nil
@@ -40,37 +38,51 @@ func NewCloudns(m map[string]string, metadata json.RawMessage) (providers.DNSSer
 var features = providers.DocumentationNotes{
 	// The default for unlisted capabilities is 'Cannot'.
 	// See providers/capabilities.go for the entire list of capabilities.
+	providers.CanAutoDNSSEC:          providers.Can(),
+	providers.CanConcur:              providers.Can(),
 	providers.CanGetZones:            providers.Can(),
-	providers.CanConcur:              providers.Cannot(),
+	providers.CanOnlyDiff1Features:   providers.Can(),
 	providers.CanUseAlias:            providers.Can(),
 	providers.CanUseCAA:              providers.Can(),
 	providers.CanUseDNAME:            providers.Can(),
 	providers.CanUseDSForChildren:    providers.Can(),
-	providers.CanUseLOC:              providers.Cannot(),
+	providers.CanUseLOC:              providers.Can(),
 	providers.CanUsePTR:              providers.Can(),
 	providers.CanUseSRV:              providers.Can(),
 	providers.CanUseSSHFP:            providers.Can(),
 	providers.CanUseTLSA:             providers.Can(),
 	providers.DocCreateDomains:       providers.Can(),
-	providers.DocDualHost:            providers.Unimplemented(),
+	providers.DocDualHost:            providers.Can(),
+	providers.CanUseNAPTR:            providers.Can(),
+	providers.CanUseSOA:              providers.Unimplemented("Supported by cloudns at a separate API endpoint (/dns/modify-soa.json), not implemented yet"),
+	providers.CanUseDS:               providers.Cannot("Not supported for root, only for children"),
+	providers.CanUseDHCID:            providers.Cannot(),
+	providers.CanUseSVCB:             providers.Cannot(),
+	providers.CanUseHTTPS:            providers.Cannot(),
+	providers.CanUseDNSKEY:           providers.Cannot(),
 	providers.DocOfficiallySupported: providers.Cannot(),
 }
 
 func init() {
+	const providerName = "CLOUDNS"
+	const providerMaintainer = "@pragmaton"
 	fns := providers.DspFuncs{
 		Initializer:   NewCloudns,
 		RecordAuditor: AuditRecords,
 	}
-	providers.RegisterDomainServiceProviderType("CLOUDNS", fns, features)
-	providers.RegisterCustomRecordType("CLOUDNS_WR", "CLOUDNS", "")
+	providers.RegisterDomainServiceProviderType(providerName, fns, features)
+	providers.RegisterCustomRecordType("CLOUDNS_WR", providerName, "")
+	providers.RegisterMaintainer(providerName, providerMaintainer)
 }
 
 // GetNameservers returns the nameservers for a domain.
 func (c *cloudnsProvider) GetNameservers(domain string) ([]*models.Nameserver, error) {
-	if len(c.nameserversNames) == 0 {
-		c.fetchAvailableNameservers()
+	names, err := c.fetchAvailableNameservers()
+	if err != nil {
+		return nil, err
 	}
-	return models.ToNameservers(c.nameserversNames)
+
+	return models.ToNameservers(names)
 }
 
 // // GetDomainCorrections returns the corrections for a domain.
@@ -111,32 +123,38 @@ func (c *cloudnsProvider) GetNameservers(domain string) ([]*models.Nameserver, e
 // }
 
 // GetZoneRecordsCorrections returns a list of corrections that will turn existing records into dc.Records.
-func (c *cloudnsProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, existingRecords models.Records) ([]*models.Correction, error) {
-
-	if c.domainIndex == nil {
-		if err := c.fetchDomainList(); err != nil {
-			return nil, err
-		}
-	}
-	domainID, ok := c.domainIndex[dc.Name]
-	if !ok {
-		return nil, fmt.Errorf("'%s' not a zone in ClouDNS account", dc.Name)
+func (c *cloudnsProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, existingRecords models.Records) ([]*models.Correction, int, error) {
+	domainID, ok, err := c.fetchDomainIndex(dc.Name)
+	if err != nil {
+		return nil, 0, err
+	} else if !ok {
+		return nil, 0, fmt.Errorf("'%s' not a zone in ClouDNS account", dc.Name)
 	}
 
 	// Get a list of available TTL values.
 	// The TTL list needs to be obtained for each domain, so get it first here.
-	c.fetchAvailableTTLValues(dc.Name)
-	// ClouDNS can only be specified from a specific TTL list, so change the TTL in advance.
-	for _, record := range dc.Records {
-		record.TTL = fixTTL(record.TTL)
+	allowedTTLValues, err := c.fetchAvailableTTLValues(dc.Name)
+	if err != nil {
+		return nil, 0, err
 	}
 
-	toReport, create, del, modify, err := diff.NewCompat(dc).IncrementalDiff(existingRecords)
+	// ClouDNS can only be specified from a specific TTL list, so change the TTL in advance.
+	for _, record := range dc.Records {
+		record.TTL = fixTTL(allowedTTLValues, record.TTL)
+	}
+
+	dnssecFixes, err := c.getDNSSECCorrections(dc)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
+	}
+
+	toReport, create, del, modify, actualChangeCount, err := diff.NewCompat(dc).IncrementalDiff(existingRecords)
+	if err != nil {
+		return nil, 0, err
 	}
 	// Start corrections with the reports
 	corrections := diff.GenerateMessageCorrections(toReport)
+	corrections = append(corrections, dnssecFixes...)
 
 	// Deletes first so changing type works etc.
 	for _, m := range del {
@@ -165,7 +183,7 @@ func (c *cloudnsProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, exi
 	for _, m := range create {
 		req, err := toReq(m.Desired)
 		if err != nil {
-			return nil, err
+			return nil, 0, err
 		}
 
 		// ClouDNS does not require the trailing period to be specified when creating an NS record where the A or AAAA record exists in the zone.
@@ -200,7 +218,7 @@ func (c *cloudnsProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, exi
 		id := m.Existing.Original.(*domainRecord).ID
 		req, err := toReq(m.Desired)
 		if err != nil {
-			return nil, err
+			return nil, 0, err
 		}
 
 		// ClouDNS does not require the trailing period to be specified when updating an NS record where the A or AAAA record exists in the zone.
@@ -218,8 +236,35 @@ func (c *cloudnsProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, exi
 		corrections = append(corrections, corr)
 	}
 
-	return corrections, nil
+	return corrections, actualChangeCount, nil
+}
 
+// getDNSSECCorrections returns corrections that update a domain's DNSSEC state.
+func (c *cloudnsProvider) getDNSSECCorrections(dc *models.DomainConfig) ([]*models.Correction, error) {
+	enabled, err := c.isDnssecEnabled(dc.Name)
+	if err != nil {
+		return nil, err
+	}
+
+	if enabled && dc.AutoDNSSEC == "off" {
+		return []*models.Correction{
+			{
+				Msg: "Disable DNSSEC",
+				F:   func() error { err := c.setDnssec(dc.Name, false); return err },
+			},
+		}, nil
+	}
+
+	if !enabled && dc.AutoDNSSEC == "on" {
+		return []*models.Correction{
+			{
+				Msg: "Enable DNSSEC",
+				F:   func() error { err := c.setDnssec(dc.Name, true); return err },
+			},
+		}, nil
+	}
+
+	return []*models.Correction{}, nil
 }
 
 // GetZoneRecords gets the records of a zone and returns them in RecordConfig format.
@@ -230,26 +275,26 @@ func (c *cloudnsProvider) GetZoneRecords(domain string, meta map[string]string)
 	}
 	existingRecords := make([]*models.RecordConfig, len(records))
 	for i := range records {
-		existingRecords[i] = toRc(domain, &records[i])
+		existingRecords[i], err = toRc(domain, &records[i])
+		if err != nil {
+			return nil, err
+		}
 	}
 	return existingRecords, nil
 }
 
 // EnsureZoneExists creates a zone if it does not exist
-func (c *cloudnsProvider) EnsureZoneExists(domain string) error {
-	if err := c.fetchDomainList(); err != nil {
+func (c *cloudnsProvider) EnsureZoneExists(domain string, metadata map[string]string) error {
+	if _, ok, err := c.fetchDomainIndex(domain); err != nil {
 		return err
-	}
-	// zone already exists
-	if _, ok := c.domainIndex[domain]; ok {
+	} else if ok { // zone already exists
 		return nil
 	}
 	return c.createDomain(domain)
 }
 
 // parses the ClouDNS format into our standard RecordConfig
-func toRc(domain string, r *domainRecord) *models.RecordConfig {
-
+func toRc(domain string, r *domainRecord) (*models.RecordConfig, error) {
 	ttl, _ := strconv.ParseUint(r.TTL, 10, 32)
 	priority, _ := strconv.ParseUint(r.Priority, 10, 16)
 	weight, _ := strconv.ParseUint(r.Weight, 10, 16)
@@ -266,16 +311,19 @@ func toRc(domain string, r *domainRecord) *models.RecordConfig {
 	}
 	rc.SetLabel(r.Host, domain)
 
+	var err error
 	switch rtype := r.Type; rtype { // #rtype_variations
 	case "TXT":
-		rc.SetTargetTXT(r.Target)
+		err = rc.SetTargetTXT(r.Target)
 	case "CNAME", "DNAME", "MX", "NS", "SRV", "ALIAS", "PTR":
-		rc.SetTarget(dnsutil.AddOrigin(r.Target+".", domain))
+		if err := rc.SetTarget(dnsutil.AddOrigin(r.Target+".", domain)); err != nil {
+			return nil, err
+		}
 	case "CAA":
 		caaFlag, _ := strconv.ParseUint(r.CaaFlag, 10, 8)
 		rc.CaaFlag = uint8(caaFlag)
 		rc.CaaTag = r.CaaTag
-		rc.SetTarget(r.CaaValue)
+		err = rc.SetTarget(r.CaaValue)
 	case "TLSA":
 		tlsaUsage, _ := strconv.ParseUint(r.TlsaUsage, 10, 8)
 		rc.TlsaUsage = uint8(tlsaUsage)
@@ -283,13 +331,13 @@ func toRc(domain string, r *domainRecord) *models.RecordConfig {
 		rc.TlsaSelector = uint8(tlsaSelector)
 		tlsaMatchingType, _ := strconv.ParseUint(r.TlsaMatchingType, 10, 8)
 		rc.TlsaMatchingType = uint8(tlsaMatchingType)
-		rc.SetTarget(r.Target)
+		err = rc.SetTarget(r.Target)
 	case "SSHFP":
 		sshfpAlgorithm, _ := strconv.ParseUint(r.SshfpAlgorithm, 10, 8)
 		rc.SshfpAlgorithm = uint8(sshfpAlgorithm)
 		sshfpFingerprint, _ := strconv.ParseUint(r.SshfpFingerprint, 10, 8)
 		rc.SshfpFingerprint = uint8(sshfpFingerprint)
-		rc.SetTarget(r.Target)
+		err = rc.SetTarget(r.Target)
 	case "DS":
 		dsKeyTag, _ := strconv.ParseUint(r.DsKeyTag, 10, 16)
 		rc.DsKeyTag = uint16(dsKeyTag)
@@ -298,15 +346,35 @@ func toRc(domain string, r *domainRecord) *models.RecordConfig {
 		dsDigestType, _ := strconv.ParseUint(r.DsDigestType, 10, 8)
 		rc.DsDigestType = uint8(dsDigestType)
 		rc.DsDigest = r.Target
-		rc.SetTarget(r.Target)
+		err = rc.SetTarget(r.Target)
 	case "CLOUD_WR":
 		rc.Type = "WR"
-		rc.SetTarget(r.Target)
+		err = rc.SetTarget(r.Target)
+	case "LOC":
+		loc := fmt.Sprintf("%s %s %s %s %s %s %s %s %s %s %s %s",
+			r.LocLatDeg, r.LocLatMin, r.LocLatSec, r.LocLatDir,
+			r.LocLongDeg, r.LocLongMin, r.LocLongSec, r.LocLongDir,
+			r.LocAltitude, r.LocSize, r.LocHPrecision, r.LocVPrecision)
+		err = rc.SetTargetLOCString(r.Target, loc)
+	case "NAPTR":
+		naptrOrder, _ := strconv.ParseUint(r.NaptrOrder, 10, 16)
+		naptrPreference, _ := strconv.ParseUint(r.NaptrPreference, 10, 16)
+		target := dnsutil.AddOrigin(r.NaptrReplacement+".", domain)
+		err = rc.SetTargetNAPTR(uint16(naptrOrder), uint16(naptrPreference), r.NaptrFlags, r.NaptrService, r.NaptrRegexp, target)
 	default:
-		rc.SetTarget(r.Target)
+		err = rc.SetTarget(r.Target)
 	}
 
-	return rc
+	return rc, err
+}
+
+func formatLocParam(param string) string {
+	param = strings.Split(param, "m")[0]
+	// API misbehaves with a parameter of "0.00" and treats it as the default, so convert to "0" for this case only
+	if param == "0.00" {
+		param = "0"
+	}
+	return param
 }
 
 // toReq takes a RecordConfig and turns it into the native format used by the API.
@@ -350,6 +418,27 @@ func toReq(rc *models.RecordConfig) (requestParams, error) {
 		req["algorithm"] = strconv.Itoa(int(rc.DsAlgorithm))
 		req["digest-type"] = strconv.Itoa(int(rc.DsDigestType))
 		req["record"] = rc.DsDigest
+	case "LOC":
+		parts := strings.Fields(rc.GetTargetCombined())
+		req["lat-deg"] = parts[0]
+		req["lat-min"] = parts[1]
+		req["lat-sec"] = parts[2]
+		req["lat-dir"] = parts[3]
+		req["long-deg"] = parts[4]
+		req["long-min"] = parts[5]
+		req["long-sec"] = parts[6]
+		req["long-dir"] = parts[7]
+		req["altitude"] = formatLocParam(parts[8])
+		req["size"] = formatLocParam(parts[9])
+		req["h-precision"] = formatLocParam(parts[10])
+		req["v-precision"] = formatLocParam(parts[11])
+	case "NAPTR":
+		req["order"] = strconv.Itoa(int(rc.NaptrOrder))
+		req["pref"] = strconv.Itoa(int(rc.NaptrPreference))
+		req["flag"] = rc.NaptrFlags
+		req["params"] = rc.NaptrService
+		req["regexp"] = rc.NaptrRegexp
+		req["replace"] = rc.GetTargetField()
 	default:
 		return nil, fmt.Errorf("ClouDNS.toReq rtype %q unimplemented", rc.Type)
 	}
