@@ -2,7 +2,7 @@ package cscglobal
 
 import (
 	"encoding/json"
-	"fmt"
+	"errors"
 	"strings"
 
 	"github.com/StackExchange/dnscontrol/v4/providers"
@@ -27,8 +27,9 @@ type providerClient struct {
 var features = providers.DocumentationNotes{
 	// The default for unlisted capabilities is 'Cannot'.
 	// See providers/capabilities.go for the entire list of capabilities.
-	providers.CanGetZones:            providers.Can(),
 	providers.CanConcur:              providers.Can(),
+	providers.CanGetZones:            providers.Can(),
+	providers.CanOnlyDiff1Features:   providers.Can(),
 	providers.CanUseCAA:              providers.Can(),
 	providers.CanUseSRV:              providers.Can(),
 	providers.DocOfficiallySupported: providers.Can(),
@@ -50,7 +51,7 @@ func newProvider(m map[string]string) (*providerClient, error) {
 
 	api.key, api.token = m["api-key"], m["user-token"]
 	if api.key == "" || api.token == "" {
-		return nil, fmt.Errorf("missing CSC Global api-key and/or user-token")
+		return nil, errors.New("missing CSC Global api-key and/or user-token")
 	}
 
 	if m["notification_emails"] != "" {
@@ -61,11 +62,14 @@ func newProvider(m map[string]string) (*providerClient, error) {
 }
 
 func init() {
-	providers.RegisterRegistrarType("CSCGLOBAL", newReg)
+	const providerName = "CSCGLOBAL"
+	const providerMaintainer = "@mikenz"
+	providers.RegisterRegistrarType(providerName, newReg)
 
 	fns := providers.DspFuncs{
 		Initializer:   newDsp,
 		RecordAuditor: AuditRecords,
 	}
-	providers.RegisterDomainServiceProviderType("CSCGLOBAL", fns, features)
+	providers.RegisterDomainServiceProviderType(providerName, fns, features)
+	providers.RegisterMaintainer(providerName, providerMaintainer)
 }
