@@ -3,13 +3,14 @@ package digitalocean
 import (
 	"context"
 	"encoding/json"
+	"errors"
 	"fmt"
 	"log"
 	"net/http"
 	"time"
 
 	"github.com/StackExchange/dnscontrol/v4/models"
-	"github.com/StackExchange/dnscontrol/v4/pkg/diff"
+	"github.com/StackExchange/dnscontrol/v4/pkg/diff2"
 	"github.com/StackExchange/dnscontrol/v4/providers"
 	"github.com/digitalocean/godo"
 	"github.com/miekg/dns/dnsutil"
@@ -41,7 +42,7 @@ const perPageSize = 100
 // NewDo creates a DO-specific DNS provider.
 func NewDo(m map[string]string, metadata json.RawMessage) (providers.DNSServiceProvider, error) {
 	if m["token"] == "" {
-		return nil, fmt.Errorf("no DigitalOcean token provided")
+		return nil, errors.New("no DigitalOcean token provided")
 	}
 
 	ctx := context.Background()
@@ -63,7 +64,7 @@ retry:
 		return nil, err
 	}
 	if resp.StatusCode != http.StatusOK {
-		return nil, fmt.Errorf("token for digitalocean is not valid")
+		return nil, errors.New("token for digitalocean is not valid")
 	}
 
 	return api, nil
@@ -72,25 +73,29 @@ retry:
 var features = providers.DocumentationNotes{
 	// The default for unlisted capabilities is 'Cannot'.
 	// See providers/capabilities.go for the entire list of capabilities.
+	providers.CanConcur:              providers.Can(),
 	providers.CanGetZones:            providers.Can(),
-	providers.CanConcur:              providers.Cannot(),
 	providers.CanUseCAA:              providers.Can(),
 	providers.CanUseLOC:              providers.Cannot(),
 	providers.CanUseSRV:              providers.Can(),
 	providers.DocCreateDomains:       providers.Can(),
+	providers.DocDualHost:            providers.Can(),
 	providers.DocOfficiallySupported: providers.Cannot(),
 }
 
 func init() {
+	const providerName = "DIGITALOCEAN"
+	const providerMaintainer = "@Deraen"
 	fns := providers.DspFuncs{
 		Initializer:   NewDo,
 		RecordAuditor: AuditRecords,
 	}
-	providers.RegisterDomainServiceProviderType("DIGITALOCEAN", fns, features)
+	providers.RegisterDomainServiceProviderType(providerName, fns, features)
+	providers.RegisterMaintainer(providerName, providerMaintainer)
 }
 
 // EnsureZoneExists creates a zone if it does not exist
-func (api *digitaloceanProvider) EnsureZoneExists(domain string) error {
+func (api *digitaloceanProvider) EnsureZoneExists(domain string, metadata map[string]string) error {
 retry:
 	ctx := context.Background()
 	_, resp, err := api.client.Domains.Get(ctx, domain)
@@ -98,7 +103,7 @@ retry:
 		if pauseAndRetry(resp) {
 			goto retry
 		}
-		//return err
+		// return err
 	}
 	if resp.StatusCode == http.StatusNotFound {
 		_, _, err := api.client.Domains.Create(ctx, &godo.DomainCreateRequest{
@@ -158,7 +163,10 @@ func (api *digitaloceanProvider) GetZoneRecords(domain string, meta map[string]s
 
 	var existingRecords []*models.RecordConfig
 	for i := range records {
-		r := toRc(domain, &records[i])
+		r, err := toRc(domain, &records[i])
+		if err != nil {
+			return nil, err
+		}
 		if r.Type == "SOA" {
 			continue
 		}
@@ -169,71 +177,69 @@ func (api *digitaloceanProvider) GetZoneRecords(domain string, meta map[string]s
 }
 
 // GetZoneRecordsCorrections returns a list of corrections that will turn existing records into dc.Records.
-func (api *digitaloceanProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, existingRecords models.Records) ([]*models.Correction, error) {
+func (api *digitaloceanProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, existingRecords models.Records) ([]*models.Correction, int, error) {
 	ctx := context.Background()
 
-	toReport, toCreate, toDelete, toModify, err := diff.NewCompat(dc).IncrementalDiff(existingRecords)
+	var corrections []*models.Correction
+
+	instructions, actualChangeCount, err := diff2.ByRecord(existingRecords, dc, nil)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
-	// Start corrections with the reports
-	corrections := diff.GenerateMessageCorrections(toReport)
 
-	// Deletes first so changing type works etc.
-	for _, m := range toDelete {
-		id := m.Existing.Original.(*godo.DomainRecord).ID
-		corr := &models.Correction{
-			Msg: fmt.Sprintf("%s, DO ID: %d", m.String(), id),
-			F: func() error {
-			retry:
-				resp, err := api.client.Domains.DeleteRecord(ctx, dc.Name, id)
-				if err != nil {
-					if pauseAndRetry(resp) {
-						goto retry
+	addCorrection := func(msg string, f func() (*godo.Response, error)) {
+		corrections = append(corrections,
+			&models.Correction{
+				Msg: msg,
+				F: func() error {
+				retry:
+					resp, err := f()
+					if err != nil {
+						if pauseAndRetry(resp) {
+							goto retry
+						}
 					}
-				}
-				return err
-			},
-		}
-		corrections = append(corrections, corr)
+					return err
+				},
+			})
 	}
-	for _, m := range toCreate {
-		req := toReq(m.Desired)
-		corr := &models.Correction{
-			Msg: m.String(),
-			F: func() error {
-			retry:
+
+	for _, inst := range instructions {
+		switch inst.Type {
+		case diff2.REPORT:
+			corrections = append(corrections,
+				&models.Correction{
+					Msg: inst.MsgsJoined,
+				})
+			continue
+
+		case diff2.CREATE:
+			req := toReq(inst.New[0])
+			addCorrection(inst.MsgsJoined, func() (*godo.Response, error) {
 				_, resp, err := api.client.Domains.CreateRecord(ctx, dc.Name, req)
-				if err != nil {
-					if pauseAndRetry(resp) {
-						goto retry
-					}
-				}
-				return err
-			},
-		}
-		corrections = append(corrections, corr)
-	}
-	for _, m := range toModify {
-		id := m.Existing.Original.(*godo.DomainRecord).ID
-		req := toReq(m.Desired)
-		corr := &models.Correction{
-			Msg: fmt.Sprintf("%s, DO ID: %d", m.String(), id),
-			F: func() error {
-			retry:
+				return resp, err
+			})
+
+		case diff2.CHANGE:
+			id := inst.Old[0].Original.(*godo.DomainRecord).ID
+			req := toReq(inst.New[0])
+			addCorrection(inst.MsgsJoined, func() (*godo.Response, error) {
 				_, resp, err := api.client.Domains.EditRecord(ctx, dc.Name, id, req)
-				if err != nil {
-					if pauseAndRetry(resp) {
-						goto retry
-					}
-				}
-				return err
-			},
+				return resp, err
+			})
+
+		case diff2.DELETE:
+			id := inst.Old[0].Original.(*godo.DomainRecord).ID
+			addCorrection(inst.MsgsJoined, func() (*godo.Response, error) {
+				return api.client.Domains.DeleteRecord(ctx, dc.Name, id)
+			})
+
+		default:
+			panic(fmt.Sprintf("unhandled inst.Type %s", inst.Type))
 		}
-		corrections = append(corrections, corr)
 	}
 
-	return corrections, nil
+	return corrections, actualChangeCount, nil
 }
 
 func getRecords(api *digitaloceanProvider, name string) ([]godo.DomainRecord, error) {
@@ -269,7 +275,7 @@ retry:
 	return records, nil
 }
 
-func toRc(domain string, r *godo.DomainRecord) *models.RecordConfig {
+func toRc(domain string, r *godo.DomainRecord) (*models.RecordConfig, error) {
 	// This handles "@" etc.
 	name := dnsutil.AddOrigin(r.Name, domain)
 
@@ -300,11 +306,15 @@ func toRc(domain string, r *godo.DomainRecord) *models.RecordConfig {
 	t.SetLabelFromFQDN(name, domain)
 	switch rtype := r.Type; rtype {
 	case "TXT":
-		t.SetTargetTXT(target)
+		if err := t.SetTargetTXT(target); err != nil {
+			return nil, err
+		}
 	default:
-		t.SetTarget(target)
+		if err := t.SetTarget(target); err != nil {
+			return nil, err
+		}
 	}
-	return t
+	return t, nil
 }
 
 func toReq(rc *models.RecordConfig) *godo.DomainRecordEditRequest {
