@@ -2,7 +2,7 @@ package dnsmadeeasy
 
 import (
 	"encoding/json"
-	"fmt"
+	"errors"
 	"os"
 	"strings"
 
@@ -14,8 +14,9 @@ import (
 var features = providers.DocumentationNotes{
 	// The default for unlisted capabilities is 'Cannot'.
 	// See providers/capabilities.go for the entire list of capabilities.
+	providers.CanConcur:              providers.Unimplemented(),
 	providers.CanGetZones:            providers.Can(),
-	providers.CanConcur:              providers.Cannot(),
+	providers.CanOnlyDiff1Features:   providers.Can(),
 	providers.CanUseAlias:            providers.Can(),
 	providers.CanUseCAA:              providers.Can(),
 	providers.CanUseDS:               providers.Cannot(),
@@ -31,22 +32,25 @@ var features = providers.DocumentationNotes{
 }
 
 func init() {
+	const providerName = "DNSMADEEASY"
+	const providerMaintainer = "@vojtad"
 	fns := providers.DspFuncs{
 		Initializer:   New,
 		RecordAuditor: AuditRecords,
 	}
 
-	providers.RegisterDomainServiceProviderType("DNSMADEEASY", fns, features)
+	providers.RegisterDomainServiceProviderType(providerName, fns, features)
+	providers.RegisterMaintainer(providerName, providerMaintainer)
 }
 
 // New creates a new API handle.
 func New(settings map[string]string, _ json.RawMessage) (providers.DNSServiceProvider, error) {
 	if settings["api_key"] == "" {
-		return nil, fmt.Errorf("missing DNSMADEEASY api_key")
+		return nil, errors.New("missing DNSMADEEASY api_key")
 	}
 
 	if settings["secret_key"] == "" {
-		return nil, fmt.Errorf("missing DNSMADEEASY secret_key")
+		return nil, errors.New("missing DNSMADEEASY secret_key")
 	}
 
 	sandbox := false
@@ -100,26 +104,26 @@ func New(settings map[string]string, _ json.RawMessage) (providers.DNSServicePro
 // 	return api.GetZoneRecordsCorrections(dc, existingRecords)
 // }
 
-func (api *dnsMadeEasyProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, existingRecords models.Records) ([]*models.Correction, error) {
+func (api *dnsMadeEasyProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, existingRecords models.Records) ([]*models.Correction, int, error) {
 	domainName := dc.Name
-	domain, err := api.findDomain(domainName)
+	domainID, err := api.findDomainID(domainName)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 
 	for _, rec := range dc.Records {
 		if rec.Type == "ALIAS" {
 			// ALIAS is called ANAME on DNS Made Easy
-			rec.Type = "ANAME"
+			rec.ChangeType("ANAME", dc.Name)
 		} else if rec.Type == "NS" {
 			// NS records have fixed TTL on DNS Made Easy and it cannot be changed
 			rec.TTL = fixedNameServerRecordTTL
 		}
 	}
 
-	toReport, create, del, modify, err := diff.NewCompat(dc).IncrementalDiff(existingRecords)
+	toReport, create, del, modify, actualChangeCount, err := diff.NewCompat(dc).IncrementalDiff(existingRecords)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 	// Start corrections with the reports
 	corrections := diff.GenerateMessageCorrections(toReport)
@@ -136,7 +140,7 @@ func (api *dnsMadeEasyProvider) GetZoneRecordsCorrections(dc *models.DomainConfi
 		corr := &models.Correction{
 			Msg: strings.Join(deleteDescription, "\n\t"),
 			F: func() error {
-				return api.deleteRecords(domain.ID, deleteRecordIds)
+				return api.deleteRecords(domainID, deleteRecordIds)
 			},
 		}
 		corrections = append(corrections, corr)
@@ -154,7 +158,7 @@ func (api *dnsMadeEasyProvider) GetZoneRecordsCorrections(dc *models.DomainConfi
 		corr := &models.Correction{
 			Msg: strings.Join(createDescription, "\n\t"),
 			F: func() error {
-				return api.createRecords(domain.ID, createRecords)
+				return api.createRecords(domainID, createRecords)
 			},
 		}
 		corrections = append(corrections, corr)
@@ -177,17 +181,17 @@ func (api *dnsMadeEasyProvider) GetZoneRecordsCorrections(dc *models.DomainConfi
 		corr := &models.Correction{
 			Msg: strings.Join(modifyDescription, "\n\t"),
 			F: func() error {
-				return api.updateRecords(domain.ID, modifyRecords)
+				return api.updateRecords(domainID, modifyRecords)
 			},
 		}
 		corrections = append(corrections, corr)
 	}
 
-	return corrections, nil
+	return corrections, actualChangeCount, nil
 }
 
 // EnsureZoneExists creates a zone if it does not exist
-func (api *dnsMadeEasyProvider) EnsureZoneExists(domain string) error {
+func (api *dnsMadeEasyProvider) EnsureZoneExists(domain string, metadata map[string]string) error {
 	exists, err := api.domainExists(domain)
 	if err != nil {
 		return err
