@@ -8,12 +8,11 @@ import (
 	"strconv"
 	"strings"
 
-	egoscale "github.com/exoscale/egoscale/v2"
-
 	"github.com/StackExchange/dnscontrol/v4/models"
 	"github.com/StackExchange/dnscontrol/v4/pkg/diff"
 	"github.com/StackExchange/dnscontrol/v4/pkg/printer"
 	"github.com/StackExchange/dnscontrol/v4/providers"
+	egoscale "github.com/exoscale/egoscale/v2"
 )
 
 const (
@@ -56,8 +55,9 @@ func NewExoscale(m map[string]string, metadata json.RawMessage) (providers.DNSSe
 var features = providers.DocumentationNotes{
 	// The default for unlisted capabilities is 'Cannot'.
 	// See providers/capabilities.go for the entire list of capabilities.
+	providers.CanConcur:              providers.Unimplemented(),
 	providers.CanGetZones:            providers.Unimplemented(),
-	providers.CanConcur:              providers.Cannot(),
+	providers.CanOnlyDiff1Features:   providers.Can(),
 	providers.CanUseAlias:            providers.Can(),
 	providers.CanUseCAA:              providers.Can(),
 	providers.CanUseLOC:              providers.Cannot(),
@@ -70,15 +70,18 @@ var features = providers.DocumentationNotes{
 }
 
 func init() {
+	const providerName = "EXOSCALE"
+	const providerMaintainer = "@pierre-emmanuelJ"
 	fns := providers.DspFuncs{
 		Initializer:   NewExoscale,
 		RecordAuditor: AuditRecords,
 	}
-	providers.RegisterDomainServiceProviderType("EXOSCALE", fns, features)
+	providers.RegisterDomainServiceProviderType(providerName, fns, features)
+	providers.RegisterMaintainer(providerName, providerMaintainer)
 }
 
 // EnsureZoneExists creates a zone if it does not exist
-func (c *exoscaleProvider) EnsureZoneExists(domain string) error {
+func (c *exoscaleProvider) EnsureZoneExists(domain string, metadata map[string]string) error {
 	_, err := c.findDomainByName(domain)
 
 	return err
@@ -91,7 +94,7 @@ func (c *exoscaleProvider) GetNameservers(domain string) ([]*models.Nameserver,
 
 // GetZoneRecords gets the records of a zone and returns them in RecordConfig format.
 func (c *exoscaleProvider) GetZoneRecords(domainName string, meta map[string]string) (models.Records, error) {
-	//dc.Punycode()
+	// dc.Punycode()
 
 	domain, err := c.findDomainByName(domainName)
 	if err != nil {
@@ -163,7 +166,7 @@ func (c *exoscaleProvider) GetZoneRecords(domainName string, meta map[string]str
 		switch rtype {
 		case "ALIAS", "URL":
 			rc.Type = rtype
-			rc.SetTarget(rcontent)
+			err = rc.SetTarget(rcontent)
 		case "MX":
 			var prio uint16
 			if record.Priority != nil {
@@ -184,18 +187,17 @@ func (c *exoscaleProvider) GetZoneRecords(domainName string, meta map[string]str
 }
 
 // GetZoneRecordsCorrections returns a list of corrections that will turn existing records into dc.Records.
-func (c *exoscaleProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, existingRecords models.Records) ([]*models.Correction, error) {
-
+func (c *exoscaleProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, existingRecords models.Records) ([]*models.Correction, int, error) {
 	removeOtherNS(dc)
 	domain, err := c.findDomainByName(dc.Name)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 	domainID := *domain.ID
 
-	toReport, create, toDelete, modify, err := diff.NewCompat(dc).IncrementalDiff(existingRecords)
+	toReport, create, toDelete, modify, actualChangeCount, err := diff.NewCompat(dc).IncrementalDiff(existingRecords)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 	// Start corrections with the reports
 	corrections := diff.GenerateMessageCorrections(toReport)
@@ -217,15 +219,15 @@ func (c *exoscaleProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, ex
 	}
 
 	for _, mod := range modify {
-		old := mod.Existing.Original.(*egoscale.DNSDomainRecord)
-		new := mod.Desired
+		old_ := mod.Existing.Original.(*egoscale.DNSDomainRecord)
+		new_ := mod.Desired
 		corrections = append(corrections, &models.Correction{
 			Msg: mod.String(),
-			F:   c.updateRecordFunc(old, new, domainID),
+			F:   c.updateRecordFunc(old_, new_, domainID),
 		})
 	}
 
-	return corrections, nil
+	return corrections, actualChangeCount, nil
 }
 
 // Returns a function that can be invoked to create a record in a zone.
