@@ -2,11 +2,12 @@ package hetzner
 
 import (
 	"encoding/json"
-	"fmt"
+	"errors"
 	"strings"
 
 	"github.com/StackExchange/dnscontrol/v4/models"
 	"github.com/StackExchange/dnscontrol/v4/pkg/diff"
+	"github.com/StackExchange/dnscontrol/v4/pkg/zonecache"
 	"github.com/StackExchange/dnscontrol/v4/providers"
 )
 
@@ -14,8 +15,9 @@ var features = providers.DocumentationNotes{
 	// The default for unlisted capabilities is 'Cannot'.
 	// See providers/capabilities.go for the entire list of capabilities.
 	providers.CanAutoDNSSEC:          providers.Cannot(),
+	providers.CanConcur:              providers.Can(),
 	providers.CanGetZones:            providers.Can(),
-	providers.CanConcur:              providers.Cannot(),
+	providers.CanOnlyDiff1Features:   providers.Can(),
 	providers.CanUseAlias:            providers.Cannot(),
 	providers.CanUseCAA:              providers.Can(),
 	providers.CanUseDS:               providers.Can(),
@@ -33,57 +35,54 @@ var features = providers.DocumentationNotes{
 }
 
 func init() {
+	const providerName = "HETZNER"
+	const providerMaintainer = "@das7pad"
 	fns := providers.DspFuncs{
 		Initializer:   New,
 		RecordAuditor: AuditRecords,
 	}
-	providers.RegisterDomainServiceProviderType("HETZNER", fns, features)
+	providers.RegisterDomainServiceProviderType(providerName, fns, features)
+	providers.RegisterMaintainer(providerName, providerMaintainer)
 }
 
 // New creates a new API handle.
 func New(settings map[string]string, _ json.RawMessage) (providers.DNSServiceProvider, error) {
 	apiKey := settings["api_key"]
 	if apiKey == "" {
-		return nil, fmt.Errorf("missing HETZNER api_key")
+		return nil, errors.New("missing HETZNER api_key")
 	}
 
-	return &hetznerProvider{
-		apiKey: apiKey,
-	}, nil
+	api := &hetznerProvider{apiKey: apiKey}
+	api.zoneCache = zonecache.New(api.fetchAllZones)
+	return api, nil
 }
 
 // EnsureZoneExists creates a zone if it does not exist
-func (api *hetznerProvider) EnsureZoneExists(domain string) error {
-	domains, err := api.ListZones()
-	if err != nil {
+func (api *hetznerProvider) EnsureZoneExists(domain string, metadata map[string]string) error {
+	if ok, err := api.zoneCache.HasZone(domain); err != nil || ok {
 		return err
 	}
 
-	for _, d := range domains {
-		if d == domain {
-			return nil
-		}
+	if err := api.createZone(domain); err != nil {
+		return err
 	}
-
-	// reset zone cache
-	api.zones = nil
-	return api.createZone(domain)
+	return nil
 }
 
 // GetZoneRecordsCorrections returns a list of corrections that will turn existing records into dc.Records.
-func (api *hetznerProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, existingRecords models.Records) ([]*models.Correction, error) {
+func (api *hetznerProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, existingRecords models.Records) ([]*models.Correction, int, error) {
 	domain := dc.Name
 
-	toReport, create, del, modify, err := diff.NewCompat(dc).IncrementalDiff(existingRecords)
+	toReport, create, del, modify, actualChangeCount, err := diff.NewCompat(dc).IncrementalDiff(existingRecords)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 	// Start corrections with the reports
 	corrections := diff.GenerateMessageCorrections(toReport)
 
-	z, err := api.getZone(domain)
+	z, err := api.zoneCache.GetZone(domain)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 
 	for _, m := range del {
@@ -133,12 +132,12 @@ func (api *hetznerProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, e
 		corrections = append(corrections, corr)
 	}
 
-	return corrections, nil
+	return corrections, actualChangeCount, nil
 }
 
 // GetNameservers returns the nameservers for a domain.
 func (api *hetznerProvider) GetNameservers(domain string) ([]*models.Nameserver, error) {
-	z, err := api.getZone(domain)
+	z, err := api.zoneCache.GetZone(domain)
 	if err != nil {
 		return nil, err
 	}
@@ -164,12 +163,5 @@ func (api *hetznerProvider) GetZoneRecords(domain string, meta map[string]string
 
 // ListZones lists the zones on this account.
 func (api *hetznerProvider) ListZones() ([]string, error) {
-	if err := api.getAllZones(); err != nil {
-		return nil, err
-	}
-	zones := make([]string, 0, len(api.zones))
-	for domain := range api.zones {
-		zones = append(zones, domain)
-	}
-	return zones, nil
+	return api.zoneCache.GetZoneNames()
 }
