@@ -2,7 +2,9 @@ package hexonet
 
 import (
 	"bytes"
+	"errors"
 	"fmt"
+	"math"
 	"strconv"
 	"strings"
 
@@ -47,19 +49,18 @@ func (n *HXClient) GetZoneRecords(domain string, meta map[string]string) (models
 
 	for _, rec := range actual {
 		if rec.Type == "ALIAS" {
-			return nil, fmt.Errorf("we support realtime ALIAS RR over our X-DNS service, please get in touch with us")
+			return nil, errors.New("we support realtime ALIAS RR over our X-DNS service, please get in touch with us")
 		}
 	}
 
 	return actual, nil
-
 }
 
 // GetZoneRecordsCorrections returns a list of corrections that will turn existing records into dc.Records.
-func (n *HXClient) GetZoneRecordsCorrections(dc *models.DomainConfig, actual models.Records) ([]*models.Correction, error) {
-	toReport, create, del, mod, err := diff.NewCompat(dc).IncrementalDiff(actual)
+func (n *HXClient) GetZoneRecordsCorrections(dc *models.DomainConfig, actual models.Records) ([]*models.Correction, int, error) {
+	toReport, create, del, mod, actualChangeCount, err := diff.NewCompat(dc).IncrementalDiff(actual)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 	// Start corrections with the reports
 	corrections := diff.GenerateMessageCorrections(toReport)
@@ -76,7 +77,7 @@ func (n *HXClient) GetZoneRecordsCorrections(dc *models.DomainConfig, actual mod
 		rec := cre.Desired
 		recordString, err := n.createRecordString(rec, dc.Name)
 		if err != nil {
-			return corrections, err
+			return corrections, 0, err
 		}
 		params[fmt.Sprintf("ADDRR%d", addrridx)] = recordString
 		addrridx++
@@ -91,12 +92,12 @@ func (n *HXClient) GetZoneRecordsCorrections(dc *models.DomainConfig, actual mod
 	for _, chng := range mod {
 		changes = true
 		fmt.Fprintln(buf, chng)
-		old := chng.Existing.Original.(*HXRecord)
-		new := chng.Desired
-		params[fmt.Sprintf("DELRR%d", delrridx)] = n.deleteRecordString(old)
-		newRecordString, err := n.createRecordString(new, dc.Name)
+		old_ := chng.Existing.Original.(*HXRecord)
+		new_ := chng.Desired
+		params[fmt.Sprintf("DELRR%d", delrridx)] = n.deleteRecordString(old_)
+		newRecordString, err := n.createRecordString(new_, dc.Name)
 		if err != nil {
-			return corrections, err
+			return corrections, 0, err
 		}
 		params[fmt.Sprintf("ADDRR%d", addrridx)] = newRecordString
 		addrridx++
@@ -113,7 +114,7 @@ func (n *HXClient) GetZoneRecordsCorrections(dc *models.DomainConfig, actual mod
 		})
 	}
 
-	return corrections, nil
+	return corrections, actualChangeCount, nil
 }
 
 func toRecord(r *HXRecord, origin string) *models.RecordConfig {
@@ -206,7 +207,13 @@ func (n *HXClient) getRecords(domain string) ([]*HXRecord, error) {
 				record.Fqdn = spl[0] + "." + record.Fqdn
 			}
 			if record.Type == "MX" || record.Type == "SRV" {
-				prio, _ := strconv.ParseUint(spl[4], 10, 32)
+				prio, err := strconv.ParseUint(spl[4], 10, 32)
+				if err != nil {
+					return nil, fmt.Errorf("failed to parse priority: %w", err)
+				}
+				if prio > math.MaxUint16 {
+					return nil, fmt.Errorf("priority value exceeds uint16 maximum: %s", spl[4])
+				}
 				record.Priority = uint32(prio)
 				record.Answer = strings.Join(spl[5:], " ")
 			} else {
@@ -238,7 +245,7 @@ func (n *HXClient) createRecordString(rc *models.RecordConfig, domain string) (s
 		record.Answer = txtutil.EncodeQuoted(rc.GetTargetTXTJoined())
 	case "SRV":
 		if rc.GetTargetField() == "." {
-			return "", fmt.Errorf("SRV records with empty targets are not supported (as of 2020-02-27, the API returns 'Invalid attribute value syntax')")
+			return "", errors.New("SRV records with empty targets are not supported (as of 2020-02-27, the API returns 'Invalid attribute value syntax')")
 		}
 		record.Answer = fmt.Sprintf("%d %d %v", rc.SrvWeight, rc.SrvPort, record.Answer)
 		record.Priority = uint32(rc.SrvPriority)
@@ -248,9 +255,9 @@ func (n *HXClient) createRecordString(rc *models.RecordConfig, domain string) (s
 		// that have not been updated for a new RR type.
 	}
 
-	str := record.Host + " " + fmt.Sprint(record.TTL) + " IN " + record.Type + " "
+	str := record.Host + " " + strconv.FormatUint(uint64(record.TTL), 10) + " IN " + record.Type + " "
 	if record.Type == "MX" || record.Type == "SRV" {
-		str += fmt.Sprint(record.Priority) + " "
+		str += strconv.FormatUint(uint64(record.Priority), 10) + " "
 	}
 	str += record.Answer
 	return str, nil
