@@ -2,6 +2,7 @@ package hostingde
 
 import (
 	"encoding/json"
+	"errors"
 	"fmt"
 	"math"
 	"sort"
@@ -19,8 +20,9 @@ var features = providers.DocumentationNotes{
 	// The default for unlisted capabilities is 'Cannot'.
 	// See providers/capabilities.go for the entire list of capabilities.
 	providers.CanAutoDNSSEC:          providers.Can(),
+	providers.CanConcur:              providers.Unimplemented(),
 	providers.CanGetZones:            providers.Can(),
-	providers.CanConcur:              providers.Cannot(),
+	providers.CanOnlyDiff1Features:   providers.Can(),
 	providers.CanUseAlias:            providers.Can(),
 	providers.CanUseCAA:              providers.Can(),
 	providers.CanUseDS:               providers.Can(),
@@ -37,12 +39,15 @@ var features = providers.DocumentationNotes{
 }
 
 func init() {
-	providers.RegisterRegistrarType("HOSTINGDE", newHostingdeReg)
+	const providerName = "HOSTINGDE"
+	const providerMaintainer = "@juliusrickert"
+	providers.RegisterRegistrarType(providerName, newHostingdeReg)
 	fns := providers.DspFuncs{
 		Initializer:   newHostingdeDsp,
 		RecordAuditor: AuditRecords,
 	}
-	providers.RegisterDomainServiceProviderType("HOSTINGDE", fns, features)
+	providers.RegisterDomainServiceProviderType(providerName, fns, features)
+	providers.RegisterMaintainer(providerName, providerMaintainer)
 }
 
 type providerMeta struct {
@@ -53,7 +58,7 @@ func newHostingde(m map[string]string, providermeta json.RawMessage) (*hostingde
 	authToken, ownerAccountID, filterAccountID, baseURL := m["authToken"], m["ownerAccountId"], m["filterAccountId"], m["baseURL"]
 
 	if authToken == "" {
-		return nil, fmt.Errorf("hosting.de: authtoken must be provided")
+		return nil, errors.New("hosting.de: authtoken must be provided")
 	}
 
 	if baseURL == "" {
@@ -100,19 +105,23 @@ func (hp *hostingdeProvider) GetZoneRecords(domain string, meta map[string]strin
 	if err != nil {
 		return nil, err
 	}
-	return hp.APIRecordsToStandardRecordsModel(domain, zone.Records), nil
+	return hp.APIRecordsToStandardRecordsModel(domain, zone.Records)
 }
 
-func (hp *hostingdeProvider) APIRecordsToStandardRecordsModel(domain string, src []record) models.Records {
+func (hp *hostingdeProvider) APIRecordsToStandardRecordsModel(domain string, src []record) (models.Records, error) {
 	records := []*models.RecordConfig{}
 	for _, r := range src {
 		if r.Type == "SOA" {
 			continue
 		}
-		records = append(records, r.nativeToRecord(domain))
+		newr, err := r.nativeToRecord(domain)
+		if err != nil {
+			return nil, err
+		}
+		records = append(records, newr)
 	}
 
-	return records
+	return records, nil
 }
 
 func soaToString(s soaValues) string {
@@ -120,7 +129,7 @@ func soaToString(s soaValues) string {
 }
 
 // GetZoneRecordsCorrections returns a list of corrections that will turn existing records into dc.Records.
-func (hp *hostingdeProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, records models.Records) ([]*models.Correction, error) {
+func (hp *hostingdeProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, records models.Records) ([]*models.Correction, int, error) {
 	var err error
 
 	// TTL must be between (inclusive) 1m and 1y (in fact, a little bit more)
@@ -137,12 +146,12 @@ func (hp *hostingdeProvider) GetZoneRecordsCorrections(dc *models.DomainConfig,
 
 	zone, err := hp.getZone(dc.Name)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 
-	toReport, create, del, mod, err := diff.NewCompat(dc).IncrementalDiff(records)
+	toReport, create, del, mod, actualChangeCount, err := diff.NewCompat(dc).IncrementalDiff(records)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 	// Start corrections with the reports
 	corrections := diff.GenerateMessageCorrections(toReport)
@@ -222,7 +231,7 @@ func (hp *hostingdeProvider) GetZoneRecordsCorrections(dc *models.DomainConfig,
 	if existingAutoDNSSecEnabled && desiredAutoDNSSecEnabled {
 		currentDNSSecOptions, err := hp.getDNSSECOptions(zone.ZoneConfig.ID)
 		if err != nil {
-			return nil, err
+			return nil, 0, err
 		}
 		if !currentDNSSecOptions.PublishKSK {
 			msg = append(msg, "Enabling publishKsk for AutoDNSSec")
@@ -243,7 +252,7 @@ func (hp *hostingdeProvider) GetZoneRecordsCorrections(dc *models.DomainConfig,
 	} else if existingAutoDNSSecEnabled && !desiredAutoDNSSecEnabled {
 		currentDNSSecOptions, err := hp.getDNSSECOptions(zone.ZoneConfig.ID)
 		if err != nil {
-			return nil, err
+			return nil, 0, err
 		}
 		msg = append(msg, "Disable AutoDNSSEC")
 		zone.ZoneConfig.DNSSECMode = "off"
@@ -251,7 +260,7 @@ func (hp *hostingdeProvider) GetZoneRecordsCorrections(dc *models.DomainConfig,
 		// Remove auto dnssec keys from domain
 		DomainConfig, err := hp.getDomainConfig(dc.Name)
 		if err != nil {
-			return nil, err
+			return nil, 0, err
 		}
 		for _, entry := range DomainConfig.DNSSecEntries {
 			for _, autoDNSKey := range currentDNSSecOptions.Keys {
@@ -264,13 +273,13 @@ func (hp *hostingdeProvider) GetZoneRecordsCorrections(dc *models.DomainConfig,
 	}
 
 	if !zoneChanged {
-		return nil, nil
+		return nil, 0, nil
 	}
 
 	corrections = append(corrections, &models.Correction{
-		Msg: fmt.Sprintf("\n%s", strings.Join(msg, "\n")),
+		Msg: "\n" + strings.Join(msg, "\n"),
 		F: func() error {
-			for i := 0; i < 10; i++ {
+			for i := range 10 {
 				err := hp.updateZone(&zone.ZoneConfig, DNSSecOptions, create, del, mod)
 				if err == nil {
 					return nil
@@ -284,7 +293,7 @@ func (hp *hostingdeProvider) GetZoneRecordsCorrections(dc *models.DomainConfig,
 				// Base of 1.8 seemed like a good trade-off, retrying for approximately 45 seconds.
 				time.Sleep(time.Duration(math.Pow(1.8, float64(i))) * 100 * time.Millisecond)
 			}
-			return fmt.Errorf("retry exhaustion: zone blocked for 10 attempts")
+			return errors.New("retry exhaustion: zone blocked for 10 attempts")
 		},
 	},
 	)
@@ -303,7 +312,7 @@ func (hp *hostingdeProvider) GetZoneRecordsCorrections(dc *models.DomainConfig,
 		corrections = append(corrections, correction)
 	}
 
-	return corrections, nil
+	return corrections, actualChangeCount, nil
 }
 
 func firstNonZero(items ...uint32) uint32 {
@@ -348,9 +357,9 @@ func (hp *hostingdeProvider) GetRegistrarCorrections(dc *models.DomainConfig) ([
 	return nil, nil
 }
 
-func (hp *hostingdeProvider) EnsureZoneExists(domain string) error {
+func (hp *hostingdeProvider) EnsureZoneExists(domain string, metadata map[string]string) error {
 	_, err := hp.getZoneConfig(domain)
-	if err == errZoneNotFound {
+	if errors.Is(err, errZoneNotFound) {
 		if err := hp.createZone(domain); err != nil {
 			return err
 		}
@@ -368,5 +377,4 @@ func (hp *hostingdeProvider) ListZones() ([]string, error) {
 		zones = append(zones, zoneConfig.Name)
 	}
 	return zones, nil
-
 }
