@@ -2,17 +2,21 @@ package inwx
 
 import (
 	"encoding/json"
+	"errors"
 	"fmt"
 	"sort"
 	"strings"
 	"time"
 
-	"github.com/StackExchange/dnscontrol/v4/models"
-	"github.com/StackExchange/dnscontrol/v4/pkg/diff"
-	"github.com/StackExchange/dnscontrol/v4/pkg/printer"
-	"github.com/StackExchange/dnscontrol/v4/providers"
+	"github.com/fatih/color"
 	"github.com/nrdcg/goinwx"
 	"github.com/pquerna/otp/totp"
+	"golang.org/x/net/idna"
+
+	"github.com/StackExchange/dnscontrol/v4/models"
+	"github.com/StackExchange/dnscontrol/v4/pkg/diff2"
+	"github.com/StackExchange/dnscontrol/v4/pkg/printer"
+	"github.com/StackExchange/dnscontrol/v4/providers"
 )
 
 /*
@@ -43,17 +47,17 @@ var InwxSandboxDefaultNs = []string{"ns.ote.inwx.de", "ns2.ote.inwx.de"}
 var features = providers.DocumentationNotes{
 	// The default for unlisted capabilities is 'Cannot'.
 	// See providers/capabilities.go for the entire list of capabilities.
-	providers.CanAutoDNSSEC:          providers.Unimplemented("Supported by INWX but not implemented yet."),
+	providers.CanAutoDNSSEC:          providers.Can(),
 	providers.CanGetZones:            providers.Can(),
-	providers.CanConcur:              providers.Cannot(),
-	providers.CanUseAlias:            providers.Cannot("INWX does not support the ALIAS or ANAME record type."),
+	providers.CanConcur:              providers.Unimplemented(),
+	providers.CanUseAlias:            providers.Can(),
 	providers.CanUseCAA:              providers.Can(),
 	providers.CanUseDS:               providers.Unimplemented("DS records are only supported at the apex and require a different API call that hasn't been implemented yet."),
 	providers.CanUseHTTPS:            providers.Can(),
 	providers.CanUseLOC:              providers.Unimplemented(),
 	providers.CanUseNAPTR:            providers.Can(),
 	providers.CanUsePTR:              providers.Can("PTR records with empty targets are not supported"),
-	providers.CanUseSRV:              providers.Can("SRV records with empty targets are not supported."),
+	providers.CanUseSRV:              providers.Can(),
 	providers.CanUseSSHFP:            providers.Can(),
 	providers.CanUseSVCB:             providers.Can(),
 	providers.CanUseTLSA:             providers.Can(),
@@ -71,12 +75,15 @@ type inwxAPI struct {
 
 // init registers the registrar and the domain service provider with dnscontrol.
 func init() {
-	providers.RegisterRegistrarType("INWX", newInwxReg)
+	const providerName = "INWX"
+	const providerMaintainer = "@patschi"
+	providers.RegisterRegistrarType(providerName, newInwxReg)
 	fns := providers.DspFuncs{
 		Initializer:   newInwxDsp,
 		RecordAuditor: AuditRecords,
 	}
-	providers.RegisterDomainServiceProviderType("INWX", fns, features)
+	providers.RegisterDomainServiceProviderType(providerName, fns, features)
+	providers.RegisterMaintainer(providerName, providerMaintainer)
 }
 
 // getOTP either returns the TOTPValue or uses TOTPKey and the current time to generate a valid TOTPValue.
@@ -86,11 +93,11 @@ func getOTP(TOTPValue string, TOTPKey string) (string, error) {
 	} else if TOTPKey != "" {
 		tan, err := totp.GenerateCode(TOTPKey, time.Now())
 		if err != nil {
-			return "", fmt.Errorf("INWX: Unable to generate TOTP from totp-key: %v", err)
+			return "", fmt.Errorf("INWX: Unable to generate TOTP from totp-key: %w", err)
 		}
 		return tan, nil
 	} else {
-		return "", fmt.Errorf("INWX: two factor authentication required but no TOTP configured")
+		return "", errors.New("INWX: two factor authentication required but no TOTP configured")
 	}
 }
 
@@ -98,7 +105,7 @@ func getOTP(TOTPValue string, TOTPKey string) (string, error) {
 func (api *inwxAPI) loginHelper(TOTPValue string, TOTPKey string) error {
 	resp, err := api.client.Account.Login()
 	if err != nil {
-		return fmt.Errorf("INWX: Unable to login")
+		return errors.New("INWX: Unable to login")
 	}
 
 	switch TFA := resp.TFA; TFA {
@@ -130,13 +137,13 @@ func newInwx(m map[string]string) (*inwxAPI, error) {
 	sandbox := m["sandbox"] == "1"
 
 	if username == "" {
-		return nil, fmt.Errorf("INWX: username must be provided")
+		return nil, errors.New("INWX: username must be provided")
 	}
 	if password == "" {
-		return nil, fmt.Errorf("INWX: password must be provided")
+		return nil, errors.New("INWX: password must be provided")
 	}
 	if TOTPValue != "" && TOTPKey != "" {
-		return nil, fmt.Errorf("INWX: totp and totp-key must not be specified at the same time")
+		return nil, errors.New("INWX: totp and totp-key must not be specified at the same time")
 	}
 
 	opts := &goinwx.ClientOptions{Sandbox: sandbox}
@@ -179,9 +186,9 @@ func makeNameserverRecordRequest(domain string, rec *models.RecordConfig) *goinw
 	   The API will not accept any target with a final dot but will
 	   instead always add this final dot internally.
 	   Records with empty targets (i.e. records with target ".")
-	   are not allowed.
+	   are allowed.
 	*/
-	case "CNAME", "NS":
+	case "CNAME", "NS", "ALIAS":
 		req.Content = content[:len(content)-1]
 	case "MX":
 		req.Priority = int(rec.MxPreference)
@@ -192,7 +199,11 @@ func makeNameserverRecordRequest(domain string, rec *models.RecordConfig) *goinw
 		}
 	case "SRV":
 		req.Priority = int(rec.SrvPriority)
-		req.Content = fmt.Sprintf("%d %d %v", rec.SrvWeight, rec.SrvPort, content[:len(content)-1])
+		if content == "." {
+			req.Content = fmt.Sprintf("%d %d %v", rec.SrvWeight, rec.SrvPort, content)
+		} else {
+			req.Content = fmt.Sprintf("%d %d %v", rec.SrvWeight, rec.SrvPort, content[:len(content)-1])
+		}
 	default:
 		req.Content = rec.GetTargetCombined()
 	}
@@ -219,57 +230,117 @@ func (api *inwxAPI) deleteRecord(RecordID int) error {
 	return api.client.Nameservers.DeleteRecord(RecordID)
 }
 
-// checkRecords ensures that there is no single-quote inside TXT records which would be ignored by INWX.
-func checkRecords(records models.Records) error {
-	// TODO(tlim) Remove this function.  auditrecords.go takes care of this now.
-	for _, r := range records {
-		if r.Type == "TXT" {
-			if strings.ContainsAny(r.GetTargetTXTJoined(), "`") {
-				return fmt.Errorf("INWX TXT records do not support single-quotes in their target")
-			}
-		}
+// isNullMX checks if a record is a null MX record.
+func isNullMX(rec *models.RecordConfig) bool {
+	return rec.Type == "MX" && rec.MxPreference == 0 && rec.GetTargetField() == "."
+}
+
+// AutoDnssecToggle enables and disables AutoDNSSEC for INWX domains.
+func (api *inwxAPI) AutoDnssecToggle(dc *models.DomainConfig, corrections []*models.Correction) ([]*models.Correction, error) {
+
+	dnssecStatus, err := api.DNSSecStatus(dc.Name)
+	if err != nil {
+		return corrections, err
 	}
-	return nil
+
+	if dnssecStatus == ManualDNSSECStatus && dc.AutoDNSSEC != "" {
+		return corrections, fmt.Errorf("INWX: Domain %s has manual DNSSEC enabled. Disable it before using AUTODNSSEC_ON/AUTODNSSEC_OFF", dc.Name)
+	}
+
+	if dnssecStatus != AutoDNSSECStatus && dc.AutoDNSSEC == "on" {
+		corrections = append(corrections, &models.Correction{
+			Msg: color.YellowString("Enable AutoDNSSEC"),
+			F: func() error {
+				return api.enableAutoDNSSEC(dc.Name)
+			},
+		})
+	}
+
+	if dnssecStatus == AutoDNSSECStatus && dc.AutoDNSSEC == "off" {
+		corrections = append(corrections, &models.Correction{
+			Msg: color.RedString("Disable AutoDNSSEC"),
+			F: func() error {
+				return api.disableAutoDNSSEC(dc.Name)
+			},
+		})
+	}
+	return corrections, nil
 }
 
 // GetZoneRecordsCorrections returns a list of corrections that will turn existing records into dc.Records.
-func (api *inwxAPI) GetZoneRecordsCorrections(dc *models.DomainConfig, foundRecords models.Records) ([]*models.Correction, error) {
-	err := checkRecords(dc.Records)
+func (api *inwxAPI) GetZoneRecordsCorrections(dc *models.DomainConfig, foundRecords models.Records) ([]*models.Correction, int, error) {
+
+	// INWX support for Null MX requires special handling.  MX preference changes
+	// of >0 to 0 are silently dropped, so if the change includes a null MX record
+	// we have to delete then create.  Corrections are compiled separately for
+	// deletes, changes and creates and then assembled in that order.
+	corrections := []*models.Correction{}
+	deletes := []*models.Correction{}
+	creates := []*models.Correction{}
+	deferred := []*models.Correction{}
+
+	corrections, err := api.AutoDnssecToggle(dc, corrections)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 
-	toReport, create, del, mod, err := diff.NewCompat(dc).IncrementalDiff(foundRecords)
+	changes, actualChangeCount, err := diff2.ByRecord(foundRecords, dc, nil)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
-	// Start corrections with the reports
-	corrections := diff.GenerateMessageCorrections(toReport)
+	for _, change := range changes {
+		changeMsgs := change.MsgsJoined
+		switch change.Type {
+		case diff2.REPORT:
+			corrections = append(corrections, &models.Correction{Msg: changeMsgs})
+		case diff2.CHANGE:
+			oldRec := change.Old[0]
+			newRec := change.New[0]
+			if isNullMX(newRec) || isNullMX(oldRec) {
+				// changing to or from a Null MX has to be delete then create
+				deletes = append(deletes, &models.Correction{
+					Msg: color.RedString("- DELETE %s %s %s ttl=%d", oldRec.GetLabelFQDN(), oldRec.Type, oldRec.ToComparableNoTTL(), oldRec.TTL),
+					F: func() error {
+						return api.deleteRecord(oldRec.Original.(goinwx.NameserverRecord).ID)
+					},
+				})
+				deferred = append(deferred, &models.Correction{
+					Msg: color.GreenString("+ CREATE %s %s %s ttl=%d", newRec.GetLabelFQDN(), newRec.Type, newRec.ToComparableNoTTL(), newRec.TTL),
+					F: func() error {
+						return api.createRecord(dc.Name, newRec)
+					},
+				})
+			} else {
+				recID := oldRec.Original.(goinwx.NameserverRecord).ID
+				corrections = append(corrections, &models.Correction{
+					Msg: changeMsgs,
+					F: func() error {
+						return api.updateRecord(recID, newRec)
+					},
+				})
 
-	for _, d := range create {
-		des := d.Desired
-		corrections = append(corrections, &models.Correction{
-			Msg: d.String(),
-			F:   func() error { return api.createRecord(dc.Name, des) },
-		})
+			}
+		case diff2.CREATE:
+			creates = append(creates, &models.Correction{
+				Msg: changeMsgs,
+				F: func() error {
+					return api.createRecord(dc.Name, change.New[0])
+				},
+			})
+		case diff2.DELETE:
+			recID := change.Old[0].Original.(goinwx.NameserverRecord).ID
+			deletes = append(deletes, &models.Correction{
+				Msg: changeMsgs,
+				F:   func() error { return api.deleteRecord(recID) },
+			})
+		default:
+			panic(fmt.Sprintf("unhandled change.Type %s", change.Type))
+		}
 	}
-	for _, d := range del {
-		existingID := d.Existing.Original.(goinwx.NameserverRecord).ID
-		corrections = append(corrections, &models.Correction{
-			Msg: d.String(),
-			F:   func() error { return api.deleteRecord(existingID) },
-		})
-	}
-	for _, d := range mod {
-		rec := d.Desired
-		existingID := d.Existing.Original.(goinwx.NameserverRecord).ID
-		corrections = append(corrections, &models.Correction{
-			Msg: d.String(),
-			F:   func() error { return api.updateRecord(existingID, rec) },
-		})
-	}
-
-	return corrections, nil
+	corrections = append(deletes, corrections...)
+	corrections = append(corrections, creates...)
+	corrections = append(corrections, deferred...)
+	return corrections, actualChangeCount, nil
 }
 
 // getDefaultNameservers returns string map with default nameservers based on e.g. sandbox mode.
@@ -280,9 +351,10 @@ func (api *inwxAPI) getDefaultNameservers() []string {
 	return InwxProductionDefaultNs
 }
 
-// GetNameservers returns the default nameservers for INWX.
+// GetNameservers returns the nameservers provisioned for the domain or the
+// default INWX nameservers.
 func (api *inwxAPI) GetNameservers(domain string) ([]*models.Nameserver, error) {
-	return models.ToNameservers(api.getDefaultNameservers())
+	return models.ToNameservers(api.fetchRegistrationNSSet(domain))
 }
 
 // GetZoneRecords receives the current records from Inwx and converts them to models.RecordConfig.
@@ -292,7 +364,7 @@ func (api *inwxAPI) GetZoneRecords(domain string, meta map[string]string) (model
 		return nil, err
 	}
 
-	var records = []*models.RecordConfig{}
+	records := []*models.RecordConfig{}
 
 	for _, record := range info.Records {
 		if record.Type == "SOA" {
@@ -304,9 +376,10 @@ func (api *inwxAPI) GetZoneRecords(domain string, meta map[string]string) (model
 		   The API will not accept any target with a final dot but will
 		   instead always add this final dot internally.
 		   Records with empty targets (i.e. records with target ".")
-		   are not allowed.
+		   are allowed.
 		*/
-		var rtypeAddDot = map[string]bool{
+		rtypeAddDot := map[string]bool{
+			"ALIAS": true,
 			"CNAME": true,
 			"MX":    true,
 			"NS":    true,
@@ -316,6 +389,8 @@ func (api *inwxAPI) GetZoneRecords(domain string, meta map[string]string) (model
 		if rtypeAddDot[record.Type] {
 			if record.Type == "MX" && record.Content == "." {
 				// null records don't need to be modified
+			} else if record.Type == "SRV" && strings.HasSuffix(record.Content, ".") {
+				// null targets don't need to be modified
 			} else {
 				record.Content = record.Content + "."
 			}
@@ -376,18 +451,21 @@ func (api *inwxAPI) updateNameservers(ns []string, domain string) func() error {
 
 // GetRegistrarCorrections is part of the registrar provider and determines if the nameservers have to be updated.
 func (api *inwxAPI) GetRegistrarCorrections(dc *models.DomainConfig) ([]*models.Correction, error) {
-	info, err := api.client.Domains.Info(dc.Name, 0)
-	if err != nil {
-		return nil, err
-	}
-
-	sort.Strings(info.Nameservers)
-	foundNameservers := strings.Join(info.Nameservers, ",")
-	expected := []string{}
+	regNameservers := api.fetchRegistrationNSSet(dc.Name)
+	combined := map[string]bool{}
 	for _, ns := range dc.Nameservers {
-		expected = append(expected, ns.Name)
+		combined[ns.Name] = true
+	}
+	for _, rs := range regNameservers {
+		combined[rs] = true
+	}
+	var expected []string
+	for k := range combined {
+		expected = append(expected, k)
+
 	}
 	sort.Strings(expected)
+	foundNameservers := strings.Join(regNameservers, ",")
 	expectedNameservers := strings.Join(expected, ",")
 
 	if foundNameservers != expectedNameservers {
@@ -413,7 +491,14 @@ func (api *inwxAPI) fetchNameserverDomains() error {
 			return err
 		}
 		for _, domain := range info.Domains {
-			zones[domain.Domain] = domain.RoID
+			// If this is an IDN domain, Nameservers.List.Domains[].Domain
+			// will contain the Unicode name but subsequent calls use the ACE
+			// encoded name. We will convert it now for use as the cache key
+			aceName, err := idna.ToASCII(domain.Domain)
+			if err != nil {
+				return err
+			}
+			zones[aceName] = domain.RoID
 		}
 		if len(zones) >= info.Count {
 			break
@@ -424,8 +509,17 @@ func (api *inwxAPI) fetchNameserverDomains() error {
 	return nil
 }
 
+func (api *inwxAPI) fetchRegistrationNSSet(domain string) []string {
+	info, err := api.client.Domains.Info(domain, 0)
+	if err != nil {
+		return api.getDefaultNameservers()
+	}
+	sort.Strings(info.Nameservers)
+	return info.Nameservers
+}
+
 // EnsureZoneExists creates a zone if it does not exist
-func (api *inwxAPI) EnsureZoneExists(domain string) error {
+func (api *inwxAPI) EnsureZoneExists(domain string, metadata map[string]string) error {
 	if api.domainIndex == nil { // only pull the data once.
 		if err := api.fetchNameserverDomains(); err != nil {
 			return err
@@ -440,13 +534,13 @@ func (api *inwxAPI) EnsureZoneExists(domain string) error {
 	request := &goinwx.NameserverCreateRequest{
 		Domain:      domain,
 		Type:        "MASTER",
-		Nameservers: api.getDefaultNameservers(),
+		Nameservers: api.fetchRegistrationNSSet(domain),
 	}
-	var id int
 	id, err := api.client.Nameservers.Create(request)
 	if err != nil {
 		return err
 	}
 	printer.Printf("Added zone for %s to INWX account with id %d\n", domain, id)
+	api.domainIndex[domain] = id
 	return nil
 }
