@@ -2,6 +2,7 @@ package providers
 
 import (
 	"encoding/json"
+	"errors"
 	"fmt"
 	"log"
 
@@ -21,7 +22,7 @@ type DNSServiceProvider interface {
 // ZoneCreator should be implemented by providers that have the ability to create zones
 // (used for automatically creating zones if they don't exist)
 type ZoneCreator interface {
-	EnsureZoneExists(domain string) error
+	EnsureZoneExists(domain string, metadata map[string]string) error
 }
 
 // ZoneLister should be implemented by providers that have the
@@ -69,9 +70,19 @@ func RegisterDomainServiceProviderType(name string, fns DspFuncs, pm ...Provider
 		log.Fatalf("Cannot register registrar type %q multiple times", name)
 	}
 	DNSProviderTypes[name] = fns
+
 	unwrapProviderCapabilities(name, pm)
 }
 
+var ProviderMaintainers = map[string]string{}
+
+func RegisterMaintainer(
+	providerName string,
+	gitHubUsername string,
+) {
+	ProviderMaintainers[providerName] = gitHubUsername
+}
+
 // CreateRegistrar initializes a registrar instance from given credentials.
 func CreateRegistrar(rType string, config map[string]string) (Registrar, error) {
 	var err error
@@ -112,7 +123,7 @@ func beCompatible(n string, config map[string]string) (string, error) {
 	if n == "" || n == "-" {
 		// But no TYPE exists in creds.json...
 		if ct == "" {
-			return "-", fmt.Errorf("creds.json entry missing TYPE field")
+			return "-", errors.New("creds.json entry missing TYPE field")
 		}
 		// Otherwise, use the value from creds.json.
 		return ct, nil
@@ -128,7 +139,7 @@ func beCompatible(n string, config map[string]string) (string, error) {
 
 	// NB(tlim): My hope is that in 4.0 this entire function will simply be the
 	// following, but I may be wrong:
-	//return config["TYPE"], nil
+	// return config["TYPE"], nil
 }
 
 // AuditRecords calls the RecordAudit function for a provider.
@@ -162,8 +173,8 @@ func (n None) GetZoneRecords(domain string, meta map[string]string) (models.Reco
 }
 
 // GetZoneRecordsCorrections gets the records of a zone and returns them in RecordConfig format.
-func (n None) GetZoneRecordsCorrections(dc *models.DomainConfig, records models.Records) ([]*models.Correction, error) {
-	return nil, nil
+func (n None) GetZoneRecordsCorrections(dc *models.DomainConfig, records models.Records) ([]*models.Correction, int, error) {
+	return nil, 0, nil
 }
 
 // GetDomainCorrections returns corrections to update a domain.
