@@ -20,10 +20,12 @@ import (
 	"github.com/aws/aws-sdk-go-v2/aws"
 	"github.com/aws/aws-sdk-go-v2/config"
 	"github.com/aws/aws-sdk-go-v2/credentials"
+	"github.com/aws/aws-sdk-go-v2/credentials/stscreds"
 	r53 "github.com/aws/aws-sdk-go-v2/service/route53"
 	r53Types "github.com/aws/aws-sdk-go-v2/service/route53/types"
 	r53d "github.com/aws/aws-sdk-go-v2/service/route53domains"
 	r53dTypes "github.com/aws/aws-sdk-go-v2/service/route53domains/types"
+	"github.com/aws/aws-sdk-go-v2/service/sts"
 )
 
 type route53Provider struct {
@@ -50,7 +52,7 @@ func newRoute53(m map[string]string, _ json.RawMessage) (*route53Provider, error
 		config.WithRegion("us-east-1"),
 	}
 
-	keyID, secretKey, tokenID := m["KeyId"], m["SecretKey"], m["Token"]
+	keyID, secretKey, tokenID, roleArn, externalId := m["KeyId"], m["SecretKey"], m["Token"], m["RoleArn"], m["ExternalId"]
 	// Token is optional and left empty unless required
 	if keyID != "" || secretKey != "" {
 		optFns = append(optFns, config.WithCredentialsProvider(credentials.NewStaticCredentialsProvider(keyID, secretKey, tokenID)))
@@ -61,6 +63,20 @@ func newRoute53(m map[string]string, _ json.RawMessage) (*route53Provider, error
 		return nil, err
 	}
 
+	if roleArn != "" {
+		stsClient := sts.NewFromConfig(config)
+		sessionName := fmt.Sprintf("dnscontrol-route53-%d", time.Now().Unix())
+
+		var assumeOpts []func(*stscreds.AssumeRoleOptions)
+		if externalId != "" {
+			assumeOpts = append(assumeOpts, func(o *stscreds.AssumeRoleOptions) { o.ExternalID = aws.String(externalId) })
+		}
+		assumeOpts = append(assumeOpts, func(o *stscreds.AssumeRoleOptions) { o.RoleSessionName = sessionName })
+
+		stsCredsProvider := stscreds.NewAssumeRoleProvider(stsClient, roleArn, assumeOpts...)
+		config.Credentials = aws.NewCredentialsCache(stsCredsProvider)
+	}
+
 	var dls *string
 	if val, ok := m["DelegationSet"]; ok {
 		printer.Printf("ROUTE53 DelegationSet %s configured\n", val)
@@ -81,23 +97,30 @@ var features = providers.DocumentationNotes{
 	providers.CanConcur:              providers.Can(),
 	providers.CanUseAlias:            providers.Cannot("R53 does not provide a generic ALIAS functionality. Use R53_ALIAS instead."),
 	providers.CanUseCAA:              providers.Can(),
+	providers.CanUseHTTPS:            providers.Can(),
 	providers.CanUseLOC:              providers.Cannot(),
 	providers.CanUsePTR:              providers.Can(),
 	providers.CanUseRoute53Alias:     providers.Can(),
 	providers.CanUseSRV:              providers.Can(),
+	providers.CanUseSSHFP:            providers.Can(),
+	providers.CanUseSVCB:             providers.Can(),
+	providers.CanUseTLSA:             providers.Can(),
 	providers.DocCreateDomains:       providers.Can(),
 	providers.DocDualHost:            providers.Can(),
 	providers.DocOfficiallySupported: providers.Can(),
 }
 
 func init() {
+	const providerName = "ROUTE53"
+	const providerMaintainer = "@tresni"
 	fns := providers.DspFuncs{
 		Initializer:   newRoute53Dsp,
 		RecordAuditor: AuditRecords,
 	}
-	providers.RegisterDomainServiceProviderType("ROUTE53", fns, features)
-	providers.RegisterRegistrarType("ROUTE53", newRoute53Reg)
-	providers.RegisterCustomRecordType("R53_ALIAS", "ROUTE53", "")
+	providers.RegisterDomainServiceProviderType(providerName, fns, features)
+	providers.RegisterRegistrarType(providerName, newRoute53Reg)
+	providers.RegisterCustomRecordType("R53_ALIAS", providerName, "")
+	providers.RegisterMaintainer(providerName, providerMaintainer)
 }
 
 func withRetry(f func() error) {
@@ -136,7 +159,6 @@ func (r *route53Provider) ListZones() ([]string, error) {
 }
 
 func (r *route53Provider) getZones() error {
-
 	if r.zonesByDomain != nil {
 		return nil
 	}
@@ -241,7 +263,6 @@ func (r *route53Provider) GetZoneRecords(domain string, meta map[string]string)
 }
 
 func (r *route53Provider) getZone(dc *models.DomainConfig) (r53Types.HostedZone, error) {
-
 	if err := r.getZones(); err != nil {
 		return r53Types.HostedZone{}, err
 	}
@@ -267,7 +288,7 @@ func (r *route53Provider) getZoneRecords(zone r53Types.HostedZone) (models.Recor
 		return nil, err
 	}
 
-	var existingRecords = []*models.RecordConfig{}
+	existingRecords := []*models.RecordConfig{}
 	for _, set := range records {
 		rts, err := nativeToRecords(set, unescape(zone.Name))
 		if err != nil {
@@ -279,10 +300,10 @@ func (r *route53Provider) getZoneRecords(zone r53Types.HostedZone) (models.Recor
 }
 
 // GetZoneRecordsCorrections returns a list of corrections that will turn existing records into dc.Records.
-func (r *route53Provider) GetZoneRecordsCorrections(dc *models.DomainConfig, existingRecords models.Records) ([]*models.Correction, error) {
+func (r *route53Provider) GetZoneRecordsCorrections(dc *models.DomainConfig, existingRecords models.Records) ([]*models.Correction, int, error) {
 	zone, err := r.getZone(dc)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 
 	// update zone_id to current zone.id if not specified by the user
@@ -298,21 +319,19 @@ func (r *route53Provider) GetZoneRecordsCorrections(dc *models.DomainConfig, exi
 
 	// Amazon Route53 is a "ByRecordSet" API.
 	// At each label:rtype pair, we either delete all records or UPSERT the desired records.
-	instructions, err := diff2.ByRecordSet(existingRecords, dc, nil)
+	instructions, actualChangeCount, err := diff2.ByRecordSet(existingRecords, dc, nil)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
-	instructions = reorderInstructions(instructions)
 	var reports []*models.Correction
 
-	//wasReport := false
+	// wasReport := false
 	for _, inst := range instructions {
 		instNameFQDN := inst.Key.NameFQDN
 		instType := inst.Key.Type
 		var chg r53Types.Change
 
 		switch inst.Type {
-
 		case diff2.REPORT:
 			// REPORTs are held in a separate list so that they aren't part of the batching process.
 			reports = append(reports,
@@ -329,7 +348,7 @@ func (r *route53Provider) GetZoneRecordsCorrections(dc *models.DomainConfig, exi
 			// Make the rrset to be UPSERTed:
 			var rrset *r53Types.ResourceRecordSet
 			if instType == "R53_ALIAS" || strings.HasPrefix(instType, "R53_ALIAS_") {
-				// A R53_ALIAS_* requires ResourceRecordSet to a a single item, not a list.
+				// A R53_ALIAS_* requires ResourceRecordSet to a single item, not a list.
 				if len(inst.New) != 1 {
 					log.Fatal("Only one R53_ALIAS_ permitted on a label")
 				}
@@ -365,7 +384,6 @@ func (r *route53Provider) GetZoneRecordsCorrections(dc *models.DomainConfig, exi
 
 		default:
 			panic(fmt.Sprintf("unhandled inst.Type %s", inst.Type))
-
 		}
 
 		changes = append(changes, chg)
@@ -400,32 +418,10 @@ func (r *route53Provider) GetZoneRecordsCorrections(dc *models.DomainConfig, exi
 		addCorrection(descBatchStr, req)
 	}
 	if err := batcher.Err(); err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 
-	return append(reports, corrections...), nil
-
-}
-
-// reorderInstructions returns changes reordered to comply with AWS's requirements:
-//   - The R43_ALIAS updates must come after records they refer to.  To handle
-//     this, we simply move all R53_ALIAS instructions to the end of the list, thus
-//     guaranteeing they will happen after the records they refer to have been
-//     reated.
-func reorderInstructions(changes diff2.ChangeList) diff2.ChangeList {
-	var main, tail diff2.ChangeList
-	for _, change := range changes {
-		// Reports should be early in the list.
-		// R53_ALIAS_ records should go to the tail.
-		if change.Type != diff2.REPORT && strings.HasPrefix(change.Key.Type, "R53_ALIAS_") {
-			tail = append(tail, change)
-		} else {
-			main = append(main, change)
-		}
-	}
-	return append(main, tail...)
-	// NB(tlim): This algorithm is O(n*2) but it is simple and usually only
-	// operates on very small lists.
+	return append(reports, corrections...), actualChangeCount, nil
 }
 
 func nativeToRecords(set r53Types.ResourceRecordSet, origin string) ([]*models.RecordConfig, error) {
@@ -441,7 +437,9 @@ func nativeToRecords(set r53Types.ResourceRecordSet, origin string) ([]*models.R
 			},
 		}
 		rc.SetLabelFromFQDN(unescape(set.Name), origin)
-		rc.SetTarget(aws.ToString(set.AliasTarget.DNSName))
+		if err := rc.SetTarget(aws.ToString(set.AliasTarget.DNSName)); err != nil {
+			return nil, err
+		}
 		// rc.Original stores a pointer to the original set for use by
 		// r53Types.ChangeActionDelete and anything else that needs the
 		// native record verbatim.
@@ -648,7 +646,7 @@ func unescape(s *string) string {
 	return name
 }
 
-func (r *route53Provider) EnsureZoneExists(domain string) error {
+func (r *route53Provider) EnsureZoneExists(domain string, metadata map[string]string) error {
 	if err := r.getZones(); err != nil {
 		return err
 	}
@@ -664,7 +662,7 @@ func (r *route53Provider) EnsureZoneExists(domain string) error {
 	in := &r53.CreateHostedZoneInput{
 		Name:            &domain,
 		DelegationSetId: r.delegationSet,
-		CallerReference: aws.String(fmt.Sprint(time.Now().UnixNano())),
+		CallerReference: aws.String(strconv.FormatInt(time.Now().UnixNano(), 10)),
 	}
 
 	// reset zone cache
