@@ -2,6 +2,7 @@ package softlayer
 
 import (
 	"encoding/json"
+	"errors"
 	"fmt"
 	"regexp"
 	"strings"
@@ -24,18 +25,22 @@ type softlayerProvider struct {
 var features = providers.DocumentationNotes{
 	// The default for unlisted capabilities is 'Cannot'.
 	// See providers/capabilities.go for the entire list of capabilities.
-	providers.CanGetZones: providers.Unimplemented(),
-	providers.CanConcur:   providers.Cannot(),
-	providers.CanUseLOC:   providers.Cannot(),
-	providers.CanUseSRV:   providers.Can(),
+	providers.CanConcur:            providers.Unimplemented(),
+	providers.CanGetZones:          providers.Unimplemented(),
+	providers.CanOnlyDiff1Features: providers.Can(),
+	providers.CanUseLOC:            providers.Cannot(),
+	providers.CanUseSRV:            providers.Can(),
 }
 
 func init() {
+	const providerName = "SOFTLAYER"
+	const providerMaintainer = "NEEDS VOLUNTEER"
 	fns := providers.DspFuncs{
 		Initializer:   newReg,
 		RecordAuditor: AuditRecords,
 	}
-	providers.RegisterDomainServiceProviderType("SOFTLAYER", fns, features)
+	providers.RegisterDomainServiceProviderType(providerName, fns, features)
+	providers.RegisterMaintainer(providerName, providerMaintainer)
 }
 
 func newReg(conf map[string]string, _ json.RawMessage) (providers.DNSServiceProvider, error) {
@@ -43,7 +48,7 @@ func newReg(conf map[string]string, _ json.RawMessage) (providers.DNSServiceProv
 	s := session.New(conf["username"], conf["api_key"], conf["endpoint_url"], conf["timeout"])
 
 	if len(s.UserName) == 0 || len(s.APIKey) == 0 {
-		return nil, fmt.Errorf("SoftLayer UserName and APIKey must be provided")
+		return nil, errors.New("SoftLayer UserName and APIKey must be provided")
 	}
 
 	// s.Debug = true
@@ -78,15 +83,15 @@ func (s *softlayerProvider) GetZoneRecords(domainName string, meta map[string]st
 }
 
 // GetZoneRecordsCorrections returns a list of corrections that will turn existing records into dc.Records.
-func (s *softlayerProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, actual models.Records) ([]*models.Correction, error) {
+func (s *softlayerProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, actual models.Records) ([]*models.Correction, int, error) {
 	domain, err := s.getDomain(&dc.Name)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 
-	toReport, create, deletes, modify, err := diff.NewCompat(dc).IncrementalDiff(actual)
+	toReport, create, deletes, modify, actualChangeCount, err := diff.NewCompat(dc).IncrementalDiff(actual)
 	if err != nil {
-		return nil, err
+		return nil, 0, err
 	}
 	// Start corrections with the reports
 	corrections := diff.GenerateMessageCorrections(toReport)
@@ -114,7 +119,7 @@ func (s *softlayerProvider) GetZoneRecordsCorrections(dc *models.DomainConfig, a
 		})
 	}
 
-	return corrections, nil
+	return corrections, actualChangeCount, nil
 }
 
 func (s *softlayerProvider) getDomain(name *string) (*datatypes.Dns_Domain, error) {
@@ -124,7 +129,6 @@ func (s *softlayerProvider) getDomain(name *string) (*datatypes.Dns_Domain, erro
 		Filter(filter.Path("domains.name").Eq(name).Build()).
 		Mask("resourceRecords").
 		GetDomains()
-
 	if err != nil {
 		return nil, err
 	}
@@ -153,11 +157,13 @@ func (s *softlayerProvider) getExistingRecords(domain *datatypes.Dns_Domain) (mo
 			TTL:      uint32(*record.Ttl),
 			Original: record,
 		}
-		recConfig.SetTarget(*record.Data)
+		if err := recConfig.SetTarget(*record.Data); err != nil {
+			return nil, err
+		}
 
 		switch recType {
 		case "SRV":
-			var service, protocol = "", "_tcp"
+			service, protocol := "", "_tcp"
 
 			if record.Weight != nil {
 				recConfig.SrvWeight = uint16(*record.Weight)
@@ -177,7 +183,9 @@ func (s *softlayerProvider) getExistingRecords(domain *datatypes.Dns_Domain) (mo
 			recConfig.SetLabel(fmt.Sprintf("%s.%s", service, strings.ToLower(protocol)), *domain.Name)
 		case "TXT":
 			// OLD: recConfig.TxtStrings = append(recConfig.TxtStrings, *record.Data)
-			recConfig.SetTargetTXTs(append(recConfig.GetTargetTXTSegmented(), *record.Data))
+			if err := recConfig.SetTargetTXTs(append(recConfig.GetTargetTXTSegmented(), *record.Data)); err != nil {
+				return nil, err
+			}
 			// NB(tlim) The above code seems too complex.  Can it be simplied to this?
 			// recConfig.SetTargetTXT(*record.Data)
 			fallthrough
@@ -197,9 +205,9 @@ func (s *softlayerProvider) getExistingRecords(domain *datatypes.Dns_Domain) (mo
 }
 
 func (s *softlayerProvider) createRecordFunc(desired *models.RecordConfig, domain *datatypes.Dns_Domain) func() error {
-	var ttl, preference, domainID = verifyMinTTL(int(desired.TTL)), int(desired.MxPreference), *domain.Id
-	var weight, priority, port = int(desired.SrvWeight), int(desired.SrvPriority), int(desired.SrvPort)
-	var host, data, newType = desired.GetLabel(), desired.GetTargetField(), desired.Type
+	ttl, preference, domainID := verifyMinTTL(int(desired.TTL)), int(desired.MxPreference), *domain.Id
+	weight, priority, port := int(desired.SrvWeight), int(desired.SrvPriority), int(desired.SrvPort)
+	host, data, newType := desired.GetLabel(), desired.GetTargetField(), desired.Type
 	var err error
 
 	srvRegexp := regexp.MustCompile(`^_(?P<Service>\w+)\.\_(?P<Protocol>\w+)$`)
@@ -233,7 +241,7 @@ func (s *softlayerProvider) createRecordFunc(desired *models.RecordConfig, domai
 				return fmt.Errorf("SRV Record must match format \"_service._protocol\" not %s", host)
 			}
 
-			var serviceName, protocol = result[1], strings.ToLower(result[2])
+			serviceName, protocol := result[1], strings.ToLower(result[2])
 
 			newSrv := datatypes.Dns_Domain_ResourceRecord_SrvType{
 				Dns_Domain_ResourceRecord: newRecord,
@@ -267,11 +275,11 @@ func (s *softlayerProvider) deleteRecordFunc(resID int) func() error {
 }
 
 func (s *softlayerProvider) updateRecordFunc(existing *datatypes.Dns_Domain_ResourceRecord, desired *models.RecordConfig) func() error {
-	var ttl, preference = verifyMinTTL(int(desired.TTL)), int(desired.MxPreference)
-	var priority, weight, port = int(desired.SrvPriority), int(desired.SrvWeight), int(desired.SrvPort)
+	ttl, preference := verifyMinTTL(int(desired.TTL)), int(desired.MxPreference)
+	priority, weight, port := int(desired.SrvPriority), int(desired.SrvWeight), int(desired.SrvPort)
 
 	return func() error {
-		var changes = false
+		changes := false
 		var err error
 
 		switch desired.Type {
@@ -302,7 +310,7 @@ func (s *softlayerProvider) updateRecordFunc(existing *datatypes.Dns_Domain_Reso
 			}
 
 			if !changes {
-				return fmt.Errorf("didn't find changes when I expect some")
+				return errors.New("didn't find changes when I expect some")
 			}
 
 			_, err = service.Id(*existing.Id).EditObject(&updated)
@@ -347,7 +355,7 @@ func (s *softlayerProvider) updateRecordFunc(existing *datatypes.Dns_Domain_Reso
 			// delete and recreate?
 
 			if !changes {
-				return fmt.Errorf("didn't find changes when I expect some")
+				return errors.New("didn't find changes when I expect some")
 			}
 
 			_, err = service.Id(*existing.Id).EditObject(&updated)
@@ -374,7 +382,7 @@ func (s *softlayerProvider) updateRecordFunc(existing *datatypes.Dns_Domain_Reso
 			}
 
 			if !changes {
-				return fmt.Errorf("didn't find changes when I expect some")
+				return errors.New("didn't find changes when I expect some")
 			}
 
 			_, err = service.Id(*existing.Id).EditObject(&updated)
